package formatNo4Misp;

import java.io.InputStream;
import java.io.OutputStream;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.PrintStream;
import java.io.IOException;
import java.util.List;
import java.net.Socket;
import java.net.ConnectException;

import daruma.util.CharacerCodeConstant;

import xmlCsv.CsvReader;
import xmlCsv.TableToXmlConverter;
import xmlCsv.NewlineType;


public class CsvToMispInsert
{
	private	static final String	DEFAULT_MISP_HOST = "localhost";
	private	static final int	DEFAULT_MISP_PORT = 5050;

	// XXX: should read from option or config file
	private	static final String  FORMAT_NO4_NAMESPACE
			= "http://www.infosharp.org/schemas/dml";


	private	static final String  MISP_HEADER
			= "<misp:Transaction "
			  + "xmlns:misp=\"http://www.infosharp.org/misp\">\n"
			  + "  <misp:Insert>\n";

	private	static final String  MISP_FOOTER
			= "  </misp:Insert>\n"
			+ "</misp:Transaction>\n";

	private	static final int  MISP_PAYLOAD_INDENT = 2;



	public static void main( String  args[] )
	{
		Socket		socket = null;

		try
		{
			CsvToMispInsertOptionAnalyzer	opt;
			opt = new CsvToMispInsertOptionAnalyzer
				  ( args , "CsvToMispInsert" ,
				    DEFAULT_MISP_HOST , DEFAULT_MISP_PORT );

			if ( ! opt.analyze() )
			{
				opt.printHelp( System.err );

				System.exit( 1 );
			}
			else if ( opt.getHelp() )
			{
				opt.printHelp( System.out );

				System.exit( 0 );
			}


			OutputStream	out;

			if ( opt.getOutputToStdout() )
			{
				out = System.out;
			}
			else
			{
				//
				// socket
				//
				try
				{
					socket = new Socket( opt.getHost() ,
							     opt.getPort() );
				}
				catch( ConnectException  e )
				{
					System.err.println
					  ( "Can't connect to "
					    + "host [" + opt.getHost() + "], "
					    + "port " + opt.getPort()
					    + ": " + e.getMessage() );

					System.exit( 1 );
				}

				out = socket.getOutputStream();
			}


			//
			// read input XML file
			//
			InputStream	in = null;

			if ( opt.getFilePath() == null )
			{
				in = System.in;
			}
			else
			{
				try
				{
					in = new FileInputStream
						 ( opt.getFilePath() );
				}
				catch( FileNotFoundException  e )
				{
					System.err.println
					  ( "file [" + opt.getFilePath()
					    + "] not found" );

					System.exit( 1 );
				}
			}



			//
			// read
			//
			CsvReader  csv = new CsvReader();

			List<List<String>>	table;

			table = csv.read( in , opt.getCsvEncoding() ,
					  opt.getDebug() );


			try
			{
				TableToXmlConverter.convert
					( table ,
					  FORMAT_NO4_NAMESPACE ,
					  out ,
					  MISP_PAYLOAD_INDENT ,
					  MISP_HEADER ,
					  MISP_FOOTER );
			}
			catch( TableToXmlConverter.FormatException  e )
			{
				System.err.println( "error: "
						    + e.getMessage() );

				System.exit( 1 );
			}

			if ( socket != null )
			{
				int	ch;
				while( (ch = socket.getInputStream().read())
				       != -1 )
				{
					System.out.write( ch );
				}
			}
		}
		catch( Exception  e )
		{
			e.printStackTrace();

			System.exit( 1 );
		}
		finally
		{
			if ( socket != null )
			{
				try
				{
					socket.close();
				}
				catch( IOException  e )
				{
					System.err.println( e.getMessage() );
					System.exit( 1 );
				}
			}
		}
	}


	public static class  CsvToMispInsertOptionAnalyzer
	{
		private	String[]	args;
		private	String		programName;
		private	String		filePath;
		private	String		csvEncoding;
		/*
		private	String		csvNewlineType;
		*/
		private	String		host;
		private	int		port;
		private	boolean	helpFlag  = false;
		private	boolean	debugFlag = false;
		private	boolean	outputToStdout = false;

		public	CsvToMispInsertOptionAnalyzer
					( final String[]  args ,
					  final String  programName ,
					  final String  defaultMispHost ,
					  final int  defaultMispPort )
		{
			this.args = (String[])args.clone();
			this.programName = programName;
			this.filePath = null;
			this.csvEncoding = CharacerCodeConstant.JAVA_UTF8;
			/*
			this.csvNewlineType = NewlineType.CRLF;
			*/
			this.host = defaultMispHost;
			this.port = defaultMispPort;
		}

		public	boolean	analyze()
		{
			int	index = 0;
			while( index < args.length )
			{
				final	String	arg = this.args[index];
				int	numArgs = args.length - index - 1;

				if ( arg.equals( "--help" ) )
				{
					this.helpFlag = true;
				}
				else if ( arg.equals( "--debug" ) )
				{
					this.debugFlag = true;
				}
				else if ( arg.equals( "--csv-encoding" ) )
				{
					if ( numArgs < 1 )
					{
						return( false );
					}

					index ++;

					this.csvEncoding = this.args[index];
				}
				/*
				else if ( arg.equals( "--csv-newline-type" ) )
				{
					if ( numArgs < 1 )
					{
						return( false );
					}

					index ++;

					String	type = this.args[index];

					if ( type.equals( "LF" ) )
					{
						this.csvNewlineType
							= NewlineType.LF;
					}
					else if ( type.equals( "CR" ) )
					{
						this.csvNewlineType
							= NewlineType.CR;
					}
					else if ( type.equals( "CRLF" ) )
					{
						this.csvNewlineType
							= NewlineType.CRLF;
					}
					else
					{
						System.err.println
						  ( "invalid newline type ["
						    + type + "], "
						    + "expected was one of "
						    + "[LF|CR|CRLF]" );

						return( false );
					}
				}
				*/
				else if ( arg.equals( "--host" )
				       || arg.equals( "-h" ) )
				{
					if ( numArgs < 1 )
					{
						return( false );
					}

					index ++;

					this.host = this.args[index];
				}
				else if ( arg.equals( "--port" )
				       || arg.equals( "-p" ) )
				{
					if ( numArgs < 1 )
					{
						return( false );
					}

					index ++;

					try
					{
						this.port
						  = Integer.parseInt
						    ( this.args[index] );

					}
					catch( NumberFormatException  e )
					{
						System.err.println
						( "invalid port number: "
						  + this.args[index] );

						return( false );
					}
				}
				else if ( arg.equals( "--stdout" ) )
				{
					this.outputToStdout = true;
				}
				else if ( arg.startsWith( "-" ) )
				{
					System.err.println
						( "invalid option: " + arg );
					return( false );
				}
				else
				{
					if ( this.filePath != null )
					{
						System.err.println
						 ( "file path"
						   + " multiply specified" );

						return( false );
					}

					this.filePath = arg;
				}

				index ++;
			}

			return( true );
		}

		public	void	printHelp( OutputStream  out )
		{
			String	mes = "";

			mes += "Usage: " + this.programName
				+ " [options] [FILE]"                 + "\n";
			mes += "Possible options are:"                + "\n";
			mes += "      --help"                         + "\n";
			mes += "      --debug"                        + "\n";
			mes += "   -h,--host HOST"                    + "\n";
			mes += "   -p,--port PORT"                    + "\n";
			mes += "      --csv-encoding"
			    + " [UTF-8|EUC-JP|ISO-2022-JP|Shift_JIS]" + "\n";
			mes += "      --stdout"                       + "\n";
			/*
			mes += "      --csv-newline-type [LF|CR|CRLF]" + "\n";
			*/

			PrintStream	p = new PrintStream( out );
			p.print( mes );
		}

		public	boolean	getHelp()
		{
			return( this.helpFlag );
		}

		public	boolean	getDebug()
		{
			return( this.debugFlag );
		}

		public	String	getFilePath()
		{
			return( this.filePath );
		}

		public	String	getCsvEncoding()
		{
			return( this.csvEncoding );
		}

		/*
		public	String	getCsvNewlineType()
		{
			return( this.csvNewlineType );
		}
		*/

		public	String	getHost()
		{
			return( this.host );
		}

		public	int	getPort()
		{
			return( this.port );
		}
		public	boolean	getOutputToStdout()
		{
			return( this.outputToStdout );
		}
	}
}
