package formatNo4Misp;

import java.io.InputStream;
import java.io.OutputStream;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.PrintStream;
import java.io.IOException;
import java.util.List;
import java.net.Socket;
import java.net.ConnectException;


import org.w3c.dom.Element;
import javax.xml.transform.TransformerException;
import daruma.xml.UniversalName;
import daruma.xml.QName;
import daruma.xml.util.ElementUtil;
import daruma.xml.util.NodeUtil;
import daruma.xml.util.XMLFormatConverter;

import daruma.util.FatalException;
import daruma.util.CharacerCodeConstant;

import xmlCsv.XmlToCsvConverter;
import xmlCsv.NewlineType;



public class MispGetFeatureResponceToCsv
{
	private	static final UniversalName  FEATURE_NAME_CONFIG
		= new UniversalName
		( "http://www.kedm.bosai.go.jp/tools/XmlCsv/config" ,
		  "feature" );

	private	static final String	DEFAULT_MISP_HOST = "localhost";
	private	static final int	DEFAULT_MISP_PORT = 5050;


	public static void main( String  args[] )
	{
		Socket		socket = null;

		try
		{
			MispGetFeatureResponceToCsvOptionAnalyzer	opt;
			opt = new MispGetFeatureResponceToCsvOptionAnalyzer
				( args , "MispGetFeatureResponceToCsv" ,
				  DEFAULT_MISP_HOST , DEFAULT_MISP_PORT );

			if ( ! opt.analyze() )
			{
				opt.printHelp( System.err );

				System.exit( 1 );
			}
			else if ( opt.getHelp() )
			{
				opt.printHelp( System.out );

				System.exit( 0 );
			}


			//
			// read config file
			//
			InputStream	configInput = null;

			if ( opt.getConfigFilePath() == null )
			{
				System.err.println
					( "config file not specified" );

				System.exit( 1 );
			}
			else
			{
				try
				{
					configInput
					  = new FileInputStream
						( opt.getConfigFilePath() );
				}
				catch( FileNotFoundException  e )
				{
					System.err.println
					  ( "file "
					    + "["
					    + opt.getConfigFilePath()
					    + "] not found" );

					System.exit( 1 );
				}
			}

			UniversalName  featureName
					= getFeatureNameFromConfig
							  ( configInput );


			//
			// read input XML file
			//
			InputStream	in = null;

			if ( opt.getFilePath() == null )
			{
				in = System.in;
			}
			else
			{
				try
				{
					in = new FileInputStream
						( opt.getFilePath() );
				}
				catch( FileNotFoundException  e )
				{
					System.err.println
					  ( "file "
					    + "[" + opt.getFilePath()
					    + "] not found" );

					System.exit( 1 );
				}
			}


			InputStream	xmlInput;

			if ( opt.getReadFromSocket() )
			{
				try
				{
					socket = new Socket( opt.getHost() ,
							     opt.getPort() );
				}
				catch( ConnectException  e )
				{
					System.err.println
					  ( "Can't connect to "
					    + "host [" + opt.getHost() + "], "
					    + "port " + opt.getPort()
					    + ": " + e.getMessage() );

					System.exit( 1 );
				}

				xmlInput = getSocketResponse( socket , in );
			}
			else
			{
				xmlInput = in;
			}


			//
			// convert
			//
			XmlToCsvConverter	conv;
			conv = new XmlToCsvConverter();

			conv.convert( xmlInput ,
				      System.out ,
				      featureName ,
				      opt.getCsvEncoding() ,
				      opt.getCsvNewline() ,
				      opt.getDebug() );

		}
		catch( Exception  e )
		{
			e.printStackTrace();
			System.exit( 1 );
		}
		finally
		{
			if ( socket != null )
			{
				try
				{
					socket.close();
				}
				catch( IOException  e )
				{
					System.err.println( e.getMessage() );
					System.exit( 1 );
				}
			}
		}
	}


	private	static UniversalName	getFeatureNameFromConfig
						( InputStream  configInput )
		throws FatalException
	{
		Element	config;

		try
		{
			config = XMLFormatConverter.streamToDOMDocument
				( configInput ).getDocumentElement();
		}
		catch( TransformerException  e )
		{
			throw new FatalException( e.getMessage() , e );
		}


		List<Element>	featureNameList
			= ElementUtil.getChildElements
			( config , FEATURE_NAME_CONFIG );

		if ( featureNameList.size() == 0 )
		{
			System.err.println( "invalid config, "
					    + FEATURE_NAME_CONFIG
					    + " not found" );

			System.exit( 1 );
		}
		else if ( featureNameList.size() >= 2 )
		{
			System.err.println( "invalid config, "
					    + FEATURE_NAME_CONFIG
					    + "multiply defined" );

			System.exit( 1 );
		}

		Element	featureElement = featureNameList.get(0);


		QName	featureQName
			= new QName( ElementUtil.getChildNodesWholeText
				     ( featureElement ) );

		if ( ! featureQName.isValid() )
		{
			System.err.println( "invalid QName ["
					    + featureQName + "]" );
			System.exit( 1 );
		}

		String	featureNamespace = NodeUtil.lookupNamespaceURI
					   ( featureElement ,
					     featureQName.getPrefix() );

		return( new UniversalName( featureNamespace ,
					   featureQName.getLocalName() ) );
	}


	private	static InputStream	getSocketResponse( Socket  socket ,
							   InputStream  in )
							throws IOException
	{
		int	ch;
		while( (ch = in.read()) != -1 )
		{
			socket.getOutputStream().write( ch );
		}

//		socket.shutdownOutput();

		return( socket.getInputStream() );
	}



	public static class  MispGetFeatureResponceToCsvOptionAnalyzer
	{
		private	String[]	args;
		private	String		programName;
		private	String		filePath;
		private	String		configFilePath;
		private	String		csvEncoding;
		private	String		csvNewline;
		private	String		host;
		private	int		port;

		private	boolean	helpFlag  = false;
		private	boolean	debugFlag = false;
		private	boolean	readFromSocket = false;

		public	MispGetFeatureResponceToCsvOptionAnalyzer
			( final String[]  args ,
			  final String  programName ,
			  final String  defaultMispHost ,
			  final int  defaultMispPort )
		{
			this.args = (String[])args.clone();
			this.programName = programName;
			this.filePath = null;
			this.configFilePath = null;
			this.csvEncoding = CharacerCodeConstant.JAVA_UTF8;
			this.csvNewline = NewlineType.CRLF;

			this.host = defaultMispHost;
			this.port = defaultMispPort;
			/*
			this.readFromSocket = false;
			*/
			this.readFromSocket = true;
		}

		public	boolean	analyze()
		{
			int	index = 0;
			while( index < args.length )
			{
				final	String	arg = this.args[index];
				int	numArgs = args.length - index - 1;

				if ( arg.equals( "--help" ) )
				{
					this.helpFlag = true;
				}
				else if ( arg.equals( "--debug" ) )
				{
					this.debugFlag = true;
				}
				else if ( arg.equals( "--config" ) )
				{
					if ( numArgs < 1 )
					{
						return( false );
					}

					index ++;

					this.configFilePath = this.args[index];
				}
				else if ( arg.equals( "--csv-encoding" ) )
				{
					if ( numArgs < 1 )
					{
						return( false );
					}

					index ++;

					this.csvEncoding = this.args[index];
				}
				else if ( arg.equals( "--csv-newline-type" ) )
				{
					if ( numArgs < 1 )
					{
						return( false );
					}

					index ++;

					String	type = this.args[index];

					if ( type.equals( "LF" ) )
					{
						this.csvNewline
							= NewlineType.LF;
					}
					else if ( type.equals( "CR" ) )
					{
						this.csvNewline
							= NewlineType.CR;
					}
					else if ( type.equals( "CRLF" ) )
					{
						this.csvNewline
							= NewlineType.CRLF;
					}
					else
					{
						System.err.println
						  ( "invalid newline type ["
						    + type + "], "
						    + "expected was one of "
						    + "[LF|CR|CRLF]" );

						return( false );
					}
				}
				/*
				else if ( arg.equals( "--read-from-socket" ) )
				{
					this.readFromSocket = true;
				}
				*/
				else if ( arg.equals( "--host" )
				       || arg.equals( "-h" ) )
				{
					if ( numArgs < 1 )
					{
						return( false );
					}

					index ++;

					this.host = this.args[index];
				}
				else if ( arg.equals( "--port" )
				       || arg.equals( "-p" ) )
				{
					if ( numArgs < 1 )
					{
						return( false );
					}

					index ++;

					try
					{
						this.port
						  = Integer.parseInt
						    ( this.args[index] );

					}
					catch( NumberFormatException  e )
					{
						System.err.println
						( "invalid port number: "
						  + this.args[index] );

						return( false );
					}
				}
				else if ( arg.startsWith( "-" ) )
				{
					System.err.println
						( "invalid option: " + arg );
					return( false );
				}
				else
				{
					if ( this.filePath != null )
					{
						System.err.println
						 ( "file path"
						   + " multiply specified" );

						return( false );
					}

					this.filePath = arg;
				}

				index ++;
			}

			return( true );
		}

		public	void	printHelp( OutputStream  out )
		{
			String	mes = "";

			mes += "Usage: " + this.programName
				+ " --config CONFIG_FILE [options] [FILE]"
				+ "\n";
			mes += "Possible options are:"                 + "\n";
			mes += "      --help"                          + "\n";
			mes += "      --debug"                         + "\n";
			mes += "   -h,--host HOST"                     + "\n";
			mes += "   -p,--port PORT"                     + "\n";
			/*
			mes += "      --read-from-socket"              + "\n";
			*/
			mes += "      --config CONFIG_FILE"            + "\n";
			mes += "      --csv-encoding"
			    + " [UTF-8|EUC-JP|ISO-2022-JP|Shift_JIS]"  + "\n";
			mes += "      --csv-newline-type [LF|CR|CRLF]" + "\n";


			PrintStream	p = new PrintStream( out );
			p.print( mes );
		}

		public	boolean	getHelp()
		{
			return( this.helpFlag );
		}

		public	boolean	getDebug()
		{
			return( this.debugFlag );
		}

		public	String	getFilePath()
		{
			return( this.filePath );
		}

		public	String	getConfigFilePath()
		{
			return( this.configFilePath );
		}

		public	String	getCsvEncoding()
		{
			return( this.csvEncoding );
		}

		public	String	getCsvNewline()
		{
			return( this.csvNewline );
		}

		public	boolean	getReadFromSocket()
		{
			return( this.readFromSocket );
		}

		public	String	getHost()
		{
			return( this.host );
		}

		public	int	getPort()
		{
			return( this.port );
		}
	}
}
