package xmlCsv;

import java.util.List;
import java.util.ArrayList;
import java.util.Stack;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.io.IOException;

import daruma.xml.SimpleXPath;
import daruma.xml.UniversalName;
import daruma.util.CharacerCodeConstant;


public class TableToXmlConverter
{
	private	static final String	TAG_LINE_IDENTIFIER = "#";
	private	static int		XML_INDENT_LEVEL = 2;

	public static class  FormatException extends Exception
	{
		private	static final long serialVersionUID = 0L;

		public	FormatException( String  message )
		{
			super( message );
		}

		public	FormatException( String  message ,
					 Throwable  throwable )
		{
			super( message , throwable );
		}
	}

	private	TableToXmlConverter()
	{
	}

	public	static	void	convert( List< List<String> >  table ,
					 String  namespaceURI ,
					 OutputStream  out ,
					 int  indentOffset ,
					 String  header ,
					 String  footer )
		throws FormatException , IOException
	{
		if ( table.isEmpty() )
		{
			throw new FormatException( "empty table" );
		}

		if ( table.get(0).isEmpty() )
		{
			throw new FormatException
				( "tag line not found" );
		}

		if ( ! table.get(0).get(0).startsWith( TAG_LINE_IDENTIFIER ) )
		{
			throw new FormatException
				( "first line must be a tag line, "
				  + "fiest line must start with '"
				  + TAG_LINE_IDENTIFIER + "'" );
		}

		List<SimpleXPath>	tags = getTagXPath( table.get(0) );

		PrintWriter	w;
		w = new PrintWriter( new OutputStreamWriter
				     ( out ,
				       CharacerCodeConstant.JAVA_UTF8 ) );

		w.print( header );

		for ( int  i = 1  ;  i < table.size()  ;  ++ i )
		{
			List<String>	fields = table.get(i);

			String	xmlString = convertToXML( tags ,
							  namespaceURI ,
							  fields ,
							  indentOffset );
			w.print( xmlString );
		}

		w.print( footer );

		w.flush();
	}

	private	static List<SimpleXPath>  getTagXPath( List<String>  tag )
	{
		List<SimpleXPath>	ret = new ArrayList<SimpleXPath>();

		for ( int  i = 0  ;  i < tag.size()  ;  ++ i )
		{
			String	tagValue;

			if ( i == 0
			  && tag.get(0).length()
			     >= TAG_LINE_IDENTIFIER.length() )
			{
				tagValue = tag.get(0).substring
					   ( TAG_LINE_IDENTIFIER.length() );
			}
			else
			{
				tagValue = tag.get(i);
			}


			SimpleXPath	path = new SimpleXPath();

			for ( String  p : tagValue.split( "/" ) )
			{
				path.add( new UniversalName( null , p ) );
			}

			ret.add( path );
		}

		return( ret );
	}

	private	static	String	convertToXML( List<SimpleXPath>  paths ,
					      String  namespaceURI ,
					      List<String>  fields ,
					      int  indentOffset )
		throws FormatException
	{
		if ( paths.size() != fields.size() )
		{
			throw new FormatException
				( "number of fields unmatched, "
				  + "number of tags ware " + paths.size()
				  + ", number of fields ware "
				  + fields.size() );
		}

		Stack<UniversalName>	tagStack;
		tagStack = new Stack<UniversalName>();

		StringBuilder	buf = new StringBuilder();

		List<SimpleXPath>	extendedPaths
					= new ArrayList<SimpleXPath>( paths );
		extendedPaths.add( new SimpleXPath() );

		for ( int  i = 0  ;  i < extendedPaths.size()  ;  ++ i )
		{
			SimpleXPath	p = extendedPaths.get(i);
			SimpleXPath	s = new SimpleXPath
					    ( new ArrayList<UniversalName>
					      ( tagStack ) );

			int	c = s.getNumberOfCommonPrefixElement( p );

			int	pSize = p.getList().size();
			int	sSize = s.getList().size();

			if ( sSize - c != 0 )
			{
				for ( int  n = 0  ;  n < sSize - c  ;  ++ n )
				{
					UniversalName	name = tagStack.pop();

					if ( n != 0 )
					{
						appendIndentToStringBuilder
						    ( buf ,
						      tagStack.size()
						      + indentOffset );
					}

					buf.append( "</" );
					buf.append( name.getLocalName() );
					buf.append( ">\n" );
				}
			}

			if ( pSize - c != 0 )
			{
				for ( int  n = c  ;  n < pSize  ;  ++ n )
				{
					appendIndentToStringBuilder
					  ( buf ,
					    tagStack.size() + indentOffset );

					buf.append( "<" );
					buf.append( p.getList().get(n)
						    .getLocalName() );

					if ( tagStack.size() == 0 )
					{
						buf.append( " xmlns=\"" );
						buf.append( namespaceURI );
						buf.append( "\"" );
					}

					buf.append( ">" );

					if ( n != pSize - 1 )
					{
						buf.append( "\n" );
					}

					tagStack.push( p.getList().get(n) );
				}
			}

			if ( i < fields.size() )
			{
				buf.append( fields.get(i) );
			}
		}

		return( buf.toString() );
	}

	private	static void  appendIndentToStringBuilder( StringBuilder  buf ,
							  int  n )
	{
		for ( int  nSpace = 0  ;  nSpace < n  * XML_INDENT_LEVEL  ;
		      ++ nSpace )
		{
			buf.append( ' ' );
		}
	}
}
