package xmlCsv;

import daruma.xml.SimpleXPath;
import daruma.xml.UniversalName;
import daruma.xml.util.ElementUtil;
import daruma.xml.util.XMLFormatConverter;
import daruma.util.FatalException;
import daruma.util.Pair;
import daruma.util.CloseIgnoreInputStream;

import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Element;
import org.w3c.dom.Document;

import java.io.InputStream;
import java.io.OutputStream;
import java.io.Writer;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.io.OutputStreamWriter;
import java.io.PrintStream;
import java.io.PrintWriter;
import java.util.List;
import java.util.ArrayList;

import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathFactory;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathConstants;
import javax.xml.transform.TransformerException;

import xmlCsv.CsvColumnWriter;



public class XmlToCsvConverter
{
	private	static final String	TAG_LINE_INDICATOR = "#";

	public	XmlToCsvConverter()
	{
	}

	public	void	convert( InputStream  in  ,
				 OutputStream  out ,
				 UniversalName  featureName ,
				 String  outputEncoding ,
				 String  newline ,
				 boolean  debug )
		throws FatalException, IOException
	{
		//
		// read data
		//
		Document	doc;

		try
		{
			doc = XMLFormatConverter.streamToDOMDocument
				( new CloseIgnoreInputStream( in ) );
		}
		catch( TransformerException  e )
		{
			throw new FatalException( e.getMessage() , e );
		}


		//
		// choose nodes
		//
		XPath	xPath = XPathFactory.newInstance().newXPath();

		NodeList	targetNodes;

		if ( debug )
		{
			System.err.println( "featureName = ["
					    + featureName + "]" );
		}


		try
		{
			targetNodes
			  = (NodeList)xPath.evaluate
			      ( "//*[local-name()='"
				+ featureName.getLocalName() + "'"
				+ " and namespace-uri()='"
				+ featureName.getNamespace() + "']" ,
				doc.getDocumentElement() ,
				XPathConstants.NODESET );
		}
		catch( XPathExpressionException  e )
		{
			throw new FatalException( e.getMessage() , e );
		}


		Writer	ew;
		try
		{
			ew = new OutputStreamWriter( out , outputEncoding );
		}
		catch ( UnsupportedEncodingException  e )
		{
			throw new FatalException( e.getMessage() );
		}

		PrintWriter	writer = new PrintWriter( ew );


		for ( int  i = 0  ;  i < targetNodes.getLength()  ;  i ++ )
		{
			// XXX: check tag lines are same or not
			this.printElement( (Element)targetNodes.item(i) ,
					   (i == 0) ,
					   writer ,
					   newline ,
					   debug );
		}

		writer.flush();
		out.flush();
	}

	private	void	printElement( Element  element ,
				      boolean  printTagLine ,
				      PrintWriter  writer ,
				      String  newline ,
				      boolean  debug ) throws IOException
	{
		//
		// create list
		//
		List< Pair<SimpleXPath, String> >
		  pathValueList = new ArrayList< Pair<SimpleXPath, String> >();

		this.createXPathAndValueList( pathValueList ,
					      new SimpleXPath() ,
					      element );

		if ( printTagLine )
		{
			//
			// print tag line
			//
			writer.print( TAG_LINE_INDICATOR );
			for( int  i = 0  ;  i < pathValueList.size()  ;  ++ i )
			{
				SimpleXPath	path = pathValueList
							.get(i).getFirst();

				if ( i != 0 )
				{
					writer.print( "," );
				}

				CsvColumnWriter.write
				( writer ,
				  path.getStringExpressionWithoutNamespace() );
			}
			writer.print( newline );
		}


		//
		// print value
		//
		for( int  i = 0  ;  i < pathValueList.size()  ;  ++ i )
		{
			String	value = pathValueList.get(i).getSecond();

			if ( i != 0 )
			{
				writer.print( "," );
			}

			CsvColumnWriter.write( writer , value );
		}
		writer.print( newline );
		writer.flush();
	}

	private	void	createXPathAndValueList
				( List< Pair<SimpleXPath, String> >  result ,
				  SimpleXPath  prefixPath ,
				  Element  element )
	{
		SimpleXPath	path = new SimpleXPath( prefixPath , element );

		List<Element>	chileElements = ElementUtil.getChildElements
								   ( element );

		if ( chileElements.size() == 0 )
		{
			String	text = ElementUtil
					.getChildNodesWholeText( element );

			result.add( new Pair<SimpleXPath, String>
				    ( path , text ) );
		}
		else
		{
			for( Element  c : chileElements )
			{
				this.createXPathAndValueList
					( result , path , c );
			}
		}
	}
}
