package xmlCsv.test;

import org.junit.Test;
import junit.framework.TestCase;

import xmlCsv.CsvReader;
import xmlCsv.CsvFormatException;

import daruma.util.CharacerCodeConstant;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.io.InputStream;
import java.io.ByteArrayInputStream;

import java.util.List;


public class CsvReaderTest extends TestCase
{
	private	static	final	String	NEWLINE = "\n";
//	private	static	final	String	NEWLINE = "\r\n";

	@Test
	public	void	testRead() throws Exception
	{
		List< List<String> >	result;

		String		data =    "aaa,bbb,ccc" + NEWLINE
					+ "ddd,eee,fff" + NEWLINE;

		result = this.getResult( data.getBytes() );

		assertEquals( 2 , result.size() );
		assertEquals( 3 , result.get(0).size() );
		assertEquals( 3 , result.get(1).size() );
		assertEquals( "aaa" , result.get(0).get(0) );
		assertEquals( "bbb" , result.get(0).get(1) );
		assertEquals( "ccc" , result.get(0).get(2) );
		assertEquals( "ddd" , result.get(1).get(0) );
		assertEquals( "eee" , result.get(1).get(1) );
		assertEquals( "fff" , result.get(1).get(2) );
	}


	@Test
	public	void	testDoubleQuoteWithOutQuoting() throws Exception
	{
		List< List<String> >	result;

		String		data = "aaa,b\"\"b,ccc" + NEWLINE;

		try
		{
			this.getResult( data.getBytes() );
		}
		catch( CsvFormatException  e )
		{
			assert( true );
		}
	}

	@Test
	public	void	testQuote() throws Exception
	{
		List< List<String> >	result;

		String		data = "aaa,\"b\"\"bb\",ccc" + NEWLINE;

		result = this.getResult( data.getBytes() );

		assertEquals( 1 , result.size() );
		assertEquals( 3 , result.get(0).size() );
		assertEquals( "aaa"   , result.get(0).get(0) );
		assertEquals( "b\"bb" , result.get(0).get(1) );
		assertEquals( "ccc"   , result.get(0).get(2) );
	}

	@Test
	public	void	testComma() throws Exception
	{
		List< List<String> >	result;

		String		data = "aaa,\"b,bb\",ccc" + NEWLINE;

		result = this.getResult( data.getBytes() );

		assertEquals( 1 , result.size() );
		assertEquals( 3 , result.get(0).size() );
		assertEquals( "aaa"  , result.get(0).get(0) );
		assertEquals( "b,bb" , result.get(0).get(1) );
		assertEquals( "ccc"  , result.get(0).get(2) );
	}

	@Test
	public	void	testNewline() throws Exception
	{
		List< List<String> >	result;

		String	data = "\"a\naa\",\"b\rbb\",\"c\r\ncc\",\"d\n\rdd\""
				+ NEWLINE;

		result = this.getResult( data.getBytes() );

		assertEquals( 1 , result.size() );
		assertEquals( 4 , result.get(0).size() );
		assertEquals( "a\naa"   , result.get(0).get(0) );
		assertEquals( "b\rbb"   , result.get(0).get(1) );
		assertEquals( "c\r\ncc" , result.get(0).get(2) );
		assertEquals( "d\n\rdd" , result.get(0).get(3) );
	}

	@Test
	public	void	testNullString() throws Exception
	{
		List< List<String> >	result;

		String		data = "";

		result = this.getResult( data.getBytes() );

		assertEquals( 0 , result.size() );
	}

	@Test
	public	void	testOneNewlineOnly() throws Exception
	{
		List< List<String> >	result;

		String		data = NEWLINE;

		result = this.getResult( data.getBytes() );

		assertEquals( 1 , result.size() );
	}

	@Test
	public	void	testNewlinesOnly() throws Exception
	{
		List< List<String> >	result;

		String		data = NEWLINE + NEWLINE;

		result = this.getResult( data.getBytes() );

		assertEquals( 2 , result.size() );
	}


	private	List< List<String> >	getResult( byte[]  dataBytes )
		throws CsvFormatException ,
		       UnsupportedEncodingException ,
		       IOException
	{
		List< List<String> >	result;

		CsvReader	reader = new CsvReader();

		InputStream  in = new ByteArrayInputStream( dataBytes );

		result = reader.read( in , CharacerCodeConstant.JAVA_UTF8 );

		return( result );
	}
}
