package daruma.server;

import java.io.*;
import java.net.Socket;
import java.util.zip.GZIPInputStream;
import java.util.zip.GZIPOutputStream;

import daruma.strage_manager.DBMSStrageManager;
import daruma.sql.DatabaseConnectionException;
import daruma.sql.MySQLDatabaseConnectionFactory;
import daruma.wfs.SOAPWFSProcessor;
import daruma.util.TeeInputStream;
import daruma.util.TeeOutputStream;


public class ConnectionHandler implements Runnable
{
	private	Socket			socket;
	private	DBMSStrageManager	strage;
	private	boolean			debug = false;
	private	boolean			compressStreamByGzip = false;

	public	ConnectionHandler( Socket  s ,
				   String  databaseName ,
				   String  user ,
				   String  passwd ,
				   boolean  debug )
					throws DatabaseConnectionException
	{
		this.socket = s;
		this.debug = debug;
		this.strage = new DBMSStrageManager
				  ( (new MySQLDatabaseConnectionFactory())
								  .create() );

		try
		{
			this.strage.connect( databaseName , user , passwd );
		}
		catch( DatabaseConnectionException e )
		{
			throw e;
		}
	}

	public	void	setCompressStreamByGzip( boolean  flag )
	{
		this.compressStreamByGzip = flag;
	}

	public	void	run()
	{
		try
		{
			//
			// prepare I/O
			//
			InputStream	in  = this.socket.getInputStream();
			OutputStream	out = this.socket.getOutputStream();

			if ( this.compressStreamByGzip )
			{
				in  = new GZIPInputStream ( in );
				out = new GZIPOutputStream( out );
			}

			if ( this.debug )
			{
				in  = new TeeInputStream
					  ( in  , System.out , false );

				out = new TeeOutputStream
					  ( out , System.out , false );
			}


			//
			// process document
			//
			SOAPWFSProcessor	p = new SOAPWFSProcessor();

			p.setDebugTag( this.socket.getInetAddress()
				       + ":" + this.socket.getPort() );

			p.process( in , out , strage );


			//
			// close database connection
			//
			try
			{
				this.strage.close();
			}
			catch( DatabaseConnectionException  e )
			{
				throw e;
			}


			//
			// close I/O connection
			//
			out.flush();
			out.close();
			in.close();

			this.socket.close();
		}
		catch( IOException  e )
		{
			e.printStackTrace();
		}
		catch( Exception  e )
		{
			e.printStackTrace();
		}
		finally
		{
			try
			{
				this.socket.close();
			}
			catch( IOException  e )
			{
				e.printStackTrace();
			}
		}
	}
}
