package daruma.sql;

import daruma.sql.TableColumn;

import daruma.sql.DatabaseConnection;
import java.sql.PreparedStatement;
import java.sql.SQLException;

import daruma.util.ISO8601DateFormat;

import daruma.strage_manager.type_definition.ElementColumnNameFactory;

import daruma.geometry.Geometry;

import java.util.Date;
import java.lang.StringBuilder;
import java.util.List;
import java.text.ParseException;



public class TableRow
{
	private	List<TableColumn>	columns;

	public	TableRow( List<TableColumn>  columns )
	{
		this.columns = columns;
	}

	public	void	executeInsert( DatabaseConnection  db ,
				       String  tableName )
					  throws DatabaseConnectionException
	{
		long	id = db.getMaxLongValueFromTable
				( tableName , "_id_" );
		id ++;

		StringBuilder	s = new StringBuilder();

		s.append( "INSERT INTO `" + tableName + "` (" );

		boolean	firstColumn = true;
		for ( TableColumn  c  :  this.columns )
		{
			if ( ! firstColumn )
			{
				s.append( "," );
			}

			firstColumn = false;

			s.append( "`" );
			s.append( c.getDefinition().getColumnName() );
			s.append( "`" );
		}

		s.append( ") VALUES (" );


		int	wildcardCount = 0;
		firstColumn = true;
		for ( TableColumn  c  :  this.columns )
		{
			if ( ! firstColumn )
			{
				s.append( "," );
			}

			firstColumn = false;

			Class	javaClass = c.getDefinition()
					      .getSQLDataType()
					      .getJavaClass();

			if ( Geometry.class.isAssignableFrom( javaClass ) )
			{
				s.append( "GeomFromText('" );
				s.append( c.getValue() );
				s.append( "')" );
			}
			else if ( Date.class.isAssignableFrom( javaClass ) )
			{
				ISO8601DateFormat
					f = new ISO8601DateFormat();

				Date	dateTime;

				try
				{
					dateTime = f.parse( c.getValue() );

					// XXX: should not drop
					//      nanoseconds information
				}
				catch( ParseException  e )
				{
					throw new DatabaseConnectionException
									( e ) ;
				}

				s.append( "\"" );
				s.append( SQLTimeFormatConverter
					    .convertDateToString( dateTime ) );
				s.append( "\"" );
			}
			else
			{
				wildcardCount ++;
				s.append( "?" );
			}
		}

		s.append( ")" );

		System.err.println( "#" + s.toString() );


		if ( wildcardCount == 0 )
		{
			db.executeUpdate( s.toString() );

			return;
		}


		PreparedStatement	st = null;

		try
		{
			st = db.prepareStatement( s.toString() );
		}
		catch( DatabaseConnectionException  e  )
		{
			throw e;
		}

		try
		{
			int	wildcard_index = 1;

			for ( int  i = 0  ;  i < this.columns.size()  ;  ++ i )
			{
				TableColumn	col = this.columns.get(i);

				Class	c = col.getDefinition()
					     .getSQLDataType().getJavaClass();

				if ( c.equals( String.class ) )
				{
					st.setString( wildcard_index ,
						      col.getValue() );

					wildcard_index ++;
				}
				else if ( c.equals( Boolean.class ) )
				{
					boolean	value = Boolean
							.parseBoolean
							  ( col.getValue() );

					st.setBoolean( wildcard_index ,
						       value );

					wildcard_index ++;
				}
				else if ( c.equals( Float.class ) )
				{
					float	value = Float
							.parseFloat
							  ( col.getValue() );

					st.setFloat( wildcard_index ,
						     value );

					wildcard_index ++;
				}
				else if ( c.equals( Double.class ) )
				{
					double	value = Double
							.parseDouble
							  ( col.getValue() );

					st.setDouble( wildcard_index ,
						      value );

					wildcard_index ++;
				}
				else if ( Geometry.class
					  .isAssignableFrom( c ) )
				{
				}
				else if ( Date.class.isAssignableFrom( c ) )
				{
				}
				else // XXX
				{
					st.setString( wildcard_index ,
						      col.getValue() );

					wildcard_index ++;
				}
			}

			st.executeUpdate();
		}
		catch( SQLException  e )
		{
			throw new DatabaseConnectionException( e.getMessage() ,
							       e );
		}

		executeInsertToSubTable( db , id );
	}

	private	void	executeInsertToSubTable( DatabaseConnection  db ,
						 long  parentID )
					  throws DatabaseConnectionException
	{
		for ( TableColumn  c  :  this.columns )
		{
			if ( c.getDefinition().isExternalTable() )
			{
				String	tableName = c.getValue();

				System.err.println( "tableName = ["
						    + tableName + "]" );

				long	id = db.getMaxLongValueFromTable
						( tableName , "_id_" );
				id ++;

				for ( String  v : c.getSubValues() )
				{
					System.err.println( "value = ["
							    + v + "]" );

					String	sql
					  = "INSERT INTO `"
					    + c.getValue()
					    + "` (`_id_`,`_parent_id_`, `"
					    + ElementColumnNameFactory
						      .getTopLevelColumnName()
					    + "`) VALUES ("
					    + id + ","
					    + parentID
					    + ",\"" + v + "\")";

					System.err.println( "#" + sql );

					db.executeUpdate( sql );
				}
			}
		}
	}
}
