package daruma.strage_manager;

import daruma.strage_manager.StrageException;
import daruma.strage_manager.schema.XMLSchemaParseAndRegister;
import daruma.strage_manager.schema.XMLSchemaParserException;
import daruma.strage_manager.type_definition.ElementName;
import daruma.strage_manager.type_definition.TypeName;
import daruma.strage_manager.type_definition.TypeDefinition;
import daruma.strage_manager.type_definition.TypedInstance;
import daruma.strage_manager.type_definition.TypedInstanceSet;
import daruma.strage_manager.type_definition.TypeException;
import daruma.strage_manager.type_definition.ElementColumnNameFactory;

import daruma.wfs.filter.PredicateDescription;

import daruma.xml.UniversalName;
import daruma.xml.SimpleXPath;
import daruma.xml.util.PrefixMap;
import daruma.xml.util.XMLFormatConverter;
import javax.xml.transform.TransformerException;
import org.w3c.dom.Element;

import daruma.sql.DatabaseConnection;
import daruma.sql.DatabaseConnectionException;
import daruma.sql.SQLDataType;
import daruma.sql.TableColumnDefinition;
import daruma.sql.TableDefinition;
import daruma.sql.TableColumn;
import daruma.sql.TableRow;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.PreparedStatement;

import daruma.util.Pair;
import daruma.util.StringSplitter;

import java.util.Date;

import java.util.List;
import java.util.ArrayList;
import java.util.Map;
import java.util.HashMap;
import java.io.ByteArrayOutputStream;


public class DBMSStrageManager extends StrageManager
{
	static final String	SCHEMA_TABLE_NAME  = "__schema__";
	static final String	ELEMENT_TABLE_NAME = "__element__";
	static final String	TYPE_TABLE_NAME    = "__type__";
	static final String	UNIVERSAL_NAME_ALIAS_TABLE_NAME
					    = "__universal_name_alias__";

	private	DatabaseConnection	db;

	private	static
		TypeDictionary	typeDictionaryCache = new TypeDictionary();

	private	Map<ElementName,TypeName>	elementDictionaryCache;
	private	Map<UniversalName, String>	universalNameDictionaryCache;

	public	DBMSStrageManager( DatabaseConnection  db )
	{
		assert db != null;

		this.db = db;

		this.elementDictionaryCache
				    = new HashMap<ElementName, TypeName>();

		this.universalNameDictionaryCache
				    = new HashMap<UniversalName, String>();
	}


	//
	// DBMSStrageManager extentions
	//
	public	void	connect( String  databaseName ,
				 String  user ,  String  passwd )
					  throws DatabaseConnectionException
	{
		this.db.connect( databaseName , user , passwd );
	}

	public	void	close() throws DatabaseConnectionException
	{
		this.db.close();
	}


	public void	registerSchema( Element  schema ,
					PrefixMap  prefixMap )
						throws StrageException
	{
		// XXX: this.db.startTransaction()

		XMLSchemaParseAndRegister
			parser = new XMLSchemaParseAndRegister();
		try
		{
			parser.parseXMLSchemaDOMElement
					( schema , prefixMap , this , false );
		}
		catch( XMLSchemaParserException  e )
		{
			throw new StrageException( e.getMessage() );
		}



		//
		// convert schema to string
		//
		ByteArrayOutputStream	buf = new ByteArrayOutputStream();

		try
		{
			XMLFormatConverter.print( schema , buf );
		}
		catch( TransformerException  e )
		{
			throw new StrageException
				  ( "Can't create schema string" , e );
		}

		try
		{
			PreparedStatement  deleteStatement
					   = this.db.prepareStatement
					     ( "UPDATE "
					       + this.SCHEMA_TABLE_NAME
					       + " SET deleted=TRUE"
					       + " WHERE schema_id=?" );

			PreparedStatement  insertStatement
					   = this.db.prepareStatement
					     ( "INSERT INTO "
					       + this.SCHEMA_TABLE_NAME
					       + " (target_namespace,"
					       + "schema_id,"
					       + "schema_definition)"
					       + " VALUES (?, ?, ?)" );

			try
			{
				deleteStatement.setString
				   ( 1 , parser.getSchemaID() );

				deleteStatement.executeUpdate();


				insertStatement.setString
				   ( 1 , parser.getTargetNamespace() );

				insertStatement.setString
				   ( 2 , parser.getSchemaID() );

				insertStatement.setString
				   ( 3 , buf.toString() );

				insertStatement.executeUpdate();
			}
			catch( SQLException  e )
			{
				throw new StrageException
				  ( "Can't update schema definition: "
				    + e.getMessage() , e );
			}
		}
		catch( DatabaseConnectionException  e )
		{
			throw new StrageException
				  ( "Can't update schema definition: "
				    + e.getMessage() , e );
		}
	}

	public List<String>	getSchemasOfElement
					( ElementName  elementName )
						throws StrageException
	{
		String	query = "SELECT `schema_definition`"
				+ " FROM `" + SCHEMA_TABLE_NAME + "`"
				+ "WHERE `target_namespace`=\""
				+ elementName.getNamespace()
				+ "\" AND deleted=FALSE";

		System.err.println( "#" + query );


		List<String>	ret = new ArrayList<String>();

		try
		{
			DatabaseConnection.QueryResult
				queryResult = this.db.executeQuery( query );

			ResultSet	r = queryResult.getResultSet();

			while( r.next() )
			{
				ret.add( r.getString( /*column*/ 1 ) );
			}

			queryResult.close();
		}
		catch( SQLException  e )
		{
			throw new StrageException( e );
		}
		catch( DatabaseConnectionException e )
		{
			throw new StrageException( e );
		}

		return( ret );
	}


	public String	getSchema( String  schemaID ) throws StrageException
	{
		String	query = "SELECT `schema_definition`"
				+ " FROM `" + SCHEMA_TABLE_NAME + "`"
				+ "WHERE `schema_id`=\"" + schemaID
				+ "\" AND deleted=FALSE";

		System.err.println( "#" + query );


		String	ret = null;

		try
		{
			DatabaseConnection.QueryResult
				queryResult = this.db.executeQuery( query );

			ResultSet	r = queryResult.getResultSet();

			while( r.next() )
			{
				if ( ret != null )
				{
					throw new StrageException
					  ( "schema bounded to schema id ["
					    + schemaID
					    + "] multiply defined" );
				}

				ret = r.getString( /*column*/ 1 );
			}

			queryResult.close();
		}
		catch( SQLException  e )
		{
			throw new StrageException( e );
		}
		catch( DatabaseConnectionException e )
		{
			throw new StrageException( e );
		}

		if ( ret == null )
		{
			throw new StrageException
			  ( "schema bounded to schema id ["
			    + schemaID + "] not found" );
		}

		return( ret );
	}


	synchronized /* XXX: synchronized for typeDictionaryCache */
	public	void	registerTypeDefinition( TypeName  typeName ,
						TypeDefinition  definition ,
						String  schemaID ,
						boolean  cacheUpdateOnly )
					      throws StrageException
	{
		System.err.println( this.getClass().getName() + ": "
				    + "registerTypeDefinition: "
				    + "[" + typeName.getNamespace() + " "
					  + typeName.getLocalName() + "]" );

		this.registerUniversalNameIfNotRegistrated( typeName );

		this.typeDictionaryCache.registerType
				( typeName , definition );

		if ( definition == null )
		{
			// XXX
			System.err.println( "definition == null" );

			/*
			throw new StrageException
				  ( "unknown type: "
				    + typeName.getLocalName()
				    + " in namespace "
				    + typeName.getNamespace() );
			*/

			// XXX
			return;
		}
		else
		{
			System.err.println( "definition != null" );
		}

		if ( cacheUpdateOnly )
		{
			return;
		}


		SQLDataType	singleSQLDataType
					= definition.getSingleSQLDataType();

		if ( singleSQLDataType != null )
		{
			// composite type

			// XXX
			System.err.println
				( "single sql type = ["
				  + singleSQLDataType.getSQLDataTypeString()
				  + "]" );
		}
		else
		{
			// XXX
			System.err.println( "single sql type = null" );

			List<TableColumnDefinition>	columns;

			try
			{
				columns = definition.getCompositeSQLDataType
						  ( this , (SimpleXPath)null );
			}
			catch( TypeException  e )
			{
				throw new StrageException( e.getMessage() ,
							   e );
			}

			if ( columns == null )
			{
				// internal error
				throw new StrageException
					( "Can't determine type definition" );
			}
		}


		try
		{
			//
			// XXX: should escape charcters in schemaString
			//      and targetNamespace(?)
			//
			this.db.executeUpdate
				( "UPDATE " + this.TYPE_TABLE_NAME
				  + " SET deleted=TRUE"
				  + " WHERE"
				  + " type_namespace='"
				  + typeName.getNamespace() + "'"
				  + " AND"
				  + " type_name ='"
				  + typeName.getLocalName() + "'" );

			this.db.executeUpdate
				( "INSERT INTO " + this.TYPE_TABLE_NAME
				  + " (type_namespace,type_name,schema_uri)"
				  + " VALUES ('"
				  + typeName.getNamespace() + "','"
				  + typeName.getLocalName() + "','"
				  + schemaID + "')" );
		}
		catch( DatabaseConnectionException  e )
		{
			throw new StrageException
				  ( "Can't update type definition: "
				    + e.getMessage() , e );
		}
	}


	synchronized /* XXX: for typeDictionaryCache */
	public TypeDefinition	getTypeDefinition( TypeName  typeName )
						throws StrageException
	{
		TypeDefinition
			t = this.typeDictionaryCache.getType( typeName );

		if ( t != null )
		{
			return( t );
		}


		final String	strageExceptionMessage
				= "Can't get type definition of "
					+ typeName.toString();


		//
		// fetch element type from DB
		//
		ResultSet	r = null;

		try
		{
			r = this.db.executeQuery
				( "SELECT"
				  + " schema_uri"
				  + " FROM "
				  + this.TYPE_TABLE_NAME
				  + " WHERE type_namespace='"
				  + typeName.getNamespace() + "'"
				  + " AND type_name='"
				  + typeName.getLocalName() + "'"
				  + " AND deleted=FALSE" ).getResultSet();


			int	count = 0;
			String	schemaID = null;

			while( r.next() )
			{
				count ++;

				schemaID = r.getString( 1 );
			}

			if ( count == 0
			  || schemaID == null )
			{
				return( null );
			}
			else if ( count >= 2 )
			{
				throw new StrageException
					( strageExceptionMessage + ": "
					  + "element type definition"
					  + " multiply defined" );
			}

			String	schemaString = this.getSchema( schemaID );

			XMLSchemaParseAndRegister
				parser = new XMLSchemaParseAndRegister();
			try
			{
				parser.parseXMLSchemaString
					( schemaString , this , true );
			}
			catch( XMLSchemaParserException  e )
			{
				throw new StrageException( e.getMessage() );
			}

			return( this.typeDictionaryCache.getType( typeName ) );
		}
		catch( SQLException  e )
		{
			throw new StrageException
				( strageExceptionMessage + ": "
				  + e.getMessage() , e );
		}
		catch( DatabaseConnectionException  e )
		{
			throw new StrageException
				( strageExceptionMessage + ": "
				  + e.getMessage() );
		}
		finally
		{
			try
			{
				if ( r != null )
				{
					r.close();
				}
			}
			catch( SQLException  e )
			{
				throw new StrageException
					( strageExceptionMessage + ": "
					  + e.getMessage() );
			}
		}
	}



	public	void	registerElementDefinition( ElementName  elementName ,
						   TypeName  typeName ,
						   boolean  cacheUpdateOnly )
						throws StrageException
	{
		System.err.println( "registerElementDefinition: "
				    + "[" + elementName.getNamespace() + " - "
					  + elementName.getLocalName() + "]: "
				    + "[" + typeName.getNamespace() + " - "
					  + typeName.getLocalName() + "]" );

		this.registerUniversalNameIfNotRegistrated( typeName );


		String	elementID = this.getUniversalNameID( elementName );

		if ( elementID == null )
		{
			this.registerUniversalNameIfNotRegistrated
							( elementName );
			elementID = this.getUniversalNameID( elementName );
		}

		if ( cacheUpdateOnly )
		{
			return;
		}


		try
		{
			// XXX: transaction

			//
			// update element table
			//
			PreparedStatement	updateStatement;

			updateStatement = this.db.prepareStatement
					  ( "UPDATE " + this.ELEMENT_TABLE_NAME
					    + " SET deleted=TRUE"
					    + " WHERE"
					    + " element_namespace=?"
					    + " AND"
					    + " element_name =?" );

			PreparedStatement	insertStatement;

			insertStatement = this.db.prepareStatement
				( "INSERT INTO " + this.ELEMENT_TABLE_NAME
				  + " (element_namespace,element_name,"
				  +    "type_namespace,type_name)"
				  + " VALUES (?,?,?,?)" );


			try
			{
				updateStatement.setString
					( 1 , elementName.getNamespace() );

				updateStatement.setString
					( 2 , elementName.getLocalName() );


				insertStatement.setString
					( 1 , elementName.getNamespace() );
				insertStatement.setString
					( 2 , elementName.getLocalName() );

				insertStatement.setString
					( 3 , typeName.getNamespace() );
				insertStatement.setString
					( 4 , typeName.getLocalName() );

				updateStatement.executeUpdate();
				insertStatement.executeUpdate();
			}
			catch( SQLException  e )
			{
				throw new StrageException
				  ( "Can't update element definition: "
				    + e.getMessage() , e );
			}



			TypeDefinition	type
					= this.getTypeDefinition( typeName );

			if ( type == null )
			{
				throw new StrageException
				    ( "unknown type " + typeName );
			}


			String	tableName = elementID;

			String	tableColumnsString;

			SQLDataType  t = type.getSingleSQLDataType();

			TableDefinition	tableDefinition = null;

			if ( t != null )
			{
				tableColumnsString
					  = ElementColumnNameFactory
						    .getTopLevelColumnName()
					    + " " + t.getSQLDataTypeString();
			}
			else
			{
				try
				{
					tableDefinition
					 = new TableDefinition
					   ( type
					     .getCompositeSQLDataType
					      (this ,
					       (SimpleXPath)null) );

					tableColumnsString
						= tableDefinition
						  .getTableDefinitionString();
				}
				catch( TypeException  e )
				{
					throw new StrageException
					 ( "Can't update"
					   + " element definition: "
					   + e.getMessage() , e );
				}
			}


			String	managementColumnsString
				= "`_id_` integer not null auto_increment,"
				  + "index(`_id_`),"
				  + "_deleted_ boolean not null"
				  + " default FALSE,";

			String	sqlInstruction
				= "CREATE TABLE "
				  + "`" + tableName + "`"
				  + " (" + managementColumnsString
					 + tableColumnsString + ")";

			System.err.println( "#" + sqlInstruction );


			this.db.dropTableIfExists( tableName );
			this.db.executeUpdate( sqlInstruction );


			if ( tableDefinition != null )
			{
				for ( String  subTableName
					: tableDefinition.getSubTables
							      ( tableName ) )
				{
					System.err.println( "sub table = "
							    + subTableName );

					this.db.dropTableIfExists
							    ( subTableName );

					String	subTableSqlInstruction
					  = "CREATE TABLE "
					  + "`" + subTableName + "`"
					  + " (`_id_` integer not null,"
					  + "index(`_id_`),"
					  + "`_parent_id_` integer not null,"
					  + "index(`_parent_id_`),"
					  + ElementColumnNameFactory
						    .getTopLevelColumnName()
					  + " blob)";

					System.err.println
					    ( "#" + subTableSqlInstruction );

					this.db.executeUpdate
						( subTableSqlInstruction );
				}

			}

			this.elementDictionaryCache.put( elementName ,
							 typeName );
		}
		catch( DatabaseConnectionException  e )
		{
			throw new StrageException
				  ( "Can't update element definition: "
				    + e.getMessage() , e );
		}
	}


	/**
	 * returns type of element.
	 *
	 * If type has not found, returns null
	 */
	public	TypeName	getElementTypeName( ElementName  elementName )
							throws StrageException
	{
		TypeName  ret = this.elementDictionaryCache.get( elementName );

		if ( ret != null )
		{
			return( ret );
		}


		final String	strageExceptionMessage
				= "Can't get type definition of element "
					+ elementName.toString();


		//
		// fetch element type from DB
		//
		PreparedStatement	queryStatement = null;
		ResultSet		r = null;

		try
		{
			/*
			queryStatement = this.db.prepareStatement
					      ( "SELECT"
						+ " type_namespace,type_name"
						+ " FROM "
						+ this.ELEMENT_TABLE_NAME
						+ " WHERE element_namespace=?"
						+ " AND element_name=?"
						+ " AND deleted=TRUE" );

			queryStatement.setString
					( 1 , elementName.getNamespace() );
			queryStatement.setString
					( 2 , elementName.getLocalName() );

			r = queryStatement.executeQuery();
			*/

			r = this.db.executeQuery
				( "SELECT"
				  + " type_namespace,type_name"
				  + " FROM "
				  + this.ELEMENT_TABLE_NAME
				  + " WHERE element_namespace='"
				  + elementName.getNamespace() + "'"
				  + " AND element_name='"
				  + elementName.getLocalName() + "'"
				  + " AND deleted=FALSE" ).getResultSet();


			int	count = 0;

			String	typeNamespace = null;
			String	typeLocalName = null;

			while( r.next() )
			{
				count ++;

				typeNamespace = r.getString( /*column*/ 1 );
				typeLocalName = r.getString( /*column*/ 2 );
			}

			if ( count == 0
			  || typeNamespace == null
			  || typeLocalName == null )
			{
				/*
				throw new StrageException
					( strageExceptionMessage + ": "
					  + "element type definition"
					  + " not found in DB" );
				*/
				return( null );
			}
			else if ( count >= 2 )
			{
				throw new StrageException
					( strageExceptionMessage + ": "
					  + "element type definition"
					  + " multiply defined" );
			}

			TypeName  typeName = new TypeName
						 ( typeNamespace ,
						   typeLocalName );

			this.elementDictionaryCache
				.put( elementName , typeName );

			return( typeName );
		}
		catch( SQLException  e )
		{
			throw new StrageException
				( strageExceptionMessage + ": "
				  + e.getMessage() , e );
		}
		catch( DatabaseConnectionException  e )
		{
			throw new StrageException
				( strageExceptionMessage + ": "
				  + e.getMessage() );
		}
		finally
		{
			try
			{
				if ( r != null )
				{
					r.close();
				}
			}
			catch( SQLException  e )
			{
				throw new StrageException
					( strageExceptionMessage + ": "
					  + e.getMessage() );
			}
			/*
			finally
			{

				try
				{
					queryStatement.close();
				}
				catch( SQLException  e )
				{
					throw new StrageException
						( strageExceptionMessage + ": "
						  + e.getMessage() );
				}
			}
			*/
		}
	}


	public	void	insertElement( UniversalName  elementName ,
				       TypedInstance  obj )
							throws StrageException
	{
		String	tableName;

		try
		{
			tableName = this.getUniversalNameID( elementName );
		}
		catch( StrageException e )
		{
			throw e;
		}


		List<TableColumn>	columns = obj.getColumns();

		TableRow	row = new TableRow( columns );

		try
		{
			row.executeInsert( this.db , tableName );
		}
		catch( DatabaseConnectionException  e )
		{
			throw new StrageException( "data insertion failed: "
						   + e.getMessage() ,
						   e );
		}
	}

	public	void	deleteElement( UniversalName  elementName ,
				       PredicateDescription  predicate ,
				       boolean  virtualDelete )
							throws StrageException
	{
		String	id = this.getUniversalNameID( elementName );

		if ( id == null )
		{
			throw new StrageException( elementName
						   + " not found" );
		}

		String	sqlString;

		if ( virtualDelete )
		{
			sqlString = "UPDATE `" + id + "`"
					+ " SET `_deleted_`=TRUE";
		}
		else
		{
			sqlString = "DELETE FROM `" + id + "`";
		}


		if ( predicate != null )
		{
			sqlString += " WHERE " + predicate.getSQLExpression();
		}

		System.err.println( "#" + sqlString );

		try
		{
			this.db.executeUpdate( sqlString );
		}
		catch( DatabaseConnectionException  e )
		{
			throw new StrageException
				  ( "Can't delete element "
				    + elementName.toString()
				    + ": " + e.getMessage() , e );
		}
	}

	public	void	restoreElement( UniversalName  elementName ,
					PredicateDescription  predicate )
							throws StrageException
	{
		String	id = this.getUniversalNameID( elementName );

		if ( id == null )
		{
			throw new StrageException( elementName
						   + " not found" );
		}

		String	sqlString;

		sqlString = "UPDATE `" + id + "` SET `_deleted_`=FALSE";

		if ( predicate != null )
		{
			sqlString += " WHERE " + predicate.getSQLExpression();
		}

		System.err.println( "#" + sqlString );

		try
		{
			this.db.executeUpdate( sqlString );
		}
		catch( DatabaseConnectionException  e )
		{
			throw new StrageException
				  ( "Can't restore element "
				    + elementName.toString()
				    + ": " + e.getMessage() , e );
		}
	}

	public	void	updateElement
			    ( UniversalName  elementName ,
			      List< Pair<SimpleXPath,String> >  updateValues ,
			      PredicateDescription  predicate ,
			      boolean  virtualUpdate ) throws StrageException
	{
		String	id = this.getUniversalNameID( elementName );

		if ( id == null )
		{
			throw new StrageException
				( elementName + " not found" );
		}

		if ( updateValues.isEmpty() )
		{
			return;
		}

		StringBuilder	sqlString = new StringBuilder();
		sqlString.append( "UPDATE `" + id + "` SET " );

		boolean	firstColumn = true ;
		for ( Pair<SimpleXPath, String>  p  :  updateValues )
		{
			String	columnName;

			try
			{
				columnName = super.getShortXPathString
							( p.getFirst() );
			}
			catch( StrageException  e )
			{
				throw new StrageException
				    ( "invalid property: " + p.getFirst() ,
				      e );
			}

			if ( ! firstColumn )
			{
				sqlString.append( "," );
			}
			firstColumn = false;

			sqlString.append( "`" );
			sqlString.append( columnName );
			sqlString.append( "`='" );
			sqlString.append( p.getSecond() );
			sqlString.append( "'" );
		}

		if ( predicate != null )
		{
			sqlString.append
				  ( " WHERE " + predicate.getSQLExpression() );
		}

		System.err.println( "#" + sqlString );


		try
		{
			this.db.executeUpdate( sqlString.toString() );
		}
		catch( DatabaseConnectionException  e )
		{
			throw new StrageException
				  ( "Can't update element "
				    + elementName.toString()
				    + ": " + e.getMessage() , e );
		}
	}

	public	TypedInstanceSet	getTypedInstanceSet
					  ( ElementName  elementName ,
					    String sqlWhereBlock )
							throws StrageException
	{
		TypeDefinition	type;

		try
		{
			type = super.getElementTypeDefinition( elementName );
		}
		catch( StrageException  e )
		{
			throw e;
		}

		if ( type == null )
		{
			throw new StrageException( "type of element "
						   + elementName
						   + " not found" );
		}


		String	tableName;

		try
		{
			tableName = this.getUniversalNameID( elementName );
		}
		catch( StrageException e )
		{
			throw e;
		}


		try
		{
			String	retrieveColumnString;

			SQLDataType  t = type.getSingleSQLDataType();

			if ( t != null )
			{
				String	tableColumnsString
					= ElementColumnNameFactory
						      .getTopLevelColumnName();

				retrieveColumnString
				  = t.getSQLRetrieveColumnString
					  ( tableColumnsString );
			}
			else
			{
				retrieveColumnString
				= new TableDefinition
					( type.getCompositeSQLDataType
					  (this ,
					   (SimpleXPath)null) )
				  .getCommaSeparatedColumnNamesForRawSQLString();
			}


			String	sqlString = "SELECT "
					    + "_id_,"
					    + retrieveColumnString
					    + " FROM "
					    + "`" + tableName + "`";

			if ( sqlWhereBlock != null )
			{
				sqlString += " WHERE `_deleted_`=FALSE AND ("
						+ sqlWhereBlock + ")";
			}
			else
			{
				sqlString += " WHERE `_deleted_`=FALSE";
			}

			System.err.println( "#" + sqlString );

			return( new TypedInstanceSet
				( type ,
				  this.db.executeQuery( sqlString ) ,
				  this ) );
		}
		catch( DatabaseConnectionException  e )
		{
			throw new StrageException
				  ( "data retrieve failed: " + e.getMessage() ,
				    e );
		}
		catch( TypeException  e )
		{
			throw new StrageException
				  ( "data retrieve failed: " + e.getMessage() ,
				    e );
		}
	}

	public	List<String>	getSubTableValues( String  tableName ,
						   long  parentID )
							throws StrageException
	{
		String	query = "SELECT `"
				+ ElementColumnNameFactory
						.getTopLevelColumnName()
				+ "` FROM `" + tableName + "`"
				+ "WHERE `_parent_id_`=" + parentID;

		System.err.println( "#" + query );

		List<String>	ret = new ArrayList<String>();

		try
		{
			DatabaseConnection.QueryResult
				queryResult = this.db.executeQuery( query );

			ResultSet	r = queryResult.getResultSet();

			while( r.next() )
			{
				ret.add( r.getString( /*column*/ 1 ) );
			}

			queryResult.close();
		}
		catch( SQLException  e )
		{
			throw new StrageException( e );
		}
		catch( DatabaseConnectionException e )
		{
			throw new StrageException( e );
		}

		return( ret );
	}

	public	List<ElementName>  getElementList() throws StrageException
	{
		DatabaseConnection.QueryResult	queryResult;

		try
		{
			queryResult
			      = this.db.executeQuery
				( "SELECT element_namespace,element_name "
				  + "FROM " + ELEMENT_TABLE_NAME
				  + " WHERE deleted=FALSE" );
		}
		catch( DatabaseConnectionException  e )
		{
			throw new StrageException( e );
		}


		List<ElementName>	featureTypeList
					    = new ArrayList<ElementName>();

		try
		{
			ResultSet	r = queryResult.getResultSet();

			while( r.next() )
			{
				featureTypeList
				    .add( new ElementName
					  ( r.getString( /*column*/ 1 ) ,
					    r.getString( /*column*/ 2 ) ) );
			}

			queryResult.close();
		}
		catch( SQLException  e )
		{
			throw new StrageException( e );
		}
		catch( DatabaseConnectionException e )
		{
			throw new StrageException( e );
		}

		return( featureTypeList );
	}


	//
	// universal name alias
	//
	public void	registerUniversalNameIfNotRegistrated
					( UniversalName  universalName )
							throws StrageException
	{
		System.err.println( this.getClass().getName() + ": "
				    + "registerUniversalNameIfNotRegistrated: "
				    + universalName.getNamespace() + " - "
				    + universalName.getLocalName() + "]" );

		if ( this.getUniversalNameID( universalName ) != null )
		{
			return;
		}


		try
		{
			this.db.startTransaction();

			long	idNumber
				= this.db.getMaxLongValueFromTable
				       ( this.UNIVERSAL_NAME_ALIAS_TABLE_NAME ,
					 "id_number" );
			idNumber ++;

			String	idString = "id" + Long.toString( idNumber );

			String	alias = idString
					+ "-"
					+ StringSplitter
					  .getLastNonEmptyElement
					   ( universalName.getNamespace() ,
					     "/" )
					+ "-"
					+ universalName.getLocalName();



			//
			// when id column type equals integer auto_increment
			//
			PreparedStatement	s;
			s = this.db.prepareStatement
				    ( "INSERT INTO "
				      + this.UNIVERSAL_NAME_ALIAS_TABLE_NAME
				      + " (namespace,local_name,"
				      + "id_number,id,alias)"
				      + " VALUES (?,?,?,?,?)" );

			try
			{
				s.setString( 1 ,
					     universalName.getNamespace() );
				s.setString( 2 ,
					     universalName.getLocalName() );
				s.setLong  ( 3 , idNumber );
				s.setString( 4 , idString );
				s.setString( 5 , alias );

				s.executeUpdate();
			}
			catch( SQLException e )
			{
				throw new StrageException
				  ( "Can't update universal name id: "
				    + e.getMessage() , e );
			}

			// calls getUniversalNameID() only for cache updating
			this.getUniversalNameID( universalName );
		}
		catch( DatabaseConnectionException  e )
		{
			throw new StrageException
				  ( "Can't update universal name id: "
				    + e.getMessage() , e );
		}
		finally
		{
			try
			{
				this.db.endTransaction();
			}
			catch( DatabaseConnectionException  e )
			{
				throw new StrageException
				  ( "Can't close database connection" , e );
			}
		}
	}


	/**
	 * returns UniversalNameID which is a short alias name of universalName
	 *
	 * when UniversalNameID has not found, returns null
	 */
	public String	getUniversalNameID( UniversalName  universalName )
							throws StrageException
	{
		String	id = this.universalNameDictionaryCache
						    .get( universalName );

		if ( id != null )
		{
			return( id );
		}


		DatabaseConnection.QueryResult	queryResult = null;

		try
		{
			queryResult = this.db.executeQuery
					   ( "SELECT id FROM "
					     + this
					       .UNIVERSAL_NAME_ALIAS_TABLE_NAME
					     + " WHERE NOT deleted"
					     + " AND namespace='"
					     + universalName.getNamespace()
					     + "'"
					     + " AND local_name='"
					     + universalName.getLocalName()
					     + "'" );


			ResultSet	r = queryResult.getResultSet();

			long	count = 0;
			while( r.next() )
			{
				id = r.getString( 1 );
				count ++;
			}


			if ( count == 0 )
			{
				return null;
			}
			else if ( count >= 2 )
			{
				throw new StrageException
					  ( "Database consistency error:"
					    + " id of ["
					    + universalName.getNamespace()
					    + " - "
					    + universalName.getLocalName()
					    + "] multiply defined" );
			}


			//
			// add to local cache
			//
			this.universalNameDictionaryCache
				.put( universalName , id );
		}
		catch( SQLException  e )
		{
			throw new StrageException( e );
		}
		catch( DatabaseConnectionException  e )
		{
			throw new StrageException( e );
		}
		finally
		{
			if ( queryResult != null )
			{
				try
				{
					queryResult.close();
				}
				catch( DatabaseConnectionException  e )
				{
					throw new StrageException
					  ( "Can't close database connection" ,
					    e );
				}
			}
		}

		return( id );
	}



	public	Date	getCurrentTime() throws StrageException
	{
		try
		{
			return this.db.getCurrentTime();
		}
		catch( DatabaseConnectionException e )
		{
			throw new StrageException( e );
		}
	}
}
