package daruma.strage_manager;

import daruma.strage_manager.StrageException;
import daruma.strage_manager.type_definition.ElementName;
import daruma.strage_manager.type_definition.TypeName;
import daruma.strage_manager.type_definition.TypeDefinition;
import daruma.strage_manager.type_definition.TypedInstance;
import daruma.strage_manager.type_definition.TypedInstanceSet;

// XXX: PredicateDescription should be moved to other package?
import daruma.wfs.filter.PredicateDescription;


import daruma.xml.UniversalName;
import daruma.xml.SimpleXPath;
import daruma.xml.util.PrefixMap;
import daruma.util.Pair;

import org.w3c.dom.Element;

import java.util.Date;

import java.util.List;
import java.util.Map;


public abstract class StrageManager
{
	public	static	final	String	SHORT_XPATH_SEPARATOR = "$";

	//
	// schema
	//
	public abstract void		registerSchema
					  ( Element  schema ,
					    PrefixMap  prefixMap )
						throws StrageException;

	public abstract List<String>	getSchemasOfElement
					( ElementName  elementName )
						throws StrageException;


	//
	// type
	//
	public abstract void		registerTypeDefinition
					  ( TypeName  typeName ,
					    TypeDefinition  definition ,
					    String  schemaID ,
					    boolean  cacheUpdateOnly )
						throws StrageException;

	public abstract TypeDefinition	getTypeDefinition( TypeName  typeName )
						throws StrageException;



	//
	// element
	//
	public abstract void		registerElementDefinition
					  ( ElementName  elementName ,
					    TypeName  typeName ,
					    boolean  cacheUpdateOnly )
						throws StrageException;
	public abstract TypeName	getElementTypeName
					  ( ElementName  elementName )
						throws StrageException;

	public abstract	List<ElementName>	getElementList()
						throws StrageException;

	public abstract	void	insertElement( UniversalName  elementName ,
					       TypedInstance  obj )
							throws StrageException;

	public abstract	TypedInstanceSet	getTypedInstanceSet
						  ( ElementName  elementName ,
						    /* Query */
						    String  sqlWhereBlock )
							throws StrageException;

	public abstract	void	deleteElement
				    ( UniversalName  elementName ,
				      PredicateDescription  predicate ,
				      boolean  virtualDelete )
							throws StrageException;

	public	abstract void	restoreElement
				    ( UniversalName  elementName ,
				      PredicateDescription  predicate )
							throws StrageException;

	public	abstract void	updateElement
				    ( UniversalName  elementName ,
				      List< Pair<SimpleXPath,String> >
							  updateValues ,
				      PredicateDescription  predicate ,
				      boolean  virtualUpdate )
							throws StrageException;

	// XXX
	public abstract List<String>	getSubTableValues( String  tableName ,
							   long  parentID )
							throws StrageException;

	//
	// universal name
	//
	public abstract	void	registerUniversalNameIfNotRegistrated
					  ( UniversalName  universalName )
							throws StrageException;
	public abstract	String	getUniversalNameID( UniversalName  name )
							throws StrageException;


	//
	// time
	//
	public	abstract Date	getCurrentTime() throws StrageException;


	//
	// utility
	//
	public	String	getShortXPathString( SimpleXPath  path )
						     throws StrageException
	{
		StringBuilder	buf = new StringBuilder();

		for ( UniversalName  name  :  path.getList() )
		{
			try
			{
				if ( buf.length() != 0 )
				{
					buf.append
					    ( this.SHORT_XPATH_SEPARATOR );
				}

				String	id = this.getUniversalNameID( name );

				if ( id == null )
				{
					throw new StrageException
						( "Can't convert path ["
						  + path.toString() + "]" );
				}

				buf.append( this.getUniversalNameID( name ) );
			}
			catch( StrageException  e )
			{
				throw e;
			}
		}

		return( buf.toString() );
	}

	public	TypeDefinition	getElementTypeDefinition
					  ( ElementName  elementName )
						throws StrageException
	{
		TypeName	typeName;

		try
		{
			typeName = this.getElementTypeName( elementName );
		}
		catch( StrageException e )
		{
			throw new StrageException( "type of element \""
						   + elementName.getLocalName()
						   + "\" in namespace \""
						   + elementName.getNamespace()
						   + "\" not found" , e );
		}

		try
		{
			return( this.getTypeDefinition( typeName ) );
		}
		catch( StrageException e )
		{
			throw new StrageException( "type definition of type \""
						   + typeName.getLocalName()
						   + "\" in namespace \""
						   + typeName.getNamespace()
						   + "\" not found" , e );
		}
	}
}
