package daruma.strage_manager;

import daruma.strage_manager.type_definition.TypeName;
import daruma.strage_manager.type_definition.TypeDefinition;

import daruma.strage_manager.type_definition.types.IntegerTypeDefinition;
import daruma.strage_manager.type_definition.types.FloatTypeDefinition;
import daruma.strage_manager.type_definition.types.BooleanTypeDefinition;
import daruma.strage_manager.type_definition.types.StringTypeDefinition;
import daruma.strage_manager.type_definition.types.DateTimeTypeDefinition;
import daruma.strage_manager.type_definition.types.GeometryPropertyTypeDefinition;
import daruma.strage_manager.type_definition.types.SequenceTypeDefinition;
import daruma.strage_manager.type_definition.types.AnyXMLTypeDefinition;

import daruma.xml.URI;

import java.util.Map;
import java.util.HashMap;


public class TypeDictionary
{
	private	Map<TypeName, TypeDefinition>	typeMap;

	public	TypeDictionary()
	{
		this.typeMap = new HashMap<TypeName, TypeDefinition>();

		this.registerBuiltinType();
	}

	private	void	registerBuiltinType()
	{
		this.registerType( new TypeName( URI.XML_SCHEMA ,
						 "string" ) ,
				   new StringTypeDefinition() );
		this.registerType( new TypeName( URI.XML_SCHEMA ,
						 "integer" ) ,
				   new IntegerTypeDefinition() );
		this.registerType( new TypeName( URI.XML_SCHEMA ,
						 "double" ) ,
				   new FloatTypeDefinition() );
		this.registerType( new TypeName( URI.XML_SCHEMA ,
						 "float" ) ,
				   new FloatTypeDefinition() );
		this.registerType( new TypeName( URI.XML_SCHEMA ,
						 "decimal" ) ,
				   new FloatTypeDefinition() );
		this.registerType( new TypeName( URI.XML_SCHEMA ,
						 "boolean" ) ,
				   new BooleanTypeDefinition() );
		this.registerType( new TypeName( URI.XML_SCHEMA ,
						 "dateTime" ) ,
				   new DateTimeTypeDefinition() );

		this.registerType( new TypeName( URI.GML ,
						 "TimePositionType" ) ,
				   new DateTimeTypeDefinition() );

		this.registerType( new TypeName( URI.GML ,
						 "AbstractFeatureType" ) ,
				   new SequenceTypeDefinition() );

		// XXX
		this.registerType( new TypeName( URI.GML ,
						 "GeometryPropertyType" ) ,
				   new GeometryPropertyTypeDefinition() );
		// XXX: obsolated, should be [GeometryPropertyType]
		this.registerType( new TypeName( URI.GML ,
						 "geometryPropertyType" ) ,
				   new GeometryPropertyTypeDefinition() );

		//
		// XXX: aliases, loosely checking
		//
		this.registerType( new TypeName( URI.XML_SCHEMA ,
						 "byte" ) ,
				   new IntegerTypeDefinition() );
		this.registerType( new TypeName( URI.XML_SCHEMA ,
						 "int" ) ,
				   new IntegerTypeDefinition() );
		this.registerType( new TypeName( URI.XML_SCHEMA ,
						 "short" ) ,
				   new IntegerTypeDefinition() );
		this.registerType( new TypeName( URI.XML_SCHEMA ,
						 "long" ) ,
				   new IntegerTypeDefinition() );

		this.registerType( new TypeName( URI.XML_SCHEMA ,
						 "negativeInteger" ) ,
				   new IntegerTypeDefinition() );
		this.registerType( new TypeName( URI.XML_SCHEMA ,
						 "positiveInteger" ) ,
				   new IntegerTypeDefinition() );
		this.registerType( new TypeName( URI.XML_SCHEMA ,
						 "nonNegativeInteger" ) ,
				   new IntegerTypeDefinition() );
		this.registerType( new TypeName( URI.XML_SCHEMA ,
						 "nonPositiveInteger" ) ,
				   new IntegerTypeDefinition() );

		this.registerType( new TypeName( URI.XML_SCHEMA ,
						 "unsignedByte" ) ,
				   new IntegerTypeDefinition() );
		this.registerType( new TypeName( URI.XML_SCHEMA ,
						 "unsignedShort" ) ,
				   new IntegerTypeDefinition() );
		this.registerType( new TypeName( URI.XML_SCHEMA ,
						 "unsignedInt" ) ,
				   new IntegerTypeDefinition() );
		this.registerType( new TypeName( URI.XML_SCHEMA ,
						 "unsignedLong" ) ,
				   new IntegerTypeDefinition() );

		this.registerType( new TypeName( URI.XML_SCHEMA ,
						 "anyURI" ) ,
				   new StringTypeDefinition() );
		this.registerType( new TypeName( URI.XML_SCHEMA ,
						 "base64Binary" ) ,
				   new StringTypeDefinition() );
		this.registerType( new TypeName( URI.XML_SCHEMA ,
						 "language" ) ,
				   new StringTypeDefinition() );

		this.registerType( new TypeName( URI.MISP ,
						 "anyXML" ) ,
				   new AnyXMLTypeDefinition() );


		//
		// ... other embeded types
		// NCName
		// XXX: not implemented yet
	}


	/**
	 * register mapping of TypeName and TypeDefinition to dictionary
	 */
	public	void	registerType( TypeName  typeName ,
				      TypeDefinition  typeDefinition )
	{
		System.err.println( "TypeDictionary.registerType: "
				    + "[" + typeName + "]: "
				    + "[" + typeDefinition + "]" );

		this.typeMap.put( typeName , typeDefinition );
	}


	/**
	 * search TypeDefinition from dictionary
	 *
	 * if definition was not found, this method returns null.
	 */
	public	TypeDefinition	getType( TypeName  typeName )
	{
		return( this.typeMap.get( typeName ) );
	}


	public	void	debugPrint()
	{
		System.err.println( "TypeDictionary: size = "
				    + this.typeMap.size() );

		for ( Map.Entry<TypeName, TypeDefinition>  e
			      : this.typeMap.entrySet() )
		{
			System.err.println( "[" + e.getKey() + "]: ["
					    + e.getValue() + "]" );
		}
	}
}
