package daruma.strage_manager.type_definition;

import daruma.strage_manager.type_definition.TypeDefinition;
import daruma.strage_manager.type_definition.TypeName;
import daruma.strage_manager.type_definition.ElementName;
import daruma.strage_manager.type_definition.TypedInstance;
import daruma.strage_manager.type_definition.TypeException;
import daruma.strage_manager.StrageManager;
import daruma.strage_manager.StrageException;
import daruma.sql.SQLDataType;
import daruma.sql.TableColumnDefinition;
import daruma.sql.TableColumn;
import daruma.xml.SimpleXPath;
import daruma.util.Pair;

import org.w3c.dom.Node;
import org.w3c.dom.Element;
import org.w3c.dom.Document;

import java.util.List;
import java.util.ArrayList;


public abstract class AbstractCompositorTypeDefinition extends TypeDefinition
{
	public static class  Entry
	{
		private	Pair<TypeName, ElementName>	atom;
		private	TypeDefinition			composite;

		protected Entry( Pair<TypeName, ElementName>  atomEntry )
		{
			this.atom      = atomEntry;
			this.composite = null;
		}

		protected Entry( TypeDefinition  compositeEntry )
		{
			this.atom      = null;
			this.composite = compositeEntry;
		}

		public boolean	isCompositeEntry()
		{
			return( this.composite != null );
		}

		public Pair<TypeName, ElementName>	getAtomEntry()
		{
			return( this.atom );
		}

		public TypeDefinition	getCompositeEntry()
		{
			return( this.composite );
		}
	}


	private	List<Entry>	entries;

	public	AbstractCompositorTypeDefinition()
	{
		super( true , true );

		this.entries = new ArrayList<Entry>();
	}

	public final SQLDataType	getSingleSQLDataType()
	{
		return( null );
	}

	protected final List<Entry>	getEntries()
	{
		return( this.entries );
	}


	public List<TableColumnDefinition>
			  getCompositeSQLDataType( StrageManager  strage ,
						   SimpleXPath  path )
							throws TypeException
	{
		List<TableColumnDefinition>
			ret = new ArrayList<TableColumnDefinition>();

		for( Entry  e : this.getEntries() )
		{
			if ( e.isCompositeEntry() )
			{
				TypeDefinition	d = e.getCompositeEntry();

				ret.addAll( d.getCompositeSQLDataType
					    ( strage , path ) );

				continue;
			}


			TypeDefinition	type = null;

			try
			{
				type = strage.getTypeDefinition
				       ( e.getAtomEntry().getFirst() );
			}
			catch( StrageException  se )
			{
				throw new TypeException( se );
			}


			ElementName	elementName
					  = e.getAtomEntry().getSecond();

			SimpleXPath	childPrefixSimpleXPath
					  = new SimpleXPath( path );
			childPrefixSimpleXPath.add( elementName );

			System.err.println( "!!path = ["
					    + childPrefixSimpleXPath
					    + "]" );

			if ( ! type.isSQLMultiColumn() )
			{
				String	shortXPathString;
				try
				{
					shortXPathString
						= strage.getShortXPathString
						    ( childPrefixSimpleXPath );
				}
				catch( StrageException  se )
				{
					throw new TypeException( se );
				}

				ret.add( new TableColumnDefinition
					 ( shortXPathString ,
					   type.getSingleSQLDataType() ) );
			}
			else
			{
				ret.addAll( type.getCompositeSQLDataType
					    ( strage ,
					      childPrefixSimpleXPath ) );
			}
		}

		return( ret );
	}

	public final void	addChild( TypeName  typeName ,
					  ElementName  elementName )
	{
		System.err.println( this.getClass().getName()
				    + ".addChild():" );

		System.err.println( " typeName = [" + typeName + "],"
				    + " elementName = [" + elementName + "]" );

		this.entries.add( new Entry( new Pair<TypeName, ElementName>
					     ( typeName , elementName ) ) );
	}

	public	void	addChild( TypeDefinition  typeDefinition )
	{
		System.err.println( this.getClass().getName()
				    + ".addChild():" );

		System.err.println( " class = ["
				    + typeDefinition.getClass().getName()
				    + "]" );

		this.entries.add( new Entry( typeDefinition ) );
	}

	public abstract	Pair<TypedInstance, Integer>
				createInstance( Element  element ,
						ElementName  topLevelElement ,
						SimpleXPath  path ,
						StrageManager  strage ,
						int  elementIndex )
							  throws TypeException;

	public int  convertToXMLElement( Element  element ,
					 Document  doc ,
					 StrageManager  strage ,
					 List<TableColumn>  columns ,
					 int  index ,
					 long  id ) throws TypeException
	{
		int	i = index;

		for( Entry  e  :  this.getEntries() )
		{
			if ( e.isCompositeEntry() )
			{
				i = e.getCompositeEntry()
					.convertToXMLElement( element ,
							      doc ,
							      strage ,
							      columns ,
							      i ,
							      id );

				continue;
			}


			Pair<TypeName, ElementName>	a = e.getAtomEntry();

			TypeName	childTypeName    = a.getFirst();
			ElementName	childElementName = a.getSecond();

			Element	child = doc.createElementNS
					( childElementName.getNamespace() ,
					  childElementName.getLocalName() );

			element.appendChild( child );


			TypeDefinition	childType;
			try
			{
				childType = strage.getTypeDefinition
							( childTypeName );
			}
			catch( StrageException  se )
			{
				throw new TypeException( se );
			}


			if ( ! childType.isSQLMultiColumn()
			  && ! childType.isXMLComplexType() )
			{
				child.appendChild( doc.createTextNode
						   ( columns.get(i)
						     .getValue() ) );

				i ++;
			}
			else
			{
				i = childType.convertToXMLElement( child ,
								   doc ,
								   strage ,
								   columns ,
								   i ,
								   id );
			}
		}

		return( i );
	}


	public	void	debugPrint()
	{
		for( Entry  e : this.entries )
		{
			if ( e.isCompositeEntry() )
			{
				System.err.print( "composite, " );

				TypeDefinition	t = e.getCompositeEntry();

				System.err.println( t.getClass().getName() );
			}
			else
			{
				System.err.println
					( "atomic: "
					  + e.getAtomEntry().getFirst()
					  + ": "
					  + e.getAtomEntry().getSecond() );
			}
		}
	}
}
