package daruma.strage_manager.type_definition;

import daruma.strage_manager.type_definition.TypeDefinition;
import daruma.strage_manager.type_definition.TypeException;
import daruma.strage_manager.type_definition.ElementColumnNameFactory;

import daruma.sql.TableColumn;
import daruma.sql.TableColumnDefinition;
import daruma.sql.SQLDataType;
import daruma.sql.DatabaseConnection;
import daruma.sql.DatabaseConnectionException;
import java.sql.SQLException;
import java.sql.Timestamp;

import daruma.strage_manager.StrageManager;

import daruma.xml.util.DOMDocumentFactory;
import org.w3c.dom.Node;
import org.w3c.dom.Element;
import org.w3c.dom.Document;
import org.w3c.dom.DocumentFragment;
import javax.xml.parsers.ParserConfigurationException;

import daruma.util.ISO8601DateFormat;
import java.util.Date;
import java.util.List;
import java.util.ArrayList;


public class TypedInstance
{
	private	List<TableColumn>	columns;
	private	TypeDefinition		type;
	private	long			id;

	public	TypedInstance( List<TableColumn>  columns ,
			       TypeDefinition  type )
	{
		this.columns = columns;
		this.type    = type;
		this.id      = -1;
	}

	public	TypedInstance( TypeDefinition  type ,
			       DatabaseConnection.QueryResult  queryResult ,
			       StrageManager  strage ) throws TypeException
	{
		this.type    = type;
		this.columns = new ArrayList<TableColumn>();

		SQLDataType	sqlDataType= this.type.getSingleSQLDataType();
		if ( sqlDataType != null )
		{
			String	value = null;

			try
			{
				value = this.internalGetValueFromResultSet
					( queryResult ,
					  ElementColumnNameFactory
						    .getTopLevelColumnName() ,
					  sqlDataType );
			}
			catch( TypeException  e )
			{
				throw e;
			}

			this.columns.add( new TableColumn
					  ( new TableColumnDefinition
					    ( ElementColumnNameFactory
						    .getTopLevelColumnName() ,
					      sqlDataType ) ,
					    value ) );

			return;
		}


		List<TableColumnDefinition>	columnDefinition;
		columnDefinition = this.type.getCompositeSQLDataType( strage ,
								      null );

		if ( columnDefinition == null )
		{
			throw new TypeException
				  ( "GetFeature: single type"
				    + " not implemented yet" );
		}

		for( TableColumnDefinition  c : columnDefinition )
		{
			String	value;

			try
			{
				value = this.internalGetValueFromResultSet
					( queryResult ,
					  c.getColumnName() ,
					  c.getSQLDataType() );
			}
			catch( TypeException  e )
			{
				throw e;
			}

			this.columns.add( new TableColumn( c , value ) );
		}
	}


	private	String	internalGetValueFromResultSet
				( DatabaseConnection.QueryResult  queryResult ,
				  String  columnName ,
				  SQLDataType  type ) throws TypeException
	{
		String	realColumnName = type.getSQLRetrieveColumnString
								( columnName );


		try
		{
			this.id = queryResult.getResultSet().getLong( "_id_" );

			if ( Date.class.isAssignableFrom
					     ( type.getJavaClass() ) )
			{
				Timestamp  t = queryResult.getResultSet()
						 .getTimestamp
							( realColumnName );
				ISO8601DateFormat  f = new ISO8601DateFormat();

				return( f.format( t ) );
			}
			else
			{
				// XXX: getResultSet().get*() should be choosed
				//      by sqlDataType
				return( queryResult.getResultSet()
					    .getString( realColumnName ) );
			}
		}
		catch( SQLException  e )
		{
			throw new TypeException( e );
		}
		catch( DatabaseConnectionException  e )
		{
			throw new TypeException( e );
		}
	}


	public	List<TableColumn>	getColumns()
	{
		return( this.columns );
	}


	public	Node	convertToXMLElement( ElementName  elementName ,
					     StrageManager  strage )
							  throws TypeException
	{
		Document	doc;

		try
		{
			doc = DOMDocumentFactory.create();
		}
		catch( ParserConfigurationException  e )
		{
			throw new TypeException( e );
		}

		DocumentFragment	frag = doc.createDocumentFragment();

		Element	e = doc.createElementNS
				( elementName.getNamespace() ,
				  elementName.getLocalName() );

		/*
		if ( this.getColumns().size() == 1 )
		*/
		if ( ! this.type.isXMLComplexType() )
		{
			frag.appendChild( e );

			e.appendChild( doc.createTextNode
				       ( this.getColumns().get(0)
							 .getValue() ) );


			return( frag );
		}
		else
		{
			this.type.convertToXMLElement
				( e , doc , strage , this.getColumns() , 0 ,
				  id );

			return( e );
		}
	}
}
