package daruma.strage_manager.type_definition.types;

import daruma.strage_manager.type_definition.TypeDefinition;
import daruma.strage_manager.type_definition.TypedInstance;
import daruma.strage_manager.type_definition.TypeException;
import daruma.strage_manager.type_definition.ElementColumnNameFactory;
import daruma.strage_manager.type_definition.ElementName;
import daruma.strage_manager.StrageManager;
import daruma.strage_manager.StrageException;
import daruma.sql.SQLDataType;
import daruma.sql.TableColumnDefinition;
import daruma.sql.TableColumn;
import daruma.xml.UniversalName;
import daruma.xml.SimpleXPath;
import daruma.xml.util.ElementUtil;
import daruma.xml.util.XMLFormatConverter;
import daruma.xml.util.XMLParseErrorException;
import daruma.util.Pair;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

import javax.xml.transform.TransformerException;
import java.util.List;
import java.util.ArrayList;
import java.io.ByteArrayOutputStream;
import java.io.ByteArrayInputStream;


public class AnyXMLTypeDefinition extends TypeDefinition
{
	public	AnyXMLTypeDefinition()
	{
		super( true , false );
	}

	@Override
	public SQLDataType	getSingleSQLDataType()
	{
		return( new SQLDataType( "blob" , String.class ) );
	}

	@Override
	public	Pair<TypedInstance, Integer>
				createInstance( Element  element ,
						ElementName  topLevelElement ,
						SimpleXPath  path ,
						StrageManager  strage ,
						int  elementIndex )
							throws TypeException
	{
		//
		// debug print
		//
		System.err.println();
		System.err.println();
		System.err.println( this.getClass().getName()
				    + ": createInstance()" );
		System.err.println( "elementIndex = " + elementIndex );

		Element	data;
		try
		{
			data = ElementUtil.getSingleChildElement( element );
		}
		catch( XMLParseErrorException  e )
		{
			throw new TypeException( e );
		}

		ElementUtil.debugPrint( data , System.err );

		ByteArrayOutputStream	buf = new ByteArrayOutputStream();

		try
		{
			XMLFormatConverter.print( data , buf );
		}
		catch( TransformerException  e )
		{
			e.printStackTrace();

			throw new TypeException( "Can't convert XML to text" ,
						 e );
		}


		String	columnName;
		if ( path == null )
		{
			columnName = ElementColumnNameFactory
						    .getTopLevelColumnName();
		}
		else
		{
			SimpleXPath	newPath = new SimpleXPath( path );

			newPath.add( new UniversalName
				     ( element.getNamespaceURI() ,
				       element.getLocalName() ) );

			try
			{
				columnName
				    = strage.getShortXPathString( newPath );
			}
			catch( StrageException  e )
			{
				throw new TypeException( e );
			}
		}

		TableColumn	column;
		column = new TableColumn( new TableColumnDefinition
					  ( columnName ,
					    this.getSingleSQLDataType() ) ,
					  buf.toString() );

		List<TableColumn>  columnList = new ArrayList<TableColumn>();
		columnList.add( column );

		return( new Pair<TypedInstance, Integer>
			( new TypedInstance( columnList , this ) ,
			  elementIndex + 1 ) );
	}

	@Override
	public	int	convertToXMLElement( Element  element ,
					     Document  doc ,
					     StrageManager  strage ,
					     List<TableColumn>  columns ,
					     int  index ,
					     long  id )
							throws TypeException

	{
		if ( index >= columns.size() )
		{
			throw new TypeException
			  ( "internal error in " + this.getClass().getName()
			    + ": unexpected instantiation" );
		}

		String	value = columns.get( index ).getValue();

		ByteArrayInputStream
			buf = new ByteArrayInputStream( value.getBytes() );

		try
		{
			XMLFormatConverter.readNode( buf , element );
		}
		catch( TransformerException  e )
		{
			e.printStackTrace();

			throw new TypeException( "Can't convert text to XML" ,
						 e );
		}

		return( index + 1 );
	}
}
