package daruma.strage_manager.type_definition.types;

import daruma.strage_manager.type_definition.AbstractCompositorTypeDefinition;
import daruma.strage_manager.type_definition.TypeDefinition;
import daruma.strage_manager.type_definition.TypeName;
import daruma.strage_manager.type_definition.ElementName;
import daruma.strage_manager.type_definition.TypedInstance;
import daruma.strage_manager.type_definition.TypeException;
import daruma.strage_manager.type_definition.ElementColumnNameFactory;
import daruma.strage_manager.StrageManager;
import daruma.strage_manager.StrageException;

import daruma.sql.TableColumn;
import daruma.sql.TableColumnDefinition;
import daruma.sql.SQLDataType;
import daruma.xml.UniversalName;
import daruma.xml.SimpleXPath;
import daruma.util.Pair;

import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Element;
import org.w3c.dom.Document;

import java.util.List;
import java.util.ArrayList;
import java.util.Set;
import java.util.HashSet;

// XXX: for debug only
import daruma.xml.util.ElementUtil;


public class ChoiceTypeDefinition extends AbstractCompositorTypeDefinition
{
	public	ChoiceTypeDefinition()
	{
		super();
	}

	public List<TableColumnDefinition>
			  getCompositeSQLDataType( StrageManager  strage ,
						   SimpleXPath  path )
							throws TypeException
	{
		List<TableColumnDefinition>
			ret = new ArrayList<TableColumnDefinition>();


		String	indexColumnName;

		if ( path == null )
		{
			indexColumnName = ElementColumnNameFactory
						    .getTopLevelColumnName();
		}
		else
		{
			SimpleXPath	indexPath = new SimpleXPath( path );

			try
			{
				indexColumnName
				    = strage.getShortXPathString( indexPath );
			}
			catch( StrageException  e )
			{
				throw new TypeException( e );
			}
		}


		ret.add( new TableColumnDefinition
			 ( ElementColumnNameFactory
			   .getCoiceElementIndexColumnName( indexColumnName ) ,
			   new SQLDataType( "int" , Integer.class ) ) );


		List<TableColumnDefinition>
		 childColumns = super.getCompositeSQLDataType( strage , path );

		if ( ! this.checkDeterministic( childColumns ) )
		{
			throw new TypeException
				( "ambiguos model, must be deterministic" );
		}

		ret.addAll( childColumns );

		return( ret );
	}

	private	boolean	checkDeterministic
			( List<TableColumnDefinition>  columnDefinitions )
	{
		Set<String>	columnNameDictionary = new HashSet<String>();

		for ( TableColumnDefinition  def  : columnDefinitions )
		{
			String	c = def.getColumnName();

			if ( columnNameDictionary.contains( c ) )
			{
				return( false );
			}

			columnNameDictionary.add( c );
		}

		return( true );
	}


	public	Pair<TypedInstance, Integer>
				createInstance( Element  element ,
						ElementName  topLevelElement ,
						SimpleXPath  path ,
						StrageManager  strage ,
						int  elementIndex )
							  throws TypeException
	{
		System.err.println();
		System.err.println();
		System.err.println( this.getClass().getName()
				    + ": createInstance()" );
		System.err.println( "elementIndex = " + elementIndex );
		System.err.println( "--" );
		this.debugPrint();
		System.err.println( "--" );
		ElementUtil.debugPrint( element , System.err );



		List<TableColumn>  columnList = new ArrayList<TableColumn>();

		SimpleXPath	columnPathBase;

		if ( path != null )
		{
			columnPathBase = new SimpleXPath( path ,
					     new UniversalName( element ) );
		}
		else
		{
			columnPathBase = path;
		}



		List<TableColumnDefinition>
			indexColummns = this.getCompositeSQLDataType
						( strage ,
						  columnPathBase );
		TableColumn	choiceElementIndex
				= new TableColumn( indexColummns.get(0) ,
						   "0" );
		columnList.add( choiceElementIndex );


		NodeList	childNodes   = element.getChildNodes();
		int		nThChildNode = elementIndex;

		int	i;
		for( i = 0  ;  i < super.getEntries().size()  ;  i ++ )
		{
			Entry	entry  = super.getEntries().get(i);

			//
			// debug print
			//
			System.err.print( "next entry = " );

			if ( entry.isCompositeEntry() )
			{
				System.err.println
					( "composite, "
					  + entry.getCompositeEntry()
						  .getClass().getName() );
			}
			else
			{
				System.err.println( "atomic" );
			}


			//
			// if composite entry
			//
			if ( entry.isCompositeEntry() )
			{
				Pair<TypedInstance, Integer>	r;

				try
				{
					r = entry.getCompositeEntry()
						  .createInstance
						    ( element ,
						      topLevelElement ,
						      path ,
						      strage ,
						      nThChildNode );
				}
				catch( TypeException  e )
				{
					//
					// ignore this exeption
					//
					// check rest of next candidates

					continue;
				}

				columnList.addAll( r.getFirst().getColumns() );

				System.err.println
					( "nThChildNode updated"
					  + " from " + nThChildNode
					  + " to " + r.getSecond() );

				nThChildNode = r.getSecond();

				break;
			}


			//
			// if single entry
			//

			//
			// get next element
			//
			Node	n = null;

			for(;;)
			{
				if ( nThChildNode >= childNodes.getLength() )
				{
					System.err.println
						( "child index = "
						  + nThChildNode + ", "
						  + "number of child nodes = "
						  + childNodes.getLength() );

					throw new TypeException
					  ( "too few child elements of "
					    + new ElementName( element )
								 .toString() );
				}

				n = childNodes.item( nThChildNode );

				if ( n instanceof Element )
				{
					// XXX: check tag equals <annotation>

					break;
				}

				nThChildNode ++;
			}


			Element		childElement = (Element)n;
			ElementName	childElementName = new ElementName
							     ( childElement );

			System.err.println( "tag[" + nThChildNode + "] = ["
					    + childElementName.toString()
					    + "]" );

			ElementName	candidateElementName
					= entry.getAtomEntry().getSecond();

			System.err.println( "candidate element = ["
					    + candidateElementName + "]" );

			if ( ! childElementName
			       .equals( candidateElementName ) )
			{
				// challenge next candidate
				continue;
			}


			TypeName	typeName
					= entry.getAtomEntry().getFirst();

			System.err.println( "typeName = [" + typeName + "]" );

			TypeDefinition	type;
			try
			{
				type = strage.getTypeDefinition( typeName );
			}
			catch( StrageException  se )
			{
				throw new TypeException( se );
			}

			System.err.println( "type.class = "
					    + type.getClass().getName() );

			SimpleXPath	p = new SimpleXPath( path );

			if ( path != null )
			{
				p.add( new UniversalName( element ) );
			}

			TypedInstance	obj;
			try
			{
				obj = type.createInstance
					   ( childElement ,
					     topLevelElement ,
					     p ,
					     strage ,
					     0 ).getFirst();
			}
			catch( TypeException  e )
			{
				//
				// ignore this exeption
				//
				// check rest of next candidates

				continue;
			}

			nThChildNode ++;

			columnList.addAll( obj.getColumns() );

			break;
		}


		System.err.println();
		System.err.println();
		System.err.println( "!!!!! INDEX = " + i );
		System.err.println();
		System.err.println();

		columnList.get(0).setValue( Integer.toString(i) );

		System.err.println();
		System.err.println();

		return( new Pair<TypedInstance, Integer>
			( new TypedInstance( columnList , this ) ,
			  nThChildNode ) );
	}


	public int  convertToXMLElement( Element  element ,
					 Document  doc ,
					 StrageManager  strage ,
					 List<TableColumn>  columns ,
					 int  index ,
					 long  id ) throws TypeException
	{
		int	i = index;

		int	dataElementIndex
			= Integer.parseInt( columns.get(i).getValue() );
		i ++;

		int	elementIndex;
		for( elementIndex = 0  ;
		     elementIndex < this.getEntries().size()  ;
		     elementIndex ++ )
		{
			Entry	e = this.getEntries().get( elementIndex );

			boolean	isDataElement
				= (elementIndex == dataElementIndex);

			if ( e.isCompositeEntry() )
			{
				if ( isDataElement )
				{
					i = e.getCompositeEntry()
					    .convertToXMLElement( element ,
								  doc ,
								  strage ,
								  columns ,
								  i ,
								  id );
				}
				else
				{
					i += (e.getCompositeEntry()
						.getCompositeSQLDataType
							    ( strage, null )
						.size());
				}
			}

			Pair<TypeName, ElementName>	a = e.getAtomEntry();

			TypeName	childTypeName    = a.getFirst();
			ElementName	childElementName = a.getSecond();

			TypeDefinition	childType;
			try
			{
				childType = strage.getTypeDefinition
							( childTypeName );
			}
			catch( StrageException  se )
			{
				throw new TypeException( se );
			}

			if ( childType.getSingleSQLDataType() != null )
			{
				if ( isDataElement )
				{
					Element	child = doc.createElementNS
					  ( childElementName.getNamespace() ,
					    childElementName.getLocalName() );

					element.appendChild( child );

					child.appendChild( doc.createTextNode
							   ( columns.get(i)
							     .getValue() ) );
				}

				i ++;
			}
			else
			{
				if ( isDataElement )
				{
					Element	child = doc.createElementNS
					  ( childElementName.getNamespace() ,
					    childElementName.getLocalName() );

					element.appendChild( child );

					i = childType.convertToXMLElement
						( child ,
						  doc ,
						  strage ,
						  columns ,
						  i ,
						  id );
				}
				else
				{
					i += childType.getCompositeSQLDataType
							      ( strage, null )
							.size();
				}
			}
		}

		return( i );
	}
}
