package daruma.strage_manager.type_definition.types;

import daruma.strage_manager.type_definition.AbstractMultiColumnSingleTypeDefinition;
import daruma.strage_manager.type_definition.TypeException;
import daruma.strage_manager.type_definition.ElementColumnNameFactory;

import daruma.strage_manager.StrageManager;
import daruma.strage_manager.StrageException;
import daruma.xml.SimpleXPath;
import daruma.xml.util.WhitespaceProceccer;

import daruma.sql.TableColumnDefinition;
import daruma.sql.TableColumn;
import daruma.sql.SQLDataType;

import daruma.util.ISO8601DateFormat;
import java.sql.Timestamp;
import java.util.Date;
import java.text.ParseException;

import org.w3c.dom.Element;
import org.w3c.dom.Text;
import org.w3c.dom.Document;

import java.util.List;
import java.util.ArrayList;


public class DateTimeTypeDefinition
		    extends AbstractMultiColumnSingleTypeDefinition
{
	public	DateTimeTypeDefinition()
	{
		super();
	}

	@Override
	public	List<String>	parseValue( String  value )
							throws TypeException
	{
		String	v = WhitespaceProceccer.cutoffEdgeWhitespaces( value );

		ISO8601DateFormat	f = new ISO8601DateFormat();

		try
		{
			Date	d = f.parse( v );
		}
		catch( ParseException  e )
		{
			throw new TypeException
			( "\"" + value + "\" is not a valid dateTime value" );

		}


		List<String>	ret = new ArrayList<String>();
		ret.add( v );
		ret.add( v );

		return( ret );
	}

	@Override
	public	List<TableColumnDefinition>	getCompositeSQLDataType
						  ( StrageManager  strage ,
						    SimpleXPath  path )
							throws TypeException
	{
		List<TableColumnDefinition>
			ret = new ArrayList<TableColumnDefinition>();


		System.err.println( "In " + this.getClass().getName()
				    + ".getCompositeSQLDataType()" );
		System.err.println( "path = [" + path + "]" );



		// XXX: need code cleanning
		if ( path == null )
		{
			/*
			ret.add( new TableColumnDefinition
				 ( ElementColumnNameFactory
						    .getTopLevelColumnName(1) ,
				   new SQLDataType( "datetime" ,
						    Timestamp.class ) ) );
			*/
			ret.add( new TableColumnDefinition
				 ( ElementColumnNameFactory
						    .getTopLevelColumnName() ,
				   new SQLDataType( "datetime" ,
						    Timestamp.class ) ) );

			ret.add( new TableColumnDefinition
				 ( ElementColumnNameFactory
						    .getTopLevelColumnName(2) ,
				   new SQLDataType( "blob" ,
						    String.class ) ) );
		}
		else
		{
			SimpleXPath	newPath = new SimpleXPath( path );

			String	shortXPathString;
			try
			{
				shortXPathString
				    = strage.getShortXPathString( newPath );
			}
			catch( StrageException  e )
			{
				throw new TypeException( e );
			}

			/*
			ret.add( new TableColumnDefinition
				 ( ElementColumnNameFactory
				   .getColumnName( shortXPathString , 1 ) ,
				   new SQLDataType( "datetime" ,
						    Timestamp.class ) ) );
			*/
			ret.add( new TableColumnDefinition
				 ( shortXPathString ,
				   new SQLDataType( "datetime" ,
						    Timestamp.class ) ) );

			ret.add( new TableColumnDefinition
				 ( ElementColumnNameFactory
				   .getColumnName( shortXPathString , 2 ) ,
				   new SQLDataType( "blob" ,
						    String.class ) ) );
		}

		return( ret );
	}


	@Override
	public	int	convertToXMLElement( Element  element ,
					     Document  doc ,
					     StrageManager  strage ,
					     List<TableColumn>  columns ,
					     int  index ,
					     long  id )
						    throws TypeException
	{
		if ( index + 1 >= columns.size() )
		{
			throw new TypeException
			  ( "internal error in " + this.getClass().getName()
			    + ": unexpected instantiation" );
		}


		String	value = columns.get( index + 1 ).getValue();

		Text	text = doc.createTextNode( value );

		element.appendChild( text );

		return( index + 2 );
	}

	@Override
	public	String	getComparationColumnNameSuffix( String  shortXPath )
	{
		return( ElementColumnNameFactory.getColumnNameSuffix( 1 ) );
	}
}
