package daruma.strage_manager.type_definition.types;

import daruma.strage_manager.type_definition.TypeDefinition;
import daruma.strage_manager.type_definition.TypedInstance;
import daruma.strage_manager.type_definition.TypeException;
import daruma.strage_manager.type_definition.ElementColumnNameFactory;
import daruma.strage_manager.type_definition.ElementName;
import daruma.strage_manager.StrageManager;
import daruma.strage_manager.StrageException;
import daruma.sql.SQLDataType;
import daruma.sql.TableColumnDefinition;
import daruma.sql.TableColumn;
import daruma.xml.UniversalName;
import daruma.xml.SimpleXPath;
import daruma.xml.URI;
import daruma.xml.util.ElementUtil;
import daruma.xml.util.XMLParseErrorException;
import daruma.xml.util.XMLFormatConverter;
import daruma.util.Pair;
import daruma.geometry.Geometry;
import daruma.geometry.GeometryFormatConverter;
import daruma.geometry.GeometryException;

import org.w3c.dom.Element;
import org.w3c.dom.Text;
import org.w3c.dom.Document;

import javax.xml.transform.TransformerException;
import java.util.List;
import java.util.ArrayList;


public class GeometryPropertyTypeDefinition extends TypeDefinition
{
	private static final String COLUMN_GEO      = "_geo_";
	private static final String COLUMN_SRS_NAME = "_srs_name_";
	private static final String COLUMN_XML      = "_xml_";

	private static final int INDEX_GEO      = 0;
	private static final int INDEX_SRS_NAME = 1;
	private static final int INDEX_XML      = 2;

	private static final int NUMBER_OF_COLUMNS = 3;

	public	GeometryPropertyTypeDefinition()
	{
		super( true , true );
	}

	@Override
	public SQLDataType	getSingleSQLDataType()
	{
		return( null );
	}

	@Override
	public	List<TableColumnDefinition>	getCompositeSQLDataType
						  ( StrageManager  strage ,
						    SimpleXPath  path )
							throws TypeException
	{
		List<TableColumnDefinition>
			ret = new ArrayList<TableColumnDefinition>();

		// XXX: need code cleanning
		if ( path == null )
		{
			ret.add( new TableColumnDefinition
				 ( ElementColumnNameFactory
				   .getTopLevelColumnName( COLUMN_GEO ) ,
				   new SQLDataType( "geometry" ,
						    "AsText" ,
						    Geometry.class ) ) );
			ret.add( new TableColumnDefinition
				 ( ElementColumnNameFactory
				   .getTopLevelColumnName( COLUMN_SRS_NAME ) ,
				   new SQLDataType( "blob" ,
						    String.class ) ) );
			ret.add( new TableColumnDefinition
				 ( ElementColumnNameFactory
				   .getTopLevelColumnName( COLUMN_XML ) ,
				   new SQLDataType( "blob" ,
						    String.class ) ) );
		}
		else
		{
			SimpleXPath	newPath = new SimpleXPath( path );

			String	shortXPathString;
			try
			{
				shortXPathString
				    = strage.getShortXPathString( newPath );
			}
			catch( StrageException  e )
			{
				throw new TypeException( e );
			}

			ret.add( new TableColumnDefinition
				 ( ElementColumnNameFactory
				   .getColumnName( shortXPathString ,
						   COLUMN_GEO ) ,
				   new SQLDataType( "geometry" ,
						    "AsText" ,
						    Geometry.class ) ) );
			ret.add( new TableColumnDefinition
				 ( ElementColumnNameFactory
				   .getColumnName( shortXPathString ,
						   COLUMN_SRS_NAME ) ,
				   new SQLDataType( "blob" ,
						    String.class ) ) );
			ret.add( new TableColumnDefinition
				 ( ElementColumnNameFactory
				   .getColumnName( shortXPathString ,
						   COLUMN_XML ) ,
				   new SQLDataType( "blob" ,
						    String.class ) ) );
		}

		return( ret );
	}


	@Override
	public	Pair<TypedInstance, Integer>
				createInstance( Element  element ,
						ElementName  topLevelElement ,
						SimpleXPath  path ,
						StrageManager  strage ,
						int  elementIndex )
							throws TypeException
	{
		//
		// debug print
		//
		System.err.println();
		System.err.println();
		System.err.println( this.getClass().getName()
				    + ": createInstance()" );
		System.err.println( "elementIndex = " + elementIndex );

		ElementUtil.debugPrint( element , System.err );

		Element	geometryElement;

		try
		{
			geometryElement = ElementUtil.getSingleChildElement
								  ( element );
		}
		catch( XMLParseErrorException  e )
		{
			throw new TypeException( e.getMessage() , e );
		}


		ElementName	geometryTypeElementName
					= new ElementName( geometryElement );

		if ( ! geometryElement.getNamespaceURI().equals( URI.GML ) )
		{
			throw new TypeException( "unexpected geometry type \""
						 + geometryTypeElementName
						 + "\"" );
		}


		/*
		Element	childElement;

		try
		{
			childElement = ElementUtil.getSingleChildElement
							      ( geometryElement );
		}
		catch( XMLParseErrorException  e )
		{
			throw new TypeException( e.getMessage() , e );
		}
		*/


		String	wktValue;
		String	srsName;
		try
		{
			Geometry	geometry;

			geometry = GeometryFormatConverter
				   .parseFromGMLElement( geometryElement );

			wktValue = GeometryFormatConverter
				   .convertGeometryToWKTString( geometry );

			srsName = geometry.getSrsName();
		}
		catch( GeometryException  e )
		{
			throw new TypeException
				( "unexpected geometry type \""
				  + geometryTypeElementName + "\"" );
		}


		String	rawXMLString;

		try
		{
			rawXMLString = XMLFormatConverter
					.elementToString( geometryElement );
		}
		catch( TransformerException  e )
		{
			e.printStackTrace();

			throw new TypeException
				( "unexpected exception,"
				  + " Can't convert XML to string: "
				  + e.getMessage() , e );
		}


		//
		// create return value
		//
		SimpleXPath	p;
		if ( path == null )
		{
			p = null;
		}
		else
		{
			p = new SimpleXPath( path ,
					     new UniversalName( element ) );
		}

		List<TableColumnDefinition>
			defs = this.getCompositeSQLDataType( strage , p );

		List<TableColumn>	columns = new ArrayList<TableColumn>();

		columns.add( new TableColumn
			     ( defs.get( INDEX_GEO ) , wktValue ) );

		columns.add( new TableColumn
			     ( defs.get( INDEX_SRS_NAME ) ,
			       srsName ) );

		columns.add( new TableColumn
			     ( defs.get( INDEX_XML ) ,
			       rawXMLString ) );

		return( new Pair<TypedInstance, Integer>
			( new TypedInstance( columns , this ) ,
			  element.getChildNodes().getLength() ) );
	}


	@Override
	public	int	convertToXMLElement( Element  element ,
					     Document  doc ,
					     StrageManager  strage ,
					     List<TableColumn>  columns ,
					     int  index ,
					     long  id )
							throws TypeException

	{
		if ( index + NUMBER_OF_COLUMNS - 1 >= columns.size() )
		{
			throw new TypeException
			  ( "internal error in " + this.getClass().getName()
			    + ": unexpected instantiation" );
		}


		Element	geometryElement;
		try
		{
			geometryElement
				= XMLFormatConverter.stringToElement
				  ( columns.get( index + INDEX_XML )
								.getValue() ,
				    doc );
		}
		catch( TransformerException  e )
		{
			throw new TypeException
				( "unexpected exception,"
				  + " Can't convert string to XML: "
				  + e.getMessage() , e );
		}

		element.appendChild( geometryElement );

		return( index + NUMBER_OF_COLUMNS );
	}


	public	static	String	convertShortXPathToColumnName
						( String  shortXPathString )
	{
		return( ElementColumnNameFactory
			.getColumnName( shortXPathString ,
					COLUMN_GEO ) );
	}
}
