package daruma.strage_manager.type_definition.types;

import daruma.strage_manager.type_definition.TypeDefinition;
import daruma.strage_manager.type_definition.TypedInstance;
import daruma.strage_manager.type_definition.TypeException;
import daruma.strage_manager.type_definition.ElementName;
import daruma.strage_manager.type_definition.TypeName;
import daruma.strage_manager.StrageManager;
import daruma.strage_manager.StrageException;
import daruma.sql.SQLDataType;
import daruma.sql.TableColumnDefinition;
import daruma.sql.TableColumn;
import daruma.xml.UniversalName;
import daruma.xml.SimpleXPath;
import daruma.xml.util.ElementUtil;
import daruma.util.Pair;

import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.Text;
import org.w3c.dom.NodeList;
import org.w3c.dom.Document;

import java.util.List;
import java.util.ArrayList;


public class MultipleOccursElementTypeDefinition extends TypeDefinition
{
//	private	TypeName	typeName;
	private	ElementName	elementName;
	private	int		minOccurs;
	private	int		maxOccurs;

	public	MultipleOccursElementTypeDefinition( TypeName  typeName ,
						     ElementName  elementName ,
						     int  minOccurs ,
						     int  maxOccurs )
	{
		super( true , true );

		super.setIsSQLExternalTable( true );

//		this.typeName = typeName;
		this.elementName = elementName;
		this.minOccurs = minOccurs;
		this.maxOccurs = maxOccurs;
	}

	@Override
	public SQLDataType	getSingleSQLDataType()
	{
		return( null );
	}

	@Override
	public	List<TableColumnDefinition>	getCompositeSQLDataType
						  ( StrageManager  strage ,
						    SimpleXPath  path )
							throws TypeException
	{
		List<TableColumnDefinition>
			ret = new ArrayList<TableColumnDefinition>();

		SimpleXPath	newPath = new SimpleXPath( path ,
							   this.elementName );

		String	shortXPathString;
		try
		{
			shortXPathString
			    = strage.getShortXPathString( newPath );
		}
		catch( StrageException  e )
		{
			throw new TypeException( e );
		}

		TableColumnDefinition	c = new TableColumnDefinition
						( shortXPathString ,
						  new SQLDataType
						  ( "blob" ,
						    String.class ) );
		c.setIsExternalTable( true );
		ret.add( c );

		return( ret );
	}


	@Override
	public	Pair<TypedInstance, Integer>
				createInstance( Element  element ,
						ElementName  topLevelElement ,
						SimpleXPath  path ,
						StrageManager  strage ,
						int  elementIndex )
							throws TypeException
	{
		//
		// debug print
		//
		System.err.println();
		System.err.println();
		System.err.println( this.getClass().getName()
				    + ": createInstance()" );
		System.err.println( "elementIndex = " + elementIndex );

		ElementUtil.debugPrint( element , System.err );


		NodeList	childNodes = element.getChildNodes();
		int		nThChildNode = elementIndex;

		int		nOccurs = 0;

		List<String>	values = new ArrayList<String>();

		while( nOccurs <= this.maxOccurs )
		{
			Node	n = null;

			while( nThChildNode < childNodes.getLength() )
			{
				n = childNodes.item( nThChildNode );

				nThChildNode ++;

				if ( n instanceof Element )
				{
					// XXX: check tag equals <annotation>

					break;
				}
			}

			System.err.println( "nThChildNode = " + nThChildNode );

			if ( nThChildNode >= childNodes.getLength() )
			{
				break;
			}

			Element		e = (Element)n;
			ElementName	elementName = new ElementName( e );

			System.err.println( "elementName = "
					    + elementName );
			System.err.println( "this.elementName = "
					    + this.elementName );

			if ( elementName.equals( this.elementName ) )
			{
				System.err.println( "element mached!!" );

				nOccurs ++;

				values.add( ElementUtil
					    .getChildNodesWholeText( e ) );
			}
			else
			{
				nThChildNode --;

				System.err.println( "element unmached!!" );

				break;
			}
		}

		System.err.println( "nOccurs = " + nOccurs );

		if ( nOccurs < this.minOccurs )
		{
			throw new TypeException
				( "too few element,"
				  + " expected at least "
				  + this.minOccurs
				  + " elements of " + this.elementName );
		}
		else if ( nOccurs > this.maxOccurs )
		{
			throw new TypeException
				( "too many element,"
				  + " expected up to "
				  + this.maxOccurs
				  + " elements of " + this.elementName );
		}


		//
		// create return value
		//
		SimpleXPath	p;
		if ( path == null )
		{
			p = null;
		}
		else
		{
			p = new SimpleXPath( path ,
					     new UniversalName( element ) );
		}

		List<TableColumnDefinition>
			defs = this.getCompositeSQLDataType( strage , p );

		List<TableColumn>	columns = new ArrayList<TableColumn>();

		SimpleXPath	tableNamePath = new SimpleXPath
						( topLevelElement ,
						  new SimpleXPath
						  ( path ,
						    this.elementName ) );

		String	tableName;
		try
		{
			tableName = strage.getShortXPathString
							( tableNamePath );
		}
		catch( StrageException  e )
		{
			throw new TypeException
				( "internal error: "
				  + "cannot convert element "
				  + topLevelElement.toString()
				  + " to ID: " + e.getMessage() ,
				  e );
		}

		TableColumn	c = new TableColumn( defs.get(0) , tableName );
		c.setSubValues( values );

		columns.add( c );

		return( new Pair<TypedInstance, Integer>
			( new TypedInstance( columns , this ) ,
			  nThChildNode ) );
	}


	@Override
	public	int	convertToXMLElement( Element  element ,
					     Document  doc ,
					     StrageManager  strage ,
					     List<TableColumn>  columns ,
					     int  index ,
					     long  id )
							throws TypeException
	{
		System.err.println( this.getClass().getName()
				    + ".convertToXMLElement()" );

		if ( index >= columns.size() )
		{
			throw new TypeException
			  ( "internal error in " + this.getClass().getName()
			    + ": unexpected instantiation" );
		}

		TableColumn	c = columns.get( index );

		String	tableName = c.getValue();

		List<String>	values;

		try
		{
			values = strage.getSubTableValues( tableName , id );
		}
		catch( StrageException  e )
		{
			throw new TypeException( e.getMessage() , e );
		}

		for ( String  v  :  values )
		{
			Element	e = doc.createElementNS
					( this.elementName.getNamespace() ,
					  this.elementName.getLocalName() );

			element.appendChild( e );

			e.appendChild( doc.createTextNode( v ) );
		}

		return( index + 1 );
	}
}
