package daruma.strage_manager.type_definition.types;

import daruma.strage_manager.type_definition.AbstractCompositorTypeDefinition;
import daruma.strage_manager.type_definition.TypeDefinition;
import daruma.strage_manager.type_definition.TypeName;
import daruma.strage_manager.type_definition.ElementName;
import daruma.strage_manager.type_definition.TypedInstance;
import daruma.strage_manager.type_definition.TypeException;
import daruma.strage_manager.StrageManager;
import daruma.strage_manager.StrageException;
import daruma.sql.TableColumn;
import daruma.xml.UniversalName;
import daruma.xml.SimpleXPath;
import daruma.util.Pair;

import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Element;

import java.util.List;
import java.util.ArrayList;

// XXX: for debug only
import daruma.xml.util.ElementUtil;


public class SequenceTypeDefinition extends AbstractCompositorTypeDefinition
{
	public	SequenceTypeDefinition()
	{
		super();
	}

	public	Pair<TypedInstance, Integer>
				createInstance( Element  element ,
						ElementName  topLevelElement ,
						SimpleXPath  path ,
						StrageManager  strage ,
						int  elementIndex )
							  throws TypeException
	{
		System.err.println();
		System.err.println();
		System.err.println( this.getClass().getName()
				    + ": createInstance()" );
		System.err.println( "elementIndex = " + elementIndex );
		System.err.println( "--" );
		this.debugPrint();
		System.err.println( "--" );
		ElementUtil.debugPrint( element , System.err );


		List<TableColumn>  columnList = new ArrayList<TableColumn>();

		NodeList	childNodes   = element.getChildNodes();
		int		nThChildNode = elementIndex;

		for( Entry  entry  :  super.getEntries() )
		{
			//
			// debug print
			//
			System.err.print( "next entry = " );

			if ( entry.isCompositeEntry() )
			{
				System.err.println
					( "composite, "
					  + entry.getCompositeEntry()
						  .getClass().getName() );
			}
			else
			{
				System.err.println( "atomic" );
			}


			//
			// if composite entry
			//
			if ( entry.isCompositeEntry() )
			{
				Pair<TypedInstance, Integer>
				  r = entry.getCompositeEntry().createInstance
				      ( element ,
					topLevelElement ,
					path ,
					strage ,
					nThChildNode );

				columnList.addAll( r.getFirst().getColumns() );

				System.err.println
					( "nThChildNode updated"
					  + " from " + nThChildNode
					  + " to " + r.getSecond() );

				nThChildNode = r.getSecond();

				continue;
			}


			//
			// if single entry
			//
			Node	n = null;

			for(;;)
			{
				if ( nThChildNode >= childNodes.getLength() )
				{
					System.err.println
						( "child index = "
						  + nThChildNode + ", "
						  + "number of child nodes = "
						  + childNodes.getLength() );

					throw new TypeException
					  ( "too few child elements of "
					    + new ElementName( element )
								 .toString() );
				}

				n = childNodes.item( nThChildNode );

				if ( n instanceof Element )
				{
					// XXX: check tag equals <annotation>

					break;
				}

				nThChildNode ++;
			}


			Element		childElement = (Element)n;
			ElementName	childElementName = new ElementName
							     ( childElement );

			System.err.println( "tag[" + nThChildNode + "] = ["
					    + childElementName.toString()
					    + "]" );

			ElementName	expectedElementName
					= entry.getAtomEntry().getSecond();

			System.err.println( "expected element = ["
					    + expectedElementName + "]" );

			if ( ! childElementName.equals( expectedElementName ) )
			{
				throw new TypeException
					  ( "unexpected element \""
					    + childElementName
					    + "\", expected was "
					    + expectedElementName );
			}


			TypeName	typeName
					= entry.getAtomEntry().getFirst();

			System.err.println( "typeName = [" + typeName + "]" );

			TypeDefinition	type;
			try
			{
				type = strage.getTypeDefinition( typeName );
			}
			catch( StrageException  se )
			{
				throw new TypeException( se );
			}

			System.err.println( "type.class = "
					    + type.getClass().getName() );

			SimpleXPath	p = new SimpleXPath( path );

			if ( path != null )
			{
				p.add( new UniversalName( element ) );
			}


			TypedInstance	obj = type.createInstance
						   ( childElement ,
						     topLevelElement ,
						     p ,
						     strage ,
						     0 ).getFirst();

			nThChildNode ++;

			columnList.addAll( obj.getColumns() );
		}

		System.err.println();
		System.err.println();

		return( new Pair<TypedInstance, Integer>
			( new TypedInstance( columnList , this ) ,
			  nThChildNode ) );
	}
}
