package daruma.util;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

public class TeeInputStream extends InputStream
{
	private	InputStream	in;
	private	OutputStream	out;
	private boolean		closeOutput = true;

	public	TeeInputStream( InputStream  in ,  OutputStream  out )
	{
		this.in  = in;
		this.out = out;
	}

	public	TeeInputStream( InputStream  in ,  OutputStream  out ,
				boolean  closeOutput )
	{
		this.in  = in;
		this.out = out;
		this.closeOutput = closeOutput;
	}

	public	int	read() throws IOException
	{
		if ( in == null )
		{
			return( -1 );
		}

		int	ret = this.in.read();

		if ( out != null )
		{
			if ( ret != -1 )
			{
				out.write( ret );
			}
			else
			{
				if ( this.closeOutput )
				{
					out.close();
				}
			}
		}

		return( ret );
	}

	public	int	read( byte[]  b ,  int  off ,  int  len )
							throws IOException
	{
		if ( in == null )
		{
			return( -1 );
		}

		int	ret = this.in.read( b , off , len );

		if ( out != null )
		{
			if ( ret != -1 )
			{
				out.write( b , off , ret );
			}
			else
			{
				if ( this.closeOutput )
				{
					out.close();
				}
			}
		}

		return( ret );
	}

	public	void	flush() throws IOException
	{
		if ( out != null )
		{
			out.flush();
		}
	}

	public	long	skip( long  n ) throws IOException
	{
		if ( in != null )
		{
			return( in.skip( n ) );
		}
		else
		{
			return( 0 );
		}
	}

	public	int	available() throws IOException
	{
		if ( in != null )
		{
			return( in.available() );
		}
		else
		{
			return( 0 );
		}
	}

	public	void	close() throws IOException
	{
		if ( in != null )
		{
			in.close();
		}

		if ( out != null && this.closeOutput )
		{
			out.close();
		}
	}

	public	void	mark( int  readlimit )
	{
		if ( in != null )
		{
			in.mark( readlimit );
		}
	}

	public	void	reset() throws IOException
	{
		if ( in != null )
		{
			in.reset();
		}
	}

	public	boolean	markSupported()
	{
		if ( in != null )
		{
			return( in.markSupported() );
		}
		else
		{
			return( false );
		}
	}
}
