package daruma.wfs;

import daruma.xml.handler.XSAXDefaultHandler;
import daruma.xml.XMLTag;
import daruma.xml.URI;
import daruma.xml.UniversalName;
import daruma.xml.util.XMLFormatConverter;
import daruma.wfs.SOAPFaultDocumentBuilder;

import daruma.strage_manager.StrageManager;
import daruma.strage_manager.StrageException;
import daruma.strage_manager.type_definition.ElementName;
import daruma.strage_manager.type_definition.TypeName;

import org.xml.sax.XMLReader;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.Attributes;
import org.w3c.dom.Document;
import javax.xml.transform.TransformerException;
import javax.xml.parsers.ParserConfigurationException;

import java.io.OutputStream;
import java.io.PrintWriter;
import java.io.StringReader;

import java.util.List;
import java.util.ArrayList;


public class DescribeFeatureTypeHandler extends XSAXDefaultHandler
{
	private	StrageManager	strage;

	private	List<UniversalName>	typeList;

	private	boolean		readingTypeName;
	private	StringBuilder	typeNameBuffer;


	public	DescribeFeatureTypeHandler( OutputStream  out ,
					    XMLReader  parser ,
					    boolean  isTopLevelHandler ,
					    StrageManager  strage )
	{
		super( out , parser , isTopLevelHandler );

		this.strage = strage;

		this.typeList = new ArrayList<UniversalName>();

		this.readingTypeName = false;
		this.typeNameBuffer = null;
	}


	public	void	xStartElement( String uri ,
				       String localName ,
				       String qName ,
				       Attributes  attrs ) throws SAXException
	{
		assert super.getCurrentLevel() == 1
			|| super.getCurrentLevel() == 2;

		XMLTag	t = new XMLTag( uri , localName );

		final SAXParseException	mispNamespaceException
					 = new SAXParseException
					       ( t.getLocalName() + " tag "
						 + "should be in namespace "
						 + URI.MISP ,
						 super.getLocator() );

		// <DescribeFeatureType>
		if ( super.getCurrentLevel() == 1 )
		{
			if ( ! t.getURI().equals( URI.MISP ) )
			{
				this.throwError( mispNamespaceException );
			}

			if ( ! t.getLocalName()
				 .equals( "DescribeFeatureType" ) )
			{
				this.throwError( new SAXParseException
						 ( "unknown tag "
						   + t.getLocalName() + "." ,
						   super.getLocator() ) );

			}

			return;
		}
		// <DescribeFeatureType>/<TypeName>
		else if ( super.getCurrentLevel() == 2 )
		{
			if ( ! t.getURI().equals( URI.MISP ) )
			{
				this.throwError( mispNamespaceException );
			}

			if ( ! t.getLocalName().equals( "TypeName" ) )
			{
				this.throwError( new SAXParseException
						 ( "unexpected tag \""
						   + t.getLocalName()
						   + "\" found, "
						   + "TypeName tag expected." ,
						   super.getLocator() ) );
			}

			this.readingTypeName = true;
			this.typeNameBuffer = new StringBuilder();
		}
	}


	public void  xCharacters( char[]  str ,  int  offset ,  int  length )
							    throws SAXException
	{
		if ( this.readingTypeName )
		{
			this.typeNameBuffer.append( str , offset , length );
		}
	}


	public	void	xEndElement( String uri ,
				     String localName ,
				     String qName ) throws SAXException
	{
		if ( this.readingTypeName )
		{
			try
			{
				this.typeList.add
				( super.convertQNameStringToUniversalName
				  ( this.typeNameBuffer.toString() ) );
			}
			catch( SAXException  e )
			{
				this.throwError
					( new SAXParseException
					  ( "type definition of \""
					    + typeNameBuffer.toString()
					    + "\" not found"
					    + ": " + e.getMessage() ,
					    super.getLocator() ) );
			}

			this.readingTypeName = false;
			this.typeNameBuffer = null;
		}
	}

	public	void  xEndDocument() throws SAXException
	{
		List<String>	allSchemas = new ArrayList<String>();

		for( UniversalName  typeName : this.typeList )
		{
			try
			{
				List<String>	schemas;

				schemas = this.strage.getSchemasOfElement
					    ( new ElementName( typeName ) );

				allSchemas.addAll( schemas );
			}
			catch( StrageException  e )
			{
				this.throwError
					( new SAXParseException
					  ( "type definition of "
					    + typeName + " not found" ,
					    super.getLocator() ) );
			}
		}



		PrintWriter	out = super.getPrintWriter();

		out.println( "<misp:DescribeFeatureTypeResponse"
			     + " xmlns:misp=\"" + URI.MISP + "\">" );
		out.flush();

		try
		{
			this.getOutputStream().flush();
		}
		catch( java.io.IOException  e )
		{
			e.printStackTrace();

			throw new SAXException( e );
		}


		for( UniversalName  typeName : this.typeList )
		{
			for ( String  s : allSchemas )
			{
				out.println();
				out.println( "<!-- ========== "
					     + typeName.getLocalName()
					     + " in namespace "
					     + typeName.getNamespace() + "'s"
					     + " definition"
					     + " ========== -->" );

				try
				{
					XMLFormatConverter
					  .printTextWithoutXMLDeclaration
					     ( new StringReader( s ) , out );
				}
				catch( TransformerException e )
				{
					e.printStackTrace();
				}
			}
		}

		if ( allSchemas.size() != 0 )
		{
			out.println();
		}

		out.println( "</misp:DescribeFeatureTypeResponse>" );
		out.flush();
	}



	public	void	throwError( SAXParseException  e ) throws SAXException
	{
		Document	doc;

		try
		{
			XMLFormatConverter
				.print( new SOAPFaultDocumentBuilder(e)
							      .newDocument() ,
					this.getOutputStream() );
		}
		catch( ParserConfigurationException  pe )
		{
			throw new SAXException( pe );
		}
		catch( TransformerException  te )
		{
			throw new SAXException( te );
		}

		throw e;
	}
}
