package daruma.wfs;

import daruma.xml.handler.XSAXDefaultHandler;
import daruma.xml.util.ErrorDocumentBuilder;
import daruma.xml.util.FatalErrorDocument;
import daruma.xml.util.XMLFormatConverter;
import daruma.strage_manager.type_definition.ElementName;
import daruma.xml.URI;

import daruma.strage_manager.StrageManager;
import daruma.strage_manager.StrageException;

import org.xml.sax.XMLReader;
import javax.xml.transform.TransformerException;
import javax.xml.parsers.ParserConfigurationException;

import java.io.OutputStream;
import java.io.PrintWriter;
import java.util.List;


public class GetCapabilitiesHandler extends XSAXDefaultHandler
{
	private	StrageManager	strage;

	public	GetCapabilitiesHandler( OutputStream  out ,
					XMLReader  parser ,
					boolean  isTopLevelHandler ,
					StrageManager  strage )
	{
		super( out , parser , isTopLevelHandler );

		this.strage = strage;
	}


	public  void  xEndDocument()
	{
		List<ElementName>	featureTypeNameList;

		PrintWriter	out = super.getPrintWriter();

		try
		{
			featureTypeNameList = strage.getElementList();
		}
		catch( StrageException  e )
		{
			try
			{
				XMLFormatConverter.print
					( ErrorDocumentBuilder.create
					  ( "error: " + e.getMessage() ,
					    super.getLocator() ) ,
					  super.getOutputStream() );
			}
			catch( ParserConfigurationException  pe )
			{
				pe.printStackTrace();

				out.print( new FatalErrorDocument
						( pe.getMessage() ,
						  super.getLocator() )
					   .getString() );
			}
			catch( TransformerException  te )
			{
				te.printStackTrace();

				out.print( new FatalErrorDocument
						( te.getMessage() ,
						  super.getLocator() )
					   .getString() );
			}

			out.flush();

			return;
		}



		out.println( "<GetCapabilitiesResponse"
			     + " xmlns=\"" + URI.MISP + "\">" );
		out.println( "" );
		out.println( "  <WFS_Capabilities>" );
		out.println( "" );
		out.println( "    <Capabilities>" );
		out.println( "      <GetCapabilities/>" );
		out.println( "      <DescribeFeatureType/>" );
		out.println( "      <Transaction/>" );
		out.println( "      <GetFeature/>" );
		out.println( "      <RegisterFeatureType/>" );
		out.println( "    </Capabilities>" );
		out.println( "" );
		out.println( "    <FeatureTypeList>" );
		out.println( "      <Operations>" );
		out.println( "        <Insert/>" );
		out.println( "        <Update/>" );
		out.println( "        <Delete/>" );
		out.println( "        <Query/>" );
		out.println( "      </Operations>" );
		out.println( "" );

		for ( ElementName  e : featureTypeNameList )
		{
			out.println( "      <FeatureType>" );

			out.println( "        "
				     + "<Name xmlns:ns=\"" + e.getNamespace()
				     + "\">"
				     + "ns:" + e.getLocalName()
				     + "</Name>" );

			out.println( "      </FeatureType>" );
		}

		out.println( "    </FeatureTypeList>" );
		out.println( "" );
		out.println( "  </WFS_Capabilities>" );
		out.println( "" );
		out.println( "</GetCapabilitiesResponse>" );
		out.flush();
	}
}
