package daruma.wfs;

import daruma.wfs.SAXExceptionObserver;
import daruma.wfs.SOAPFaultDocumentBuilder;
import daruma.wfs.filter.FilterHandler;

import daruma.xml.handler.XSAXDefaultHandler;
import daruma.xml.XMLTag;
import daruma.xml.URI;
import daruma.xml.util.XMLFormatConverter;

import daruma.strage_manager.StrageManager;
import daruma.strage_manager.StrageException;
import daruma.strage_manager.type_definition.ElementName;
import daruma.strage_manager.type_definition.TypeName;
import daruma.strage_manager.type_definition.TypeDefinition;
import daruma.strage_manager.type_definition.TypedInstance;
import daruma.strage_manager.type_definition.TypedInstanceSet;
import daruma.strage_manager.type_definition.TypeException;

import daruma.util.ISO8601DateFormat;

import org.xml.sax.XMLReader;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.Attributes;
import org.w3c.dom.Document;
import javax.xml.transform.TransformerException;
import javax.xml.parsers.ParserConfigurationException;

import java.io.OutputStream;
import java.io.PrintWriter;
import java.util.Date;


public class GetFeatureHandler extends XSAXDefaultHandler
			       implements SAXExceptionObserver
{
	private	StrageManager	strage;

	private	Date		startTime;
	private	Date		endTime;
	private	ElementName	specifiedElement;
	private	TypeDefinition	specifiedElementType;
	private	boolean		filterTagFound;
	private	boolean		queryTagFound;
	private	long		tagCount;

	private	FilterHandler	filter;

	/*
	private	static	String	MODE_NORMAL = "normal";
	private	static	String	MODE_COUNT = "count";
	private	static	String	MODE_BBOX = "bbox";
	*/


	public	GetFeatureHandler( OutputStream  out ,
				   XMLReader  parser ,
				   boolean  isTopLevelHandler ,
				   StrageManager  strage )
	{
		super( out , parser , isTopLevelHandler );

		this.strage               = strage;
		this.startTime            = null;
		this.endTime              = null;
		this.specifiedElement     = null;
		this.specifiedElementType = null;
		this.queryTagFound        = false;
		this.filterTagFound       = false;
		this.tagCount             = 0;
	}


	public	void	xStartElement( String uri ,
				       String localName ,
				       String qName ,
				       Attributes  attrs ) throws SAXException
	{
		assert super.getCurrentLevel() == 1
			|| super.getCurrentLevel() == 2
			|| super.getCurrentLevel() == 3;

		XMLTag	t = new XMLTag( uri , localName );
		this.tagCount ++;

		final SAXParseException	mispNamespaceException
					 = new SAXParseException
					       ( t.getLocalName() + " tag "
						 + "should be in namespace "
						 + URI.MISP ,
						 super.getLocator() );

		// <GetFeature>
		if ( super.getCurrentLevel() == 1 )
		{
			if ( ! t.getURI().equals( URI.MISP ) )
			{
				this.throwError( mispNamespaceException );

				throw mispNamespaceException;
			}

			if ( ! t.getLocalName().equals( "GetFeature" ) )
			{
				this.throwError( new SAXParseException
						 ( "unknown tag "
						   + t.getLocalName() + "." ,
						   super.getLocator() ) );

			}


			try
			{
				this.startTime = this.strage.getCurrentTime();
			}
			catch( StrageException  e )
			{
				this.throwError( new SAXParseException
						 ( e.getMessage() ,
						   super.getLocator() ,
						   e ) );
			}

			return;
		}
		// <GetFeature>/<Query>
		else if ( super.getCurrentLevel() == 2 )
		{
			if ( ! t.getURI().equals( URI.MISP ) )
			{
				this.throwError( mispNamespaceException );

				throw mispNamespaceException;
			}

			if ( ! t.getLocalName().equals( "Query" ) )
			{
				this.throwError( new SAXParseException
						 ( "unexpected tag \""
						   + t.getLocalName()
						   + "\" found, "
						   + "Query tag expected." ,
						   super.getLocator() ) );
			}

			this.queryTagFound = true;

			String	typeNameString = attrs.getValue
							( "" , "typeName" );

			if ( typeNameString == null )
			{
				this.throwError( new SAXParseException
						 ( "typeName attribute"
						   + " not found in "
						   + t.getLocalName()
						   + " tag." ,
						   super.getLocator() ) );
			}


			try
			{
				this.specifiedElement
				 = new ElementName
				   ( super.convertQNameStringToUniversalName
							  ( typeNameString ) );
			}
			catch( SAXException  e )
			{
				this.throwError( new SAXParseException
						 ( e.getMessage() ,
						   super.getLocator() ) );
			}


			// XXX: mode attribute


			TypeName	elementTypeName = null;

			try
			{
				elementTypeName
					= this.strage.getElementTypeName
							( specifiedElement );
			}
			catch( StrageException  e )
			{
				this.throwError
				     ( new SAXParseException
				       ( e.getMessage() ,
					 super.getLocator() ,
					 e ) );
			}

			if ( elementTypeName == null )
			{
				this.throwError
				     ( new SAXParseException
				       ( "element "
					 + specifiedElement.getLocalName()
					 + " in namespace "
					 + specifiedElement.getNamespace()
					 + " not registerd" ,
					 super.getLocator() ) );
			}

			try
			{
				this.specifiedElementType
					= this.strage.getTypeDefinition
							( elementTypeName );
			}
			catch( StrageException  e )
			{
				this.throwError
				     ( new SAXParseException
				       ( "type of element "
					 + specifiedElement.getLocalName()
					 + " in namespace "
					 + specifiedElement.getNamespace()
					 + " not registerd" ,
					 super.getLocator() ) );
			}

			return;
		}
		// <GetFeature>/<Query>/<Filter>
		else if ( super.getCurrentLevel() == 3 )
		{
			if ( ! t.getURI().equals( URI.MISP ) )
			{
				this.throwError( mispNamespaceException );

				throw mispNamespaceException;
			}

			if ( ! t.getLocalName().equals( "Filter" ) )
			{
				SAXParseException
					e = new SAXParseException
						( "unexpected tag \""
						  + t.getLocalName()
						  + "\" found, "
						  + "Filter tag expected ." ,
						  super.getLocator() );

				this.throwError( e );
			}


			this.filterTagFound = true;

			this.filter = new FilterHandler
					  ( super.getOutputStream() ,
					    super.getParser() ,
					    false ,
					    this.specifiedElementType ,
					    this.strage ,
					    this );

			super.setContentHandlerDelegator
				( this.filter ,
				  uri , localName , qName , attrs );

			return;
		}
	}


	public	void  xEndDocument() throws SAXException
	{
		if ( ! this.filterTagFound )
		{
			this.throwError( new SAXParseException
					 ( "Filter tag not found" ,
					   super.getLocator() ) );
		}

		if ( ! this.queryTagFound )
		{
			this.throwError( new SAXParseException
					 ( "Query tag not found" ,
					   super.getLocator() ) );
		}

		if ( this.specifiedElement == null )
		{
			this.throwError( new SAXParseException
					 ( "element not specified" ,
					   super.getLocator() ) );
		}


		if ( this.tagCount < 3 )
		{
			this.throwError( new SAXParseException
					 ( "too few tags found" ,
					   super.getLocator() ) );
		}

		if ( this.tagCount > 3 )
		{
			this.throwError( new SAXParseException
					 ( "too many tags found" ,
					   super.getLocator() ) );
		}


		try
		{
			this.endTime = this.strage.getCurrentTime();
		}
		catch( StrageException  e )
		{
			this.throwError( new SAXParseException
					 ( e.getMessage() ,
					   super.getLocator() ,
					   e ) );
		}


		//
		// main process of this class
		//
		this.outputResult();
	}


	public	void	outputResult() throws SAXException
	{
		PrintWriter	out = super.getPrintWriter();

		out.println( "<misp:GetFeatureResponse"
			     + " xmlns:misp=\"" + URI.MISP + "\""
			     + " xmlns:gml=\"" + URI.GML + "\">" );

		out.println( "  <misp:ElapsedTime>" );
		out.println( "    <gml:beginPosition>"
				    + new ISO8601DateFormat()
						.format(this.startTime)
			       + "</gml:beginPosition>" );
		out.println( "    <gml:endPosition>"
				    + new ISO8601DateFormat()
						.format(this.endTime)
			       + "</gml:endPosition>" );

		out.println( "  </misp:ElapsedTime>" );

		out.println( "  <misp:FeatureCollection>" );
		out.flush();

		try
		{
			this.getOutputStream().flush();
		}
		catch( java.io.IOException  e )
		{
			e.printStackTrace();

			throw new SAXException( e );
		}

		TypedInstanceSet	objectSet = null;

		try
		{
			try
			{
				objectSet = strage.getTypedInstanceSet
					    ( this.specifiedElement ,
					      this.filter.getSQLExpression() );
			}
			catch( StrageException  e )
			{
				this.throwError( new SAXParseException
						 ( "Can't retrieve objects: "
						   + e.getMessage() ,
						   super.getLocator() , e ) );
			}

			TypedInstance	obj = null;
			while ( (obj = objectSet.getNextTypedInstance())
								    != null )
			{
				out.println( "    <gml:featureMember>" );
				out.flush();
				this.getOutputStream().flush();

				try
				{
					XMLFormatConverter.printElement
					  ( obj.convertToXMLElement
						    ( this.specifiedElement ,
						      this.strage ) ,
					    this.getOutputStream() );
				}
				catch( TransformerException  te )
				{
					this.throwError
					     ( new SAXParseException
					       ( te.getMessage() ,
						 super.getLocator() , te ) );
				}

				out.println( "    </gml:featureMember>" );
				out.flush();
				this.getOutputStream().flush();
			}
		}
		catch( TypeException  e )
		{
			this.throwError( new SAXParseException
					 ( e.getMessage() ,
					   super.getLocator() ,
					   e ) );
		}
		catch( java.io.IOException  e )
		{
			e.printStackTrace();

			throw new SAXException( e );
		}
		finally
		{
			try
			{
				if ( objectSet != null )
				{
					objectSet.close();
				}
			}
			catch( TypeException  e )
			{
				this.throwError( new SAXParseException
						 ( e.getMessage() ,
						   super.getLocator() ,
						   e ) );
			}
		}

		out.println( "  </misp:FeatureCollection>" );
		out.println( "</misp:GetFeatureResponse>" );
		out.flush();
	}



	public	void	notifyError( SAXParseException  e ) throws SAXException
	{
		this.throwError( e );
	}

	public	void	throwError( SAXParseException  e ) throws SAXException
	{
		Document	doc;

		try
		{
			XMLFormatConverter
				.print( new SOAPFaultDocumentBuilder(e)
							      .newDocument() ,
					this.getOutputStream() );
		}
		catch( ParserConfigurationException  pe )
		{
			throw new SAXException( pe );
		}
		catch( TransformerException  te )
		{
			throw new SAXException( te );
		}

		throw e;
	}
}
