package daruma.wfs;

import daruma.xml.handler.XSAXDefaultHandler;
import daruma.xml.handler.XSAXDOMCreateHandler;
import daruma.xml.XMLTag;
import daruma.xml.URI;
import daruma.util.ISO8601DateFormat;

import daruma.strage_manager.StrageManager;
import daruma.strage_manager.StrageException;

import org.w3c.dom.Document;

import org.xml.sax.XMLReader;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

import java.io.PrintWriter;
import java.io.OutputStream;
import java.util.Date;



public class  RegisterFeatureTypeHandler extends XSAXDefaultHandler
{
	//
	// local class XMLSchemaRegistrationHandler
	//
	//   This class reads XML Schema, and register it to strage.
	//
	//   implementation:
	//     This class inherits XSAXDOMCreateHandler it converts SAX to DOM.
	//     In xEndDocument(), this class gets DOM document from super class
	//     and passes to processDocument().
	//
	private  static class  XMLSchemaRegistrationHandler
						 extends XSAXDOMCreateHandler
	{
		private	StrageManager			strage;
		private	RegisterFeatureTypeHandler	parent;

		public	XMLSchemaRegistrationHandler
				( OutputStream  out ,
				  XMLReader  parser ,
				  boolean  isTopLevelHandler ,
				  StrageManager  strage ,
				  RegisterFeatureTypeHandler  parent )
		{
			super( out , parser , isTopLevelHandler );
			super.setInhibitEndPrefixBeforeEndDocument( true );

			this.strage = strage;
			this.parent = parent;
		}

		public  void  xEndDocument() throws SAXException
		{
			try
			{
				this.strage.registerSchema
					( super.getDOMDocument()
						  .getDocumentElement() ,
					  super.getPrefixMap() );
			}
			catch( StrageException  e )
			{
				System.err.println( "errorMessage=["
						    + e.getMessage() + "]" );

				this.parent.setError( e.getMessage() );
			}
		}
	}



	//
	// main class RegisterFeatureTypeHandler
	//
	private	StrageManager	strage;

	private	boolean		errorOccured;
	// XXX: List<String>	errorMessages;
	private	String		errorMessage;

	private	Date		startTime;
	private	Date		endTime;


	public	RegisterFeatureTypeHandler
				( OutputStream  out ,
				  XMLReader  parser ,
				  boolean  isTopLevelHandler ,
				  StrageManager  strage )
	{
		super( out , parser , isTopLevelHandler );

		this.strage = strage;
		this.errorOccured = false;
		this.errorMessage = null;
		this.startTime = null;
		this.endTime = null;
	}

	private	void	setError( String  errorMessage )
	{
		this.errorOccured = true;
		this.errorMessage = errorMessage;
	}

	public	void	xStartElement( String uri ,
				       String localName ,
				       String qName ,
				       Attributes  attrs ) throws SAXException
	{
		assert super.getCurrentLevel() == 1
			|| super.getCurrentLevel() == 2;

		XMLTag	t = new XMLTag( uri , localName );

		if ( super.getCurrentLevel() == 1 )
		{
			if ( ! t.equals
			       ( new XMLTag( URI.MISP ,
					     "RegisterFeatureType" ) ) )
			{
				SAXParseException
					e = new SAXParseException
						( "unknown top level tag \""
						  + t.getLocalName()
						  + "\"." ,
						  super.getLocator() );

				this.throwError( e );
			}

			try
			{
				this.startTime = this.strage.getCurrentTime();
			}
			catch( StrageException  e )
			{
				this.throwError( new SAXParseException
						 ( e.getMessage() ,
						   super.getLocator() ) );
			}

			return;
		}


		//
		// XXX: check errorOccured here,
		//      previous schema might cause error
		//
		if ( t.getLocalName().equals( "schema" ) )
		{
			if ( ! t.getURI().equals( URI.XML_SCHEMA ) )
			{
				this.throwError
				 ( new SAXParseException
				   ( "top level \"" + t.getLocalName() + "\" "
				     + "should be in namespace "
				     + URI.XML_SCHEMA ,
				     super.getLocator() ) );
			}


			//
			// class XMLSchemaRegistrationHandler is defined
			// at top of this file.
			//
			super.setContentHandlerDelegator
				( new XMLSchemaRegistrationHandler
				  ( super.getOutputStream() ,
				    super.getParser() ,
				    false ,
				    this.strage ,
				    this ) ,
				  uri , localName , qName , attrs );

			return;
		}
		else
		{
			SAXParseException
				e = new SAXParseException
					( "unknown second level tag \""
					  + t.getLocalName() + "\"." ,
					  super.getLocator() );

			this.throwError( e );
		}
	}

	public  void  xEndElement( String uri ,
				   String localName ,
				   String qName ) throws SAXException
	{
		try
		{
			this.endTime = this.strage.getCurrentTime();
		}
		catch( StrageException  e )
		{
			// XXX
			throw new SAXException( e );
		}


		this.printOutput();
	}

	private void printOutput()
	{
		PrintWriter	out = super.getPrintWriter();

		String	status;

		if ( this.errorOccured )
		{
			status = "FAILURE";
		}
		else
		{
			status = "SUCCESS";
		}


		out.println( "<RegisterFeatureTypeResponse"
			     + " xmlns=\"" + URI.MISP + "\">" );

		out.println( "  <ElapsedTime xmlns:gml=\"" + URI.GML + "\">" );

		if ( this.startTime == null )
		{
			out.println( "    "
				     + "<gml:beginPosition "
				     + " indeterminatePositon=\"unknown\"/>" );
		}
		else
		{
			out.println( "    <gml:beginPosition>"
				     + new ISO8601DateFormat()
						.format(this.startTime)
				     + "</gml:beginPosition>" );
		}

		if ( this.endTime == null )
		{
			out.println( "    "
				     + "<gml:endPosition"
				     + " indeterminatePositon=\"unknown\"/>" );
		}
		else
		{
			out.println( "    <gml:endPosition>"
				     + new ISO8601DateFormat()
				     .format(this.endTime)
				     + "</gml:endPosition>" );
		}

		out.println( "  </ElapsedTime>" );

		out.println( "  <Status>" + status + "</Status>" );

		if ( errorMessage != null )
		{
			out.println( "  <Error>" );
			out.println( "    <Message>" + errorMessage
				       + "</Message>" );
			out.println( "  </Error>" );
		}

		out.println( "</RegisterFeatureTypeResponse>" );
		out.flush();
	}

	private	void	throwError( SAXParseException  e ) throws SAXException
	{
		this.setError( e.getMessage() );

		this.printOutput();

		throw e;
	}
}
