package daruma.wfs;

import org.xml.sax.XMLReader;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.w3c.dom.Document;
import javax.xml.transform.TransformerException;
import javax.xml.parsers.ParserConfigurationException;

import java.io.OutputStream;
import java.io.PrintWriter;

import daruma.xml.URI;
import daruma.xml.handler.XSAXDefaultHandler;
import daruma.xml.util.XMLFormatConverter;
import daruma.strage_manager.type_definition.ElementName;

import daruma.strage_manager.StrageManager;


public class SOAPHandler extends XSAXDefaultHandler
{
	private	StrageManager	strage;
	private	boolean		soapEnvelopePrinted;


	public	SOAPHandler( OutputStream  out ,  XMLReader  parser ,
			     boolean  isTopLevelHandler ,
			     StrageManager  strage )
	{
		super( out , parser , isTopLevelHandler );

		this.strage = strage;

		this.soapEnvelopePrinted = false;
	}


	public	void	xStartElement( String uri ,
				       String localName ,
				       String qName ,
				       Attributes  attrs ) throws SAXException
	{
		assert super.getCurrentLevel() == 1
			|| super.getCurrentLevel() == 2
			|| super.getCurrentLevel() == 3;

		ElementName	t = new ElementName( uri , localName );

		if ( t.getNamespace().equals( URI.SOAP_ENV ) )
		{
			if ( super.getCurrentLevel() == 1 )
			{
				if ( ! t.equals
				     ( new ElementName( URI.SOAP_ENV ,
							"Envelope" ) ) )
				{
					SAXParseException
					e = new SAXParseException
						 ( "top level tag of SOAP"
						   + " shold be an"
						   + " \"Envelope\" tag " ,
						   super.getLocator() );

					this.printError( e );

					throw e;
				}

				return;
			}
			else if ( t.getLocalName().equals( "Header" ) )
			{
				// XXX: not implemented yet
			}
			else if ( t.getLocalName().equals( "Body" ) )
			{
				PrintWriter	out = super.getPrintWriter();

				out.println( "<SOAP-ENV:Envelope"
					     + " xmlns:SOAP-ENV=\""
					     + URI.SOAP_ENV + "\">" );
				out.println( "  <SOAP-ENV:Body>" );

				out.flush();

				this.soapEnvelopePrinted = true;
			}
		}
		else
		{
			// XXX: if ( this.soapEnvelopePrinted )
			// XXX: { add SOAP-ENV prefix }

			super.setContentHandlerDelegator
				( new WFSHandler
				  ( super.getOutputStream() ,
				    super.getParser() ,
				    (this.isTopLevelHandler()
				     && super.getCurrentLevel() == 1) ,
				    this.strage ) ,
				  uri , localName , qName , attrs );

			return;
		}
	}

	public void  xEndDocument() throws SAXException
	{
		if ( this.soapEnvelopePrinted )
		{
			PrintWriter	out = super.getPrintWriter();

			out.println( "  </SOAP-ENV:Body>" );
			out.println( "</SOAP-ENV:Envelope>" );

			out.flush();
		}
	}


	public	void	xFatalError( SAXParseException  exception )
						throws SAXException
	{
		this.printError( exception );
	}

	public	void	xError( SAXParseException  exception )
						throws SAXException
	{
		this.printError( exception );
	}

	public	void	xWarning( SAXParseException exception )
						throws SAXException
	{
		this.printError( exception );
	}


	protected  void	printError( SAXParseException  e )
						throws SAXException
	{
		Document	doc;

		try
		{
			XMLFormatConverter
				.print( new SOAPFaultDocumentBuilder(e)
							      .newDocument() ,
					this.getOutputStream() );
		}
		catch( ParserConfigurationException  pe )
		{
			throw new SAXException( pe );
		}
		catch( TransformerException  te )
		{
			throw new SAXException( te );
		}
	}
}
