package daruma.wfs;

import java.io.InputStream;
import java.io.OutputStream;
import java.io.IOException;
import java.io.PrintWriter;

import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.InputSource;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.DefaultHandler;
import javax.xml.parsers.SAXParserFactory;
import javax.xml.parsers.ParserConfigurationException;

import daruma.util.CloseIgnoreInputStream;
import daruma.util.FatalException;

import daruma.wfs.SOAPHandler;
import daruma.xml.handler.XSAXDefaultHandler;
import daruma.xml.util.EndOfDocumentException;
import daruma.strage_manager.StrageManager;


public class SOAPWFSProcessor
{
	private	String	debugTag = "";

	public	void	setDebugTag( String  debugTag )
	{
		this.debugTag = debugTag;
	}


	public	void	process( InputStream  in ,  OutputStream  out ,
				 StrageManager  strage )
					throws daruma.util.FatalException
	{
		// XXX
		System.err.println( "debugTag = [" + this.debugTag + "]" );


		//
		// create parser
		//
		SAXParserFactory	f = SAXParserFactory.newInstance();
		f.setNamespaceAware( true );

		XMLReader	parser;
		try
		{
			parser = f.newSAXParser().getXMLReader();

			XSAXDefaultHandler	h = new SOAPHandler
							( out , parser ,
							  true ,
							  strage );

			h.setThrowEODWithEndOfTopLevelElement( true );

			/*
			h.setDebugFlag( true );
			*/

			parser.setContentHandler( h );
			parser.setErrorHandler( h );
		}
		catch( ParserConfigurationException  e )
		{
			throw new FatalException
				  ( "Can't create SAX parser" , e );
		}
		catch( SAXException  e )
		{
			throw new FatalException
				  ( "Can't create SAX parser" , e );
		}


		//
		// parse
		//
		try
		{
			PrintWriter	w = new PrintWriter( out );
			System.err.println( "parse start" );

			try
			{
				parser.parse
				  ( new InputSource
				    ( new CloseIgnoreInputStream( in ) ) );
			}
			catch( EndOfDocumentException  e )
			{
				System.err.println( "end of document" );
			}

			System.err.println( "parse succeeded" );

			w.flush();
			out.flush();
		}
		catch( SAXParseException  e )
		{
			System.err.println( "parse failed" );

			notifyParseError( System.err , e.getMessage() );
		}
		catch( SAXException  e )
		{
			System.err.println( "parse failed" );

			notifyParseError( System.err , e.getMessage() );
		}
		catch( IOException  e )
		{
			// XXX
			e.printStackTrace();
		}
	}


	private	static	void	notifyParseError( OutputStream  out ,
						  String  errorMessage )
	{
		PrintWriter	w = new PrintWriter( out );

		// XXX
		w.println( "xml parse error: " + errorMessage );
		w.flush();

		try
		{
			out.flush();
		}
		catch( IOException  e )
		{
			// XXX
			e.printStackTrace();
		}
	}
}
