package daruma.wfs;

import daruma.wfs.SAXExceptionObserver;

import daruma.wfs.TransactionInsertHandler;
import daruma.wfs.TransactionDeleteHandler;
import daruma.wfs.TransactionUpdateHandler;

import daruma.util.ISO8601DateFormat;

import daruma.xml.XMLTag;
import daruma.xml.URI;
import daruma.xml.handler.XSAXDefaultHandler;
import daruma.xml.util.XMLFormatConverter;

import daruma.strage_manager.StrageManager;
import daruma.strage_manager.StrageException;


import org.xml.sax.XMLReader;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.w3c.dom.Document;
import javax.xml.transform.TransformerException;
import javax.xml.parsers.ParserConfigurationException;

import java.io.OutputStream;
import java.io.PrintWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Date;


public class TransactionHandler extends XSAXDefaultHandler
					implements SAXExceptionObserver
{
	private	StrageManager	strage;
	private	Date		startTime;
	private	Date		endTime;
	private	boolean		errorOccured;
	private	List<Exception>	errors;

	public	TransactionHandler( OutputStream  out ,  XMLReader  parser ,
				    boolean  isTopLevelHandler ,
				    StrageManager  strage )
	{
		super( out , parser , isTopLevelHandler );

		this.strage = strage;
		this.startTime = null;
		this.endTime = null;
		this.errorOccured = false;
		this.errors = new ArrayList<Exception>();
	}

	public	void	notifyError( SAXParseException  e ) throws SAXException
	{
		this.errorOccured = true;
		this.errors.add( e );

		this.finishAndResponce();
	}


	public	void	xStartElement( String uri ,
				       String localName ,
				       String qName ,
				       Attributes  attrs ) throws SAXException
	{
		assert super.getCurrentLevel() == 1
			|| super.getCurrentLevel() == 2;

		XMLTag	t = new XMLTag( uri , localName );

		if ( super.getCurrentLevel() == 1 )
		{
			if ( ! t.equals
			       ( new XMLTag( URI.MISP , "Transaction" ) ) )
			{
				SAXParseException
					e = new SAXParseException
						    ( "unexpected tag \""
						      + t.getLocalName()
						      + "\"." ,
						      super.getLocator() );

				this.printError( e , "error" );

				throw e;
			}

			try
			{
				this.startTime = this.strage.getCurrentTime();
			}
			catch( StrageException  e )
			{
				SAXParseException
					se = new SAXParseException
						 ( e.getMessage() ,
						   super.getLocator() ,
						   e );

				this.printError( se , "error" );

				throw se;
			}

			return;
		}


		//
		// Transaction/*
		//
		if ( t.getLocalName().equals( "Insert" ) )
		{
			if ( ! t.getURI().equals( URI.MISP ) )
			{
				SAXParseException	e;
				e = new SAXParseException
				    ( "top-level <" + t.getLocalName() + "> "
				      + "should be in namespace " + URI.MISP ,
				      this.getLocator() );

				this.printError( e , "error" );
				throw e;
			}

			super.setContentHandlerDelegator
				( new TransactionInsertHandler
				  ( super.getOutputStream() ,
				    super.getParser() ,
				    false ,
				    this.strage ,
				    this ) ,
				  uri , localName , qName , attrs );

			return;
		}
		else if ( t.getLocalName().equals( "Delete" ) )
		{
			if ( ! t.getURI().equals( URI.MISP ) )
			{
				SAXParseException	e;
				e = new SAXParseException
				    ( "top-level <" + t.getLocalName() + "> "
				      + "should be in namespace " + URI.MISP ,
				      this.getLocator() );

				this.printError( e , "error" );
				throw e;
			}

			super.setContentHandlerDelegator
				( new TransactionDeleteHandler
				  ( super.getOutputStream() ,
				    super.getParser() ,
				    false ,
				    this.strage ,
				    this ) ,
				  uri , localName , qName , attrs );

			return;
		}
		else if ( t.getLocalName().equals( "Update" ) )
		{
			if ( ! t.getURI().equals( URI.MISP ) )
			{
				SAXParseException	e;
				e = new SAXParseException
				    ( "top-level <" + t.getLocalName() + "> "
				      + "should be in namespace " + URI.MISP ,
				      this.getLocator() );

				this.printError( e , "error" );
				throw e;
			}

			super.setContentHandlerDelegator
				( new TransactionUpdateHandler
				  ( super.getOutputStream() ,
				    super.getParser() ,
				    false ,
				    this.strage ,
				    this ) ,
				  uri , localName , qName , attrs );
		}
		else
		{
			SAXParseException	e = new SAXParseException
						    ( "unknown top level tag <"
						      + t.getLocalName()
						      + ">." ,
						      super.getLocator() );

			this.printError( e , "error" );

			throw e;
		}
	}


	public	void  xEndDocument() throws SAXException
	{
		this.finishAndResponce();
	}


	public	void  finishAndResponce() throws SAXException
	{
		try
		{
			this.endTime = this.strage.getCurrentTime();
		}
		catch( StrageException  e )
		{
			throw new SAXException( e );
		}


		PrintWriter	out = super.getPrintWriter();

		String	status;

		if ( this.errorOccured )
		{
			status = "FAILURE";
		}
		else
		{
			status = "SUCCESS";
		}


		out.println( "<TransactionResponse"
			     + " xmlns=\"" + URI.MISP + "\">" );

		out.println( "  <ElapsedTime xmlns:gml=\"" + URI.GML + "\">" );

		out.println( "    <gml:beginPosition>"
				    + new ISO8601DateFormat()
						.format(this.startTime)
			       + "</gml:beginPosition>" );

		out.println( "    <gml:endPosition>"
				    + new ISO8601DateFormat()
						.format(this.endTime)
			       + "</gml:endPosition>" );

		out.println( "  </ElapsedTime>" );

		out.println( "  <TransactionResult>" );

		out.println( "    <Status>" + status + "</Status>" );

		if ( ! this.errors.isEmpty() )
		{
			out.println( "    <Error>" );

			for ( Exception  e  :  this.errors )
			{
				out.println( "      <Message>" + e.getMessage()
					     + "</Message>" );
			}

			out.println( "    </Error>" );
		}

		out.println( "  </TransactionResult>" );
		out.println( "</TransactionResponse>" );
		out.flush();

		try
		{
			super.getOutputStream().flush();
		}
		catch( IOException  e )
		{
			e.printStackTrace();
		}
	}

	public	void	xFatalError( SAXParseException  exception )
						throws SAXException
	{
		this.printError( exception , "fatal error" );
	}

	public	void	xError( SAXParseException  exception )
						throws SAXException
	{
		this.printError( exception , "error" );
	}

	public	void	xWarning( SAXParseException exception )
						throws SAXException
	{
		this.printError( exception , "warning" );
	}


	protected  void	printError( SAXParseException  e ,  String  type )
							throws SAXException
	{
		Document	doc;

		try
		{
			// XXX: should print TransactionResponse?

			XMLFormatConverter
				.print( new SOAPFaultDocumentBuilder(e)
							      .newDocument() ,
					this.getOutputStream() );
		}
		catch( ParserConfigurationException  pe )
		{
			throw new SAXException( pe );
		}
		catch( TransformerException  te )
		{
			throw new SAXException( te );
		}
	}
}
