package daruma.wfs.filter;

import daruma.wfs.filter.PredicateDescription;
import daruma.wfs.filter.FilterParser;

import daruma.wfs.SAXExceptionObserver;

import daruma.xml.handler.XSAXDOMCreateHandler;
import daruma.xml.util.XMLParseErrorException;

import daruma.strage_manager.StrageManager;
import daruma.strage_manager.type_definition.TypeDefinition;

import org.xml.sax.XMLReader;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

import org.w3c.dom.Element;

import java.io.OutputStream;


public class FilterHandler extends XSAXDOMCreateHandler
{
	private	TypeDefinition			type;
	private	StrageManager			strage;
	private	SAXExceptionObserver		errorReceiver;

	private	PredicateDescription		predicate;


	public	FilterHandler( OutputStream  out ,
			       XMLReader  parser ,
			       boolean  isTopLevelHandler ,
			       TypeDefinition  type ,
			       StrageManager  strage ,
			       SAXExceptionObserver  errorReceiver )
	{
		super( out , parser , isTopLevelHandler );

		this.type          = type;
		this.strage        = strage;
		this.errorReceiver = errorReceiver;
	}

	public	PredicateDescription	getPredicate()
	{
		return( this.predicate );
	}

	public	String	getSQLExpression()
	{
		if ( this.predicate == null )
		{
			return( null );
		}

		return( this.predicate.getSQLExpression() );
	}

	public	void  xEndDocument() throws SAXException
	{
		Element	filterElement = null;

		filterElement = super.getDOMDocument().getDocumentElement();

		try
		{
			this.predicate = FilterParser.parseFilter
						      ( filterElement ,
							this.type ,
							this.strage ,
							this.getPrefixMap() );
		}
		catch( XMLParseErrorException e )
		{
			SAXParseException	se = new SAXParseException
						     ( e.getMessage() ,
						       super.getLocator() );

			this.errorReceiver.notifyError( se );

			throw se;
		}
	}
}
