package daruma.wfs.filter;

import daruma.xml.QName;
import daruma.xml.UniversalName;
import daruma.xml.util.NodeUtil;
import daruma.xml.util.ElementUtil;
import daruma.xml.util.XMLParseErrorException;

import org.w3c.dom.Node;
import org.w3c.dom.Element;

import daruma.strage_manager.StrageManager;
import daruma.strage_manager.StrageException;


public class PropertyPathConverter
{
	private	PropertyPathConverter()
	{
	}

	public	static	String  convertPropertyElementToShortXPath
					( Element  element ,
					  StrageManager  strage )
						throws XMLParseErrorException
	{
		String	propertyNameString = ElementUtil.getChildNodesWholeText
						      ( element );

		return( PropertyPathConverter.convertPropertyPathToShortXPath
			( propertyNameString , element , strage ) );
	}

	public	static	String  convertPropertyPathToShortXPath
					( String  propertyPathString ,
					  Node  node ,
					  StrageManager  strage )
						throws XMLParseErrorException
	{
		String[]	propertyPathList
				= propertyPathString.trim().split( "/" );

		if ( propertyPathList.length == 0 )
		{
			throw new XMLParseErrorException
				( "empty property path "
				  + "\"" + propertyPathString + "\"" );
		}


		StringBuilder	buf = new StringBuilder( );

		for ( String  p : propertyPathList )
		{
			String	id = PropertyPathConverter
					.convertPropertyToID
						( p , node , strage );

			if ( buf.length() != 0 )
			{
				buf.append( "/" );
			}

			buf.append( id );
		}

		return( buf.toString() );
	}

	private	static	String  convertPropertyToID
					( String  propertyNameString ,
					  Node  node ,
					  StrageManager  strage )
						throws XMLParseErrorException
	{
		UniversalName	propertyName;

		try
		{
			propertyName = PropertyPathConverter
					.convertQNameStringToUniversalName
							( propertyNameString ,
							  node );
		}
		catch( XMLParseErrorException  e )
		{
			throw e;
		}

		String	propertyID;
		try
		{
			propertyID = strage.getUniversalNameID( propertyName );

			if ( propertyID == null )
			{
				throw new XMLParseErrorException
				  ( "property \"" + propertyNameString + "\""
				    + " not found" );
			}
		}
		catch( StrageException  e )
		{
			throw new XMLParseErrorException( e.getMessage() , e );
		}

		return( propertyID );
	}

	protected static UniversalName  convertQNameStringToUniversalName
							( String  qNameString ,
							  Node  node )
						throws XMLParseErrorException
	{
		QName	qName = new QName( qNameString );

		if ( ! qName.isValid() )
		{
			throw new XMLParseErrorException
				( "invalid QName [" + qNameString + "]" );
		}


		String	ns = NodeUtil.lookupNamespaceURI( node ,
							  qName.getPrefix() );

		if ( ns == null )
		{
			throw new XMLParseErrorException
				( "unbounded prefix "
				  + "[" + qName.getPrefixForPrint() + "]"
				  + " of QName [" + qNameString + "]" );
		}

		return( new UniversalName( ns , qName.getLocalName() ) );
	}
}
