package daruma.wfs.filter.predicates;

import daruma.wfs.filter.PredicateDescription;
import daruma.wfs.filter.PropertyPathConverter;

import daruma.xml.URI;
import daruma.xml.util.ElementUtil;
import daruma.xml.util.XMLParseErrorException;

import org.w3c.dom.Element;

import daruma.strage_manager.StrageManager;
import daruma.strage_manager.type_definition.TypeDefinition;
import daruma.strage_manager.type_definition.types.GeometryPropertyTypeDefinition;

import daruma.geometry.Geometry;
import daruma.geometry.GeometryException;
import daruma.geometry.GeometryFormatConverter;

import java.util.List;
import java.util.Map;
import java.util.HashMap;


public class GeometryPredicateDescription extends PredicateDescription
{
	private	String		sqlFunction;
	private	String		sqlExpression;

	private	static Map<String,String>	operatorMap;
	static
	{
		operatorMap = new HashMap<String, String>();

		operatorMap.put( "BBox"       , "MBRIntersects" );
		operatorMap.put( "Contains"   , "CONTAINS" );
		operatorMap.put( "Crosses"    , "CROSSES" );
		operatorMap.put( "Disjoint"   , "DISJOINT" );
		operatorMap.put( "Equals"     , "EQUALS" );
		operatorMap.put( "Intersects" , "INTERSECTS" );
		operatorMap.put( "Overlaps"   , "OVERLAPS" );
		operatorMap.put( "Touches"    , "TOUCHES" );
		operatorMap.put( "Within"     , "WITHIN" );
	}

	public	static	boolean	isAcceptablePredicate( String  localName )
	{
		return( GeometryPredicateDescription
			.operatorMap.containsKey( localName ) );
	}


	public	GeometryPredicateDescription
				( Element  element ,
				  TypeDefinition  type ,
				  StrageManager  strage )
					    throws XMLParseErrorException
	{
		super( strage , type );

		this.sqlFunction = GeometryPredicateDescription
				   .operatorMap.get( element.getLocalName() );

		if ( this.sqlFunction == null )
		{
			throw new XMLParseErrorException
			  ( "cannot handle filter \""
			    + element.getLocalName() + "\"" );
		}

		this.sqlExpression = parseGeometryPropertyComparation
								( element );
	}

	public	String	getSQLExpression()
	{
		return( this.sqlExpression );
	}


	private	String	parseGeometryPropertyComparation( Element  comp )
					    throws XMLParseErrorException
	{
		List<Element>	childElements
				  = ElementUtil.getChildElements( comp );

		if ( childElements.size() != 2 )
		{
			throw new XMLParseErrorException
			  ( "filter \"" + comp.getLocalName() + "\""
			    + " has invalid number of child elements,"
			    + " expected was 2" );
		}

		Element	propertyNameElement = childElements.get(0);
		Element	geometryElement     = childElements.get(1);

		if ( ! propertyNameElement.getLocalName()
		       .equals( "PropertyName" )
		  || ! propertyNameElement.getNamespaceURI()
		       .equals( URI.MISP ) )
		{
			throw new XMLParseErrorException
			       ( "unexpected \""
				 + propertyNameElement.getLocalName() + "\""
				 + " in namespace "
				 + propertyNameElement.getNamespaceURI() + ","
				 + " expected was \"PropertyName\""
				 + " in namespace "
				 + URI.MISP );
		}


		String	shortXPath = PropertyPathConverter
					.convertPropertyElementToShortXPath
						      ( propertyNameElement ,
							this.getStrage() );

		Geometry	geometry;
		try
		{
			geometry = GeometryFormatConverter
				     .parseFromGMLElement( geometryElement );
		}
		catch( GeometryException  e )
		{
			throw new XMLParseErrorException( e.getMessage() , e );
		}

		return( sqlFunction + "(`"
			+ GeometryPropertyTypeDefinition
			  .convertShortXPathToColumnName( shortXPath )
			+ "`,GeomFromText('"
			+ GeometryFormatConverter
				.convertGeometryToWKTString( geometry )
			+ "'))" );
	}
}
