package daruma.wfs.filter.predicates;

import daruma.wfs.filter.PredicateDescription;
import daruma.wfs.filter.predicates.IntegratedPredicateDescription;
import daruma.wfs.filter.predicates.PropertyComparationPredicateDescription;
import daruma.wfs.filter.predicates.PropertyIsBetweenPredicateDescripter;
import daruma.wfs.filter.predicates.ImmidiateValueBooleanDescription;
import daruma.wfs.filter.predicates.LogicalPredicateDescription;
import daruma.wfs.filter.predicates.GeometryPredicateDescription;

import daruma.xml.util.XMLParseErrorException;

import org.w3c.dom.Element;

import daruma.strage_manager.StrageManager;
import daruma.strage_manager.type_definition.TypeDefinition;


public class IntegratedPredicateDescription extends PredicateDescription
{
	public	static	boolean	isAcceptablePredicate( String  localName )
	{
		return( PropertyComparationPredicateDescription
				.isAcceptablePredicate( localName )
		     || PropertyIsBetweenPredicateDescripter
				.isAcceptablePredicate( localName )
		     || GeometryPredicateDescription
				.isAcceptablePredicate( localName )
		     || ImmidiateValueBooleanDescription
				.isAcceptablePredicate( localName )
		     || LogicalPredicateDescription
				.isAcceptablePredicate( localName ) );
	}

	private	String	sqlExpression;

	public	IntegratedPredicateDescription( Element  element ,
						TypeDefinition  type ,
						StrageManager  strage )
					    throws XMLParseErrorException
	{
		super( strage , type );

		this.sqlExpression = this.createSQLExpression( element );
	}

	public	String	getSQLExpression()
	{
		return( this.sqlExpression );
	}

	public	String	createSQLExpression( Element  element )
					    throws XMLParseErrorException
	{
		String	localName = element.getLocalName();

		if ( PropertyComparationPredicateDescription
		     .isAcceptablePredicate( localName ) )
		{
			return( new PropertyComparationPredicateDescription
				( element ,
				  super.getTypeDefinition() ,
				  super.getStrage() )
				.getSQLExpression() );
		}
		else if ( PropertyIsBetweenPredicateDescripter
			  .isAcceptablePredicate( localName ) )
		{
			return( new PropertyIsBetweenPredicateDescripter
				( element ,
				  super.getTypeDefinition() ,
				  super.getStrage() )
				.getSQLExpression() );
		}
		else if ( GeometryPredicateDescription
			  .isAcceptablePredicate( localName ) )
		{
			return( new GeometryPredicateDescription
				( element ,
				  super.getTypeDefinition() ,
				  super.getStrage() )
				.getSQLExpression() );
		}
		else if ( ImmidiateValueBooleanDescription
			  .isAcceptablePredicate( localName ) )
		{
			return( new ImmidiateValueBooleanDescription
				( element ,
				  super.getTypeDefinition() ,
				  super.getStrage() )
				.getSQLExpression() );
		}
		else if ( LogicalPredicateDescription
			  .isAcceptablePredicate( localName ) )
		{
			return( new LogicalPredicateDescription
				( element ,
				  super.getTypeDefinition() ,
				  super.getStrage() )
				.getSQLExpression() );
		}
		else
		{
			throw new XMLParseErrorException
			  ( "cannot handle filter \"" + localName + "\"" );
		}
	}
}
