package daruma.wfs.filter.predicates;

import daruma.wfs.filter.PredicateDescription;

import daruma.xml.util.ElementUtil;
import daruma.xml.util.XMLParseErrorException;

import daruma.strage_manager.StrageManager;
import daruma.strage_manager.type_definition.TypeDefinition;

import org.w3c.dom.Element;

import java.util.List;


public class LogicalPredicateDescription extends PredicateDescription
{
	public	static	boolean	isAcceptablePredicate( String  localName )
	{
		return( localName.equals( "And" )
		     || localName.equals( "Or" )
		     || localName.equals( "Not" ) );
	}

	private	String	sqlExpression;

	public	LogicalPredicateDescription( Element  element ,
					     TypeDefinition  type ,
					     StrageManager  strage )
					    throws XMLParseErrorException
	{
		super( strage , type );

		this.sqlExpression = this.createSQLExpression( element );
	}

	public	String	getSQLExpression()
	{
		return( this.sqlExpression );
	}

	public	String	createSQLExpression( Element  element )
					    throws XMLParseErrorException
	{
		String	operatorName = element.getLocalName();

		if ( operatorName.equals( "And" ) )
		{
			return( this.parseMultipleArgumentsOperation
				( element , "AND" ) );
		}
		else if ( operatorName.equals( "Or" ) )
		{
			return( this.parseMultipleArgumentsOperation
				( element , "OR" ) );
		}
		else if ( operatorName.equals( "Not" ) )
		{
			return( this.parseUnaryOperation( element , "NOT" ) );
		}
		else
		{
			throw new XMLParseErrorException
			  ( "cannot handle filter \"" + operatorName + "\"" );
		}

	}

	public	String	parseUnaryOperation( Element  element ,
					     String  operationString )
					    throws XMLParseErrorException
	{
		Element	subPredicate;

		try
		{
			subPredicate = ElementUtil
					.getSingleChildElement( element );
		}
		catch( XMLParseErrorException e )
		{
			throw new XMLParseErrorException( e.getMessage() , e );
		}

		return( operationString + " ("
			+ new IntegratedPredicateDescription
				( subPredicate ,
				  super.getTypeDefinition() ,
				  super.getStrage() )
			+ ")" );
	}

	public	String	parseMultipleArgumentsOperation
						( Element  element ,
						  String  operationString )
					    throws XMLParseErrorException
	{
		List<Element>	childElements = ElementUtil.getChildElements
								  ( element );

		if ( childElements.size() < 2 )
		{
			throw new XMLParseErrorException
				( "too few arguments of element \""
				  + element.getLocalName() + "\"" );
		}

		StringBuilder	buf = new StringBuilder( "(" );

		boolean	firstChild = true;

		for ( Element  c : childElements )
		{
			if ( ! firstChild )
			{
				buf.append( ") " + operationString + " (" );
			}

			try
			{
				buf.append
				    ( new IntegratedPredicateDescription
				      ( c ,
					super.getTypeDefinition() ,
					super.getStrage() )
				      .getSQLExpression() );
			}
			catch( XMLParseErrorException  e )
			{
				throw e;
			}

			firstChild = false;
		}

		buf.append( ")" );

		return( buf.toString() );
	}
}
