package daruma.wfs.filter.predicates;

import daruma.wfs.filter.PredicateDescription;
import daruma.wfs.filter.PropertyPathConverter;

import daruma.xml.URI;
import daruma.xml.util.ElementUtil;
import daruma.xml.util.XMLParseErrorException;

import org.w3c.dom.Element;

import daruma.strage_manager.StrageManager;
import daruma.strage_manager.type_definition.TypeDefinition;
import daruma.strage_manager.type_definition.TypeException;

import daruma.sql.SQLDataType;
import daruma.sql.TableColumnDefinition;
import daruma.sql.SQLTimeFormatConverter;

import java.util.Date;
import daruma.util.ISO8601DateFormat;
import java.text.ParsePosition;

import java.util.List;
import java.util.Map;
import java.util.HashMap;


public class PropertyComparationPredicateDescription
						extends PredicateDescription
{
	private	String		sqlOperator;
	private	String		sqlExpression;

	private	static Map<String,String>	operatorMap;
	static
	{
		operatorMap = new HashMap<String, String>();

		operatorMap.put( "PropertyIsEqualTo"              , "=" );
		operatorMap.put( "PropertyIsLessThan"             , "<" );
		operatorMap.put( "PropertyIsGreaterThan"          , ">" );
		operatorMap.put( "PropertyIsLessThanOrEqualTo"    , "<=" );
		operatorMap.put( "PropertyIsGreaterThanOrEqualTo" , ">=" );
	}

	public	static	boolean	isAcceptablePredicate( String  localName )
	{
		return( PropertyComparationPredicateDescription
			.operatorMap.containsKey( localName ) );
	}


	public	PropertyComparationPredicateDescription
				( Element  comparePredicateElement ,
				  TypeDefinition  type ,
				  StrageManager  strage )
					    throws XMLParseErrorException
	{
		super( strage , type );

		this.sqlOperator = PropertyComparationPredicateDescription
				   .operatorMap.get
				    ( comparePredicateElement.getLocalName() );

		if ( this.sqlOperator == null )
		{
			throw new XMLParseErrorException
			  ( "cannot handle filter \""
			    + comparePredicateElement.getLocalName() + "\"" );
		}

		this.sqlExpression = parsePropertyComparation
						( comparePredicateElement );
	}

	public	String	getSQLExpression()
	{
		return( this.sqlExpression );
	}

	private	String	parsePropertyComparation( Element  comp )
					    throws XMLParseErrorException
	{
		List<Element>	childElements
				  = ElementUtil.getChildElements( comp );

		if ( childElements.size() != 2 )
		{
			throw new XMLParseErrorException
			  ( "filter \"" + comp.getLocalName() + "\""
			    + " has invalid number of child elements,"
			    + " expected was 2" );
		}

		Element	propertyNameElement = childElements.get(0);
		Element	literalElement      = childElements.get(1);

		if ( ! propertyNameElement.getLocalName()
		       .equals( "PropertyName" )
		  || ! propertyNameElement.getNamespaceURI()
		       .equals( URI.MISP ) )
		{
			throw new XMLParseErrorException
			       ( "unexpected \""
				 + propertyNameElement.getLocalName() + "\""
				 + " in namespace "
				 + propertyNameElement.getNamespaceURI() + ","
				 + " expected was \"PropertyName\""
				 + " in namespace "
				 + URI.MISP );
		}

		if ( ! literalElement.getLocalName()
		       .equals( "Literal" )
		  || ! literalElement.getNamespaceURI()
		       .equals( URI.MISP ) )
		{
			throw new XMLParseErrorException
			       ( "unexpected \""
				 + literalElement.getLocalName() + "\""
				 + " in namespace "
				 + literalElement.getNamespaceURI() + ","
				 + " expected was \"Literal\""
				 + " in namespace "
				 + URI.MISP );
		}


		String	shortXPath = PropertyPathConverter
					.convertPropertyElementToShortXPath
						( propertyNameElement ,
						  super.getStrage() );

		String	literalString = ElementUtil.getChildNodesWholeText
							  ( literalElement );

		String	sqlValueString = this.convertValueToSQL
						( shortXPath ,
						  literalString );

		return( shortXPath + this.sqlOperator
			+ "\"" + sqlValueString + "\"" );
	}


	private	String	convertValueToSQL( String  columnName ,
					   String  value )
					    throws XMLParseErrorException
	{
		SQLDataType	sqlType = super.getTypeDefinition()
						  .getSingleSQLDataType();

		if ( sqlType == null )
		{
			List<TableColumnDefinition>	columns;

			try
			{
				columns = super.getTypeDefinition()
					  .getCompositeSQLDataType
					   ( super.getStrage() , null );
			}
			catch( TypeException  e )
			{
				throw new XMLParseErrorException
					( "type definition error" , e );
			}

			for ( TableColumnDefinition  c  : columns )
			{
				if ( c.getColumnName().equals( columnName ) )
				{
					sqlType = c.getSQLDataType();
					break;
				}
			}
		}

		if ( sqlType == null )
		{
			throw new XMLParseErrorException
				( "property not found" );
		}

		if ( sqlType.isTimeFamilyType() )
		{
			ISO8601DateFormat	dateParser
						  = new ISO8601DateFormat();

			Date	date = dateParser.parse
					( value , new ParsePosition( 0 ) );

			String	dateString = SQLTimeFormatConverter
						.convertDateToString( date );

			return( dateString );
		}
		else
		{
			return( value );
		}
	}
}
