package daruma.wfs.filter.predicates;

import daruma.wfs.filter.PredicateDescription;
import daruma.wfs.filter.PropertyPathConverter;


import daruma.xml.URI;
import daruma.xml.util.ElementUtil;
import daruma.xml.util.XMLParseErrorException;

import org.w3c.dom.Element;

import daruma.strage_manager.StrageManager;
import daruma.strage_manager.type_definition.TypeDefinition;

import java.util.List;


public class PropertyIsBetweenPredicateDescripter
						extends PredicateDescription
{
	private	static	final	String	PROPERTY_IS_BETWEEN
						       = "PropertyIsBetween";
	private	static	final	String	PROPERTY_NAME  = "PropertyName";
	private	static	final	String	LOWER_BOUNDARY = "LowerBoundary";
	private	static	final	String	UPPER_BOUNDARY = "UpperBoundary";

	public	static	boolean	isAcceptablePredicate( String  localName )
	{
		return( localName.equals( PropertyIsBetweenPredicateDescripter
					  .PROPERTY_IS_BETWEEN ) );
	}

	private	String	sqlExpression;

	public	String	getSQLExpression()
	{
		return( this.sqlExpression );
	}

	public	PropertyIsBetweenPredicateDescripter( Element  element ,
						      TypeDefinition  type ,
						      StrageManager  strage )
						throws XMLParseErrorException
	{
		super( strage , type );

		List<Element>	childElements = ElementUtil
						  .getChildElements( element );

		if ( childElements.size() != 3 )
		{
			throw new XMLParseErrorException
			  ( "filter \"" + element.getLocalName() + "\""
			    + " has invalid number of child elements,"
			    + " expected was 3" );
		}

		Element	propertyNameElement = childElements.get(0);
		Element	lowerBoundary       = childElements.get(1);
		Element	upperBoundary       = childElements.get(2);


		if ( ! propertyNameElement.getLocalName()
		       .equals( PropertyIsBetweenPredicateDescripter
				.PROPERTY_NAME )
		  || ! propertyNameElement.getNamespaceURI()
		       .equals( URI.MISP ) )
		{
			throw new XMLParseErrorException
			       ( "unexpected \""
				 + propertyNameElement.getLocalName() + "\""
				 + " in namespace "
				 + propertyNameElement.getNamespaceURI() + ","
				 + " expected was \"PropertyName\""
				 + " in namespace "
				 + URI.MISP );
		}

		if ( ! lowerBoundary.getLocalName()
		     .equals( PropertyIsBetweenPredicateDescripter
			      .LOWER_BOUNDARY )
		  || ! lowerBoundary.getNamespaceURI().equals( URI.MISP ) )
		{
			throw new XMLParseErrorException
			       ( "unexpected \""
				 + lowerBoundary.getLocalName() + "\""
				 + " in namespace "
				 + lowerBoundary.getNamespaceURI() + ","
				 + " expected was \""
				 + PropertyIsBetweenPredicateDescripter
					 .LOWER_BOUNDARY
				 + "\" in namespace "
				 + URI.MISP );
		}

		if ( ! upperBoundary.getLocalName()
			.equals( PropertyIsBetweenPredicateDescripter
				 .UPPER_BOUNDARY )
		  || ! upperBoundary.getNamespaceURI().equals( URI.MISP ) )
		{
			throw new XMLParseErrorException
			       ( "unexpected \""
				 + upperBoundary.getLocalName() + "\""
				 + " in namespace "
				 + upperBoundary.getNamespaceURI() + ","
				 + " expected was \""
				 + PropertyIsBetweenPredicateDescripter
					 .UPPER_BOUNDARY
				 + "\" in namespace "
				 + URI.MISP );
		}


		String	lowerBoundaryString = ElementUtil
						.getChildNodesWholeText
							    ( lowerBoundary );
		String	upperBoundaryString = ElementUtil
						.getChildNodesWholeText
							    ( upperBoundary );

		String	propertyXPath = PropertyPathConverter
					  .convertPropertyElementToShortXPath
						( propertyNameElement ,
						  super.getStrage() );

		this.sqlExpression
		     = "("
			+ propertyXPath + ">=\"" + lowerBoundaryString + "\""
			+ " AND "
			+ propertyXPath + "<=\"" + upperBoundaryString + "\""
			+ ")";
	}
}
