package daruma.wfs.insert;

import daruma.strage_manager.StrageManager;
import daruma.strage_manager.StrageException;
import daruma.strage_manager.type_definition.ElementName;
import daruma.strage_manager.type_definition.TypeName;
import daruma.strage_manager.type_definition.TypeDefinition;
import daruma.strage_manager.type_definition.TypedInstance;
import daruma.strage_manager.type_definition.TypeException;

import daruma.wfs.TransactionHandler;

import daruma.xml.SimpleXPath;
import daruma.xml.handler.XSAXDOMCreateHandler;
import daruma.xml.util.ElementUtil;
import daruma.util.Pair;

import org.xml.sax.XMLReader;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.w3c.dom.Element;

import java.io.OutputStream;

// XXX: only for debugging
import daruma.xml.util.XMLFormatConverter;
import javax.xml.transform.TransformerException;


public class InsertAnElementHandler extends XSAXDOMCreateHandler
{
	private	StrageManager		strage;
	private	TransactionHandler	errorReceiver;

	public	InsertAnElementHandler
			( OutputStream  out ,
			  XMLReader  parser ,
			  boolean  isTopLevelHandler ,
			  StrageManager  strage ,
			  TransactionHandler  errorReceiver )
	{
		super( out , parser , isTopLevelHandler );
		super.setInhibitEndPrefixBeforeEndDocument( true );

		this.strage        = strage;
		this.errorReceiver = errorReceiver;
	}

	public  void  xEndDocument() throws SAXException
	{
		Element	element = null;

		try
		{
			element = super.getDOMDocument().getDocumentElement();

			System.out.println();
			System.out.println();
			System.out.println( "data = [" );

			XMLFormatConverter.print( element , System.out );

			System.out.println( "]" );
		}
		catch( TransformerException e )
		{
			this.throwError( new SAXParseException
					 ( e.getMessage() ,
					   super.getLocator() ,
					   e ) );
		}


		ElementName	elementName = new ElementName
						  ( element.getNamespaceURI() ,
						    element.getLocalName() );

		TypeName	typeName = null;

		try
		{
			typeName = strage.getElementTypeName( elementName );
		}
		catch( StrageException  e )
		{
			this.throwError( new SAXParseException
					 ( e.getMessage() ,
					   super.getLocator() ,
					   e ) );
		}

		System.err.println( "typeName = [" + typeName + "]" );

		if ( typeName == null )
		{
			this.throwError( new SAXParseException
					 ( "type of element "
					   + elementName.getLocalName()
					   + " in namespace "
					   + elementName.getNamespace()
					   + " not found" ,
					   super.getLocator() ) );
		}

		TypeDefinition	type = null;

		try
		{
			type = strage.getTypeDefinition( typeName );
		}
		catch( StrageException  e )
		{
			this.throwError( new SAXParseException
					     ( e.getMessage() ,
					       super.getLocator() ,
					       e ) );
		}

		if ( type == null )
		{
			this.throwError( new SAXParseException
					 ( "type definition of "
					   + elementName.getLocalName()
					   + " in namespace "
					   + elementName.getNamespace()
					   + " not found" ,
					   super.getLocator() ) );
		}


		TypedInstance	obj = null;

		try
		{
			Pair<TypedInstance, Integer>
				r = type.createInstance( element ,
							 elementName ,
							 (SimpleXPath)null ,
							 this.strage ,
							 0 );
			obj = r.getFirst();

			if ( ElementUtil.hasExtraChildElements
			     ( element , r.getSecond() ) )
			{
				throw new TypeException
					  ( "too many child elements of "
					    + new ElementName( element )
								.toString() );

			}
		}
		catch( TypeException  e )
		{
			this.throwError( new SAXParseException
					 ( "invalid element " + elementName
					   + ": " + e.getMessage() ,
					   super.getLocator() ) );
		}


		// XXX
		if ( obj == null )
		{
			this.throwError( new SAXParseException
					 ( "invalid element "
					   + elementName.getLocalName()
					   + " in namespace "
					   + elementName.getNamespace() ,
					   super.getLocator() ) );
		}


		try
		{
			this.strage.insertElement( elementName , obj );
		}
		catch( StrageException  e )
		{
			this.throwError( new SAXParseException
					     ( e.getMessage() ,
					       super.getLocator() ,
					       e ) );
		}
	}


	private	void	throwError( SAXParseException  e ) throws SAXException
	{
		this.errorReceiver.notifyError( e );

		throw e;
	}
}
