package daruma.xml.util;

import daruma.xml.util.XMLParseErrorException;
import daruma.xml.util.PrefixMap;
import daruma.xml.UniversalName;
import daruma.xml.URI;

import org.w3c.dom.Element;
import org.w3c.dom.Text;
import org.w3c.dom.Attr;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.NamedNodeMap;


import java.util.List;
import java.util.ArrayList;
import java.util.Map;

import java.io.OutputStream;
import java.io.PrintWriter;
import java.io.IOException;


public class ElementUtil
{
	private	ElementUtil()
	{
	}

	public	static	boolean	hasExtraChildElements( Element  element ,
						       int  beginChildIndex )
	{
		NodeList	childNodes = element.getChildNodes();

		for ( int  i = beginChildIndex  ;
		      i < childNodes.getLength()  ;  ++ i )
		{
			Node	n = childNodes.item( i );

			if ( n instanceof Element )
			{
				return( true );
			}
		}

		return( false );
	}


	public	static	String	getChildNodesWholeText( Element  element )
	{
		/*
		NodeList	childNodes = element.getChildNodes();

		StringBuilder	buf = new StringBuilder();

		for ( int  i = 0  ;  i < childNodes.getLength()  ;  ++ i )
		{
			Node	n = childNodes.item(i);

			if ( n instanceof Text )
			{
				buf.append( ((Text)n).getWholeText() );
			}
		}

		return( buf.toString() );
		*/

		return( element.getTextContent() );
	}


	public	static	List<Element>	getChildElements( Element  element )
	{
		List<Element>	ret = new ArrayList<Element>();

		NodeList	childNodes = element.getChildNodes();

		for ( int  i = 0  ;  i < childNodes.getLength()  ;  ++ i )
		{
			Node	n = childNodes.item( i );

			if ( n instanceof Element )
			{
				ret.add( (Element)n );
			}
		}

		return( ret );
	}

	public	static	List<Element>	getChildElements( Element  element ,
							  UniversalName  name )
	{
		List<Element>	ret = new ArrayList<Element>();

		NodeList	childNodes = element.getChildNodes();

		for ( int  i = 0  ;  i < childNodes.getLength()  ;  ++ i )
		{
			Node	n = childNodes.item( i );

			if ( n instanceof Element
			     && (((Element)n).getNamespaceURI()
				 .equals(name.getNamespace())
				 && ((Element)n).getLocalName()
				    .equals(name.getLocalName())) )
			{
				ret.add( (Element)n );
			}
		}

		return( ret );
	}

	public	static	Element	getSingleChildElement( Element  element )
						  throws XMLParseErrorException
	{
		NodeList	childNodes = element.getChildNodes();

		Element		ret = null;
		for ( int  i = 0  ;  i < childNodes.getLength()  ;  ++ i )
		{
			Node	n = childNodes.item( i );

			if ( n instanceof Element )
			{
				if ( ret != null )
				{
					throw new XMLParseErrorException
						( "too many childs of element "
						  + element.getLocalName()
						  + " in namespace "
						  + element.getNamespaceURI()
						  + " found" );
				}

				ret = (Element)n;
			}

		}

		if ( ret == null )
		{
			throw new XMLParseErrorException
				( "empty child element of element \""
				  + element.getLocalName()
				  + "\" in namespace \""
				  + element.getNamespaceURI()
				  + "\"" );
		}

		return( ret );
	}

	public	static	void	debugPrint( Element  element ,
					    OutputStream  out )
	{
		NodeList	childNodes = element.getChildNodes();

		PrintWriter	w = new PrintWriter( out );

		for ( int  i = 0  ;  i < childNodes.getLength()  ;  ++ i )
		{
			Node	n = childNodes.item( i );

			w.print( i + ": " );

			if ( n instanceof Element )
			{
				Element	e = (Element)n;

				w.println( "Element {" + e.getNamespaceURI()
					   + "}" + e.getLocalName() );
			}
			else if ( n instanceof Text )
			{
				w.println( "Text [" + ((Text)n).getWholeText()
					   + "]" );
			}
			else
			{
				w.println( "???" );
			}
		}

		w.flush();

		try
		{
			out.flush();
		}
		catch( IOException  e )
		{
			e.printStackTrace();
		}
	}

	public	static	void	addPrefixMappingToElement
						( Element  element ,
						  PrefixMap  prefixMap )
	{
		for( Map.Entry<String, String>  e : prefixMap.entrySet() )
		{
			String	qName;

			if ( e.getKey().length() == 0 )
			{
				qName = "xmlns";
			}
			else
			{
				qName = "xmlns:" + e.getKey();
			}

			element.setAttributeNS( URI.XMLNS ,
						qName ,
						e.getValue() );
		}
	}

	public	static	void	addPrefixMappingToPrefixMap
						( PrefixMap  prefixMap ,
						  Node  node )
	{
		NamedNodeMap	attributes = node.getAttributes();

		if ( attributes == null )
		{
			return;
		}

		for ( int  i = 0  ;  i < attributes.getLength()  ;  i ++ )
		{
			Attr	attr = (Attr)attributes.item(i);
			String	pref = attr.getPrefix();

			if ( pref != null
			  && pref.equals( "xmlns" ) )
			{
				prefixMap.put( attr.getLocalName() ,
					       attr.getValue() );
			}
			else if ( pref == null
			       && attr.getLocalName().equals( "xmlns" ) )
			{
				prefixMap.put( "" , attr.getValue() );
			}
		}


		String	namespaceURI = node.getNamespaceURI();

		if ( namespaceURI != null )
		{
			prefixMap.put( node.getPrefix() , namespaceURI );
		}
	}
}
