package daruma.xml.util;

import daruma.xml.util.DOMDocumentFactory;

import org.w3c.dom.Node;
import org.w3c.dom.Element;
import org.w3c.dom.Document;

import java.io.OutputStream;
import java.io.InputStream;
import java.io.Reader;
import java.io.Writer;
import java.io.PrintWriter;
import java.io.IOException;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;

import javax.xml.transform.TransformerFactory;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.dom.DOMResult;
import javax.xml.transform.stream.StreamSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.parsers.ParserConfigurationException;

import javax.xml.parsers.SAXParserFactory;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.XMLReader;
import org.xml.sax.InputSource;
import daruma.xml.handler.XSAXDOMCreateHandler;



public class XMLFormatConverter
{
	private	XMLFormatConverter()
	{
	}

	public	static	void	print( Node  node ,  OutputStream  out )
						throws TransformerException
	{
		TransformerFactory  factory = TransformerFactory.newInstance();
		Transformer	transformer;
		try
		{
			transformer = factory.newTransformer();
		}
		catch( TransformerConfigurationException  te )
		{
			throw te;
		}

		transformer.setOutputProperty( OutputKeys.INDENT , "yes" );

		DOMSource	source = new DOMSource( node );
		StreamResult	result = new StreamResult
						( new PrintWriter( out ) );

		try
		{
			transformer.transform( source , result );

			try
			{
				out.flush();
			}
			catch( IOException  e )
			{
			}
		}
		catch( TransformerException  te )
		{
			throw te;
		}
	}

	public	static	void	printElement( Node  element ,
					      OutputStream  out )
						throws TransformerException
	{
		TransformerFactory  factory = TransformerFactory.newInstance();
		Transformer	transformer;
		try
		{
			transformer = factory.newTransformer();
		}
		catch( TransformerConfigurationException  te )
		{
			throw te;
		}

		transformer.setOutputProperty
			( OutputKeys.INDENT , "yes" );
		transformer.setOutputProperty
			( OutputKeys.OMIT_XML_DECLARATION , "yes" );

		DOMSource	source = new DOMSource( element );
		StreamResult	result = new StreamResult
						( new PrintWriter( out ) );

		try
		{
			transformer.transform( source , result );

			try
			{
				out.flush();
			}
			catch( IOException  e )
			{
			}
		}
		catch( TransformerException  te )
		{
			throw te;
		}
	}


	public	static	void	printTextWithoutXMLDeclaration
				( Reader  in , Writer  out )
						throws TransformerException
	{
		TransformerFactory  factory = TransformerFactory.newInstance();
		Transformer	transformer;
		try
		{
			transformer = factory.newTransformer();
		}
		catch( TransformerConfigurationException  te )
		{
			throw te;
		}

		transformer.setOutputProperty
			( OutputKeys.INDENT , "yes" );
		transformer.setOutputProperty
			( OutputKeys.OMIT_XML_DECLARATION , "yes" );

		StreamSource	source = new StreamSource( in );
		StreamResult	result = new StreamResult( out );

		try
		{
			transformer.transform( source , result );

			try
			{
				out.flush();
			}
			catch( IOException  e )
			{
			}
		}
		catch( TransformerException  te )
		{
			throw te;
		}
	}

	public	static	void	readNode( InputStream  in ,
					  Node  node )
						throws TransformerException
	{
		TransformerFactory  factory = TransformerFactory.newInstance();
		Transformer	transformer;
		try
		{
			transformer = factory.newTransformer();
		}
		catch( TransformerConfigurationException  te )
		{
			throw te;
		}

		transformer.setOutputProperty
			( OutputKeys.INDENT , "yes" );
		transformer.setOutputProperty
			( OutputKeys.OMIT_XML_DECLARATION , "yes" );

		StreamSource	source = new StreamSource( in );
		DOMResult	result = new DOMResult( node );

		try
		{
			transformer.transform( source , result );
		}
		catch( TransformerException  te )
		{
			throw te;
		}
	}

	public	static	String	elementToString( Element  element )
						throws TransformerException
	{
		ByteArrayOutputStream	buf = new ByteArrayOutputStream();

		XMLFormatConverter.printElement( element , buf );

		return( buf.toString() );
	}



	/*
	public	static	Document	stringToDOMDocument( String  str )
						throws TransformerException
	{
		TransformerFactory  factory = TransformerFactory.newInstance();
		Transformer	transformer;
		try
		{
			transformer = factory.newTransformer();
		}
		catch( TransformerConfigurationException  te )
		{
			throw te;
		}

		transformer.setOutputProperty
			( OutputKeys.INDENT , "yes" );
		transformer.setOutputProperty
			( OutputKeys.OMIT_XML_DECLARATION , "yes" );

		Document	doc;

		try
		{
			doc = DOMDocumentFactory.create();
		}
		catch( ParserConfigurationException  e )
		{
			throw new TransformerConfigurationException(e);
		}

		StreamSource	source = new StreamSource
					 ( new ByteArrayInputStream
					   ( str.getBytes() ) );
		DOMResult	result = new DOMResult( doc );

		try
		{
			transformer.transform( source , result );
		}
		catch( TransformerException  te )
		{
			throw te;
		}

		return( (Document)(result.getNode()) );
	}
	*/


	public	static	Document	stringToDOMDocument( String  str )
						throws TransformerException
	{
		return( XMLFormatConverter
			.stringToDOMDocument( str , (Document)null ) );
	}

	public	static	Document	stringToDOMDocument( String  str ,
							     Document  doc )
						throws TransformerException
	{
		return( XMLFormatConverter.streamToDOMDocument
			( new ByteArrayInputStream( str.getBytes() ) ,
			  doc ) );
	}

	public	static	Document	streamToDOMDocument( InputStream  in )
						throws TransformerException
	{
		Document	doc;

		try
		{
			doc = DOMDocumentFactory.create();
		}
		catch( ParserConfigurationException  e )
		{
			throw new TransformerException( e );
		}

		return( XMLFormatConverter.streamToDOMDocument( in , doc ) );
	}

	public	static	Document	streamToDOMDocument( InputStream  in ,
							     Document  doc )
						throws TransformerException
	{
		//
		// create parser
		//
		SAXParserFactory	f = SAXParserFactory.newInstance();
		f.setNamespaceAware( true );

		XMLReader		parser;
		XSAXDOMCreateHandler	h;

		try
		{
			parser = f.newSAXParser().getXMLReader();

			// XXX: 3rd argument should be true
			h = new XSAXDOMCreateHandler
				( new ByteArrayOutputStream() ,
				  parser , false , doc );

			parser.setContentHandler( h );
			parser.setErrorHandler( h );
		}
		catch( ParserConfigurationException  e )
		{
			throw new TransformerException
				  ( "Can't create SAX parser" , e );
		}
		catch( SAXException  e )
		{
			throw new TransformerException
				  ( "Can't create SAX parser" , e );
		}


		//
		// parse
		//
		try
		{
			PrintWriter	w = new PrintWriter( System.err );

			try
			{
				parser.parse( new InputSource( in ) );
			}
			catch( EndOfDocumentException  e )
			{
				// OK, expected
			}

			w.flush();
		}
		catch( SAXParseException  e )
		{
			throw new TransformerException( e );
		}
		catch( SAXException  e )
		{
			throw new TransformerException( e );
		}
		catch( IOException  e )
		{
			e.printStackTrace();

			throw new TransformerException( e );
		}

		return( h.getDOMDocument() );
	}

	public	static	Element	stringToElement( String  str ,  Document  doc )
						  throws TransformerException
	{
		Document	d = stringToDOMDocument( str , doc ) ;

		if ( d == null )
		{
			return( null );
		}

		return d.getDocumentElement();
	}
}
